//
//  JDStaticTools.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/5.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDStaticTools.h"
#import <LBXScanViewStyle.h>
#import "JDCloudReachability.h"

static NSString * const JDCloudUtilImageBundle     = @"JDCloudImageSource";

@implementation JDStaticTools

/*
 *设置navigationBar的左按钮样式
 */
+ (UIButton*)setNavigationLeftItem:(UIViewController *)viewController
                             title:(NSString *)title
                      withFontSize:(CGFloat)fontSize
                         textColor:(UIColor*)textColor
                   normalImageName:(NSString *)normalImageName
             hightLightedImageName:(NSString *)hightLightedImageName
{
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setFrame:CGRectMake(0, 0, 44, 44)];
    if (title) {
        leftBtn.titleLabel.font = [UIFont systemFontOfSize:fontSize];
        [leftBtn setTitleColor:textColor forState:UIControlStateNormal];
        [leftBtn setTitle:title forState:UIControlStateNormal];
    }
    //[leftBtn sizeToFit];
    [leftBtn setImage:[UIImage imageNamed:normalImageName] forState:UIControlStateNormal];
    [leftBtn setImage:[UIImage imageNamed:hightLightedImageName] forState:UIControlStateHighlighted];
    [leftBtn setImageEdgeInsets:UIEdgeInsetsMake(0, -18, 0, 18)];
    viewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:leftBtn];
    
    return leftBtn;
}

/*
 *设置navigationBar的右按钮样式
 */
+ (UIButton*)setNavigationRightItem:(UIViewController *)viewController
                              title:(NSString *)title
                       withFontSize:(CGFloat)fontSize
                          textColor:(UIColor*)textColor
                    normalImageName:(NSString *)normalImageName
              hightLightedImageName:(NSString *)hightLightedImageName
{
    UIButton* rightBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [rightBtn setFrame:CGRectMake(0, 0, 60, 40)];
   // rightBtn.contentMode = UIViewContentModeCenter;
    [rightBtn setContentEdgeInsets:UIEdgeInsetsMake(0, 20, 0, -20)];
    
    if (title)
    {
        rightBtn.titleLabel.font = [UIFont systemFontOfSize:fontSize];
        [rightBtn setTitleColor:textColor forState:UIControlStateNormal];
        [rightBtn setTitle:title forState:UIControlStateNormal];
    }
    [rightBtn sizeToFit];
    [rightBtn setBackgroundImage:[UIImage imageNamed:normalImageName] forState:UIControlStateNormal];
    [rightBtn setBackgroundImage:[UIImage imageNamed:hightLightedImageName] forState:UIControlStateHighlighted];
    
    viewController.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:rightBtn];
    
    return rightBtn;
}

/*
 @abstract 16进制颜色(html颜色值)字符串转为UIColor
 */
+ (UIColor *) colorWithHexString:(NSString *)stringToConvert{
    
    NSString *cString = [[stringToConvert stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] uppercaseString];
    
    // String should be 6 or 8 characters
    if ([cString length] < 6) return [UIColor blackColor];
    
    // strip 0X if it appears
    if ([cString hasPrefix:@"0X"]) cString = [cString substringFromIndex:2];
    if ([cString hasPrefix:@"#"]) cString = [cString substringFromIndex:1];
    if ([cString length] != 6) return [UIColor blackColor];
    // Separate into r, g, b substrings
    NSRange range;
    range.location = 0;
    range.length = 2;
    NSString *rString = [cString substringWithRange:range];
    
    range.location = 2;
    NSString *gString = [cString substringWithRange:range];
    
    range.location = 4;
    NSString *bString = [cString substringWithRange:range];
    
    // Scan values
    unsigned int r, g, b;
    [[NSScanner scannerWithString:rString] scanHexInt:&r];
    [[NSScanner scannerWithString:gString] scanHexInt:&g];
    [[NSScanner scannerWithString:bString] scanHexInt:&b];
    return [UIColor colorWithRed:((float) r / 255.0f)
                           green:((float) g / 255.0f)
                            blue:((float) b / 255.0f)
                           alpha:1.0f];
}

+ (UIColor *)colorWithHexString:(NSString *)hexStr alpha:(CGFloat)alpha{
    
    NSString *cString = [[hexStr stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] uppercaseString];
    
    // String should be 6 or 8 characters
    if ([cString length] < 6) return [UIColor blackColor];
    
    // strip 0X if it appears
    if ([cString hasPrefix:@"0X"]) cString = [cString substringFromIndex:2];
    if ([cString hasPrefix:@"#"]) cString = [cString substringFromIndex:1];
    if ([cString length] != 6) return [UIColor blackColor];
    // Separate into r, g, b substrings
    NSRange range;
    range.location = 0;
    range.length = 2;
    NSString *rString = [cString substringWithRange:range];
    
    range.location = 2;
    NSString *gString = [cString substringWithRange:range];
    
    range.location = 4;
    NSString *bString = [cString substringWithRange:range];
    
    // Scan values
    unsigned int r, g, b, a = 0;
    [[NSScanner scannerWithString:rString] scanHexInt:&r];
    [[NSScanner scannerWithString:gString] scanHexInt:&g];
    [[NSScanner scannerWithString:bString] scanHexInt:&b];
    return [UIColor colorWithRed:((float) r / 255.0f)
                           green:((float) g / 255.0f)
                            blue:((float) b / 255.0f)
                           alpha:a];
}

/**
 @abstract 根据颜色返回图片
 */
+ (UIImage*) imageWithColor:(UIColor*)color
{
    CGRect rect = CGRectMake(0.0f, 0.0f, 1.0f, 1.0f);
    UIGraphicsBeginImageContext(rect.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGContextSetFillColorWithColor(context, [color CGColor]);
    CGContextFillRect(context, rect);
    
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return image;
}

/*
 @abstract 将tableview 去掉cell多余的分割线
 */
+ (void)setExtraCellLineHidden: (UITableView *)tableView
{
    UIView *view = [UIView new];
    view.backgroundColor = [UIColor clearColor];
    [tableView setTableFooterView:view];
}

/*
 @abstract 创建Label
 */
+ (UILabel*)setupLabelframe:(CGRect)frame
                       text:(NSString *)text
                   fontSize:(CGFloat)fontSize
                  textColor:(UIColor*)textColor
              textAlignment:(NSTextAlignment)textAlignment
{
    //透明度
    UILabel* label = [[UILabel alloc]initWithFrame:frame];
    label.textAlignment = textAlignment;
    label.font = [UIFont systemFontOfSize:fontSize];
    label.textColor = textColor;
    label.text = text;

    return label;
}

/*
 @abstract 创建Button
 */
+ (UIButton*)setupButtonframe:(CGRect)frame
                        title:(NSString *)title
                     fontSize:(CGFloat)fontSize
                    textColor:(UIColor*)textColor
              normalImageName:(NSString *)normalImageName
        hightLightedImageName:(NSString *)hightLightedImageName
                          tag:(NSInteger)tag
{
    UIButton* button = [UIButton buttonWithType:UIButtonTypeCustom];
    [button setFrame:frame];
    button.tag = tag;
    if (title)
    {
        [button setTitle:title forState:UIControlStateNormal];
        button.titleLabel.font = [UIFont systemFontOfSize:fontSize];
        [button setTitleColor:textColor forState:UIControlStateNormal];
    }
    [button setBackgroundImage:[UIImage imageNamed:normalImageName] forState:UIControlStateNormal];
    [button setBackgroundImage:[UIImage imageNamed:hightLightedImageName] forState:UIControlStateHighlighted];

    return button;
}

/**
 @abstract 当前的时间字符串
 */
+ (NSString *)currentDateString {
    NSDate *date = [NSDate date];
    NSDateFormatter *formatter = [[NSDateFormatter alloc]init];
    formatter.dateFormat = @"HH:mm:ss";
    NSString *dateString = [formatter stringFromDate:date];
    return dateString;
}

/*
 @abstract 取得今天的日期 yyyy/MM/dd
 */
+ (NSString *)getTodayDateStr
{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"yyyy.MM.dd"];
    NSString *dateTime = [formatter stringFromDate:[NSDate date]];
    return dateTime;
}


/**
 @abstract 是否时竖屏
 */
+ (BOOL)isInterfaceOrientationPortrait {
    UIInterfaceOrientation o = [[UIApplication sharedApplication] statusBarOrientation];
    return o == UIInterfaceOrientationPortrait;
}

/**
 @abstract 播放时间显示格式处理
 */
+ (NSString *)timeformatFromSeconds:(NSInteger)seconds {
    //format of hour
    NSString *str_hour = [NSString stringWithFormat:@"%02ld", lround((long long) seconds / 3600)];
    //format of minute
    NSString *str_minute = [NSString stringWithFormat:@"%02ld",lround( (long long) (seconds % 3600) / 60)];
    //format of second
    NSString *str_second = [NSString stringWithFormat:@"%02ld",lround( (long long) seconds % 60)];
    //format of time
    NSString *format_time = nil;
    if (seconds / 3600 <= 0) {
        format_time = [NSString stringWithFormat:@"%@:%@", str_minute, str_second];
    } else {
        format_time = [NSString stringWithFormat:@"%@:%@:%@", str_hour, str_minute, str_second];
    }
    return format_time;
}

/*
 @abstract NSTimeInterval 转 NSString
 */
+ (NSString*)stringWithNSTimeInterval:(NSTimeInterval)time{
    
    //format of hour
    NSString *str_hour = [NSString stringWithFormat:@"%02li",lround(floor(time / 3600.)) % 100];
    //format of minute
    NSString *str_minute = [NSString stringWithFormat:@"%02li",lround(floor(time / 60.)) % 60];
    //format of second
    NSString *str_second = [NSString stringWithFormat:@"%02li",lround(floor(time)) % 60];
    
    NSString* format_time = nil;
    if (time / 3600 <= 0) {
        format_time = [NSString stringWithFormat:@"%@:%@", str_minute, str_second];
    } else {
        format_time = [NSString stringWithFormat:@"%@:%@:%@", str_hour, str_minute, str_second];
    }
    return format_time;
    
}

/*
 @abstract 检查网络状态
 */
+ (int)checkNetWork{
    
    JDCloudNetworkStatus internetStatus = [[JDCloudReachability reachabilityForInternetConnection] currentReachabilityStatus];
    return internetStatus;
}

/*
 @abstract 获取手机当前网络类型
 */
+ (NSString *)networkType
{
    NSString* netWorkType = @"";
    if ([JDStaticTools checkNetWork] > 0){  //有网络
        if ([JDStaticTools checkNetWork] == 1){ //WWAN:4G 3G 2G
           netWorkType = @"NO WIFI";
        }
        else{//WIFI
            netWorkType = @"WIFI";
        }
    }
    else{//无网络
        netWorkType = @"无网络";
    }
    return netWorkType;
}

/*
 @abstract 图片库bundle
 */
+ (NSBundle *)resourceBundle {
    NSBundle *resourceBundle = [NSBundle bundleWithPath:[[NSBundle mainBundle] pathForResource:JDCloudUtilImageBundle ofType:@"bundle"]];
    if (!resourceBundle) {
        resourceBundle = [NSBundle mainBundle];
    }
    return resourceBundle;
}

+ (NSString *)dictionary2JsonString:(NSDictionary *)dict
{
    if ([NSJSONSerialization isValidJSONObject:dict])
        {
            NSError *error = nil;
            NSData *data = [NSJSONSerialization dataWithJSONObject:dict options:0 error:&error];
            if(error)
            {
                NSLog(@"[%@] Post Json Error", [self class]);
            }
            NSString* jsonString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
            return jsonString;
        }
        else
        {
            NSLog(@"[%@] Post Json is not valid", [self class]);
        }
        return nil;
}

+ (NSDictionary *)jsonData2Dictionary:(NSString *)jsonData
{
    if (jsonData == nil) {
        return nil;
    }
    NSData *data = [jsonData dataUsingEncoding:NSUTF8StringEncoding];
    NSError *err = nil;
    NSDictionary *dic = [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:&err];
    if (err) {
        NSLog(@"Json parse failed: %@", jsonData);
        return nil;
    }
    return dic;
}

@end
