//
//  JDKLineChart.m
//  JDCloudChart
//
//  Created by 彭丹丹 on 2018/11/28.
//  Copyright © 2018年 彭丹丹. All rights reserved.
//

#import "JDKLineChart.h"
#import "YAxisView.h"
#import "XAxisView.h"
#import "JDLineModel.h"

static const int statSeondCount = 1000;

static const int statPacketCount = 50000;

//分时统计
typedef struct SendStatData
{
    //字节统计
    int inVideoBytes;
    int outVideoBytes;
    int bufferVideoBytes;
    
    int inAudioBytes;
    int outAudioBytes;
    int bufferAudioBytes;
    
    int inTotalBytes;
    int outTotalBytes;
    int bufferTotalBytes;
    
    //包统计
    int inVideoPackets;
    int outVideoPackets;
    int bufferVideoPackets;
    
    int inAudioPackets;
    int outAudioPackets;
    int bufferAudioPackets;
    
    int inTotalPackets;
    int outTotalPackets;
    int bufferTotalPackets;
    
    //丢帧统计
    int dropBytes;
    int dropPackets;
    
    //时间
    int64_t startTime;
    int64_t duration;
    
}SendStatData;

// 分包统计
typedef struct PacketStatData
{
    int frameCount;         //当前包号码
    int packetSize;
    int isVideo;
    int dropped;
    
    int64_t dts;
    int64_t pts;
    int64_t inputTime;
    int64_t outputTime;
    
}PacketStatData;

typedef struct InnerData
{
    //分时统计数据
    uint64_t lastSecondStatTime;
    SendStatData lastSecondStatData;
    SendStatData summaryStatData;
    
    SendStatData statData[statSeondCount];
    int currentStatDataIndex;
    
    //分包统计数据
    PacketStatData packetStatData[statPacketCount];
    int currentPacketStatDataIndex;
    
    //I帧被drop
    int keyFrameIsDropped;
    //当前码率
    int currentBitrate;
    //本次update时间
    uint64_t currentUpdateTime;
    //上一次上调时间
    uint64_t lastBitrateIncreaseTime;
    //上一次下调时间
    uint64_t lastBitrateDecreaseTime;
    
}InnerData;

static CGFloat KYAxisViewWidth = 40;    //Y轴View宽度

@interface JDKLineChart ()<UIScrollViewDelegate>

@property (nonatomic, strong) UIScrollView *scrollView;

@property (nonatomic, strong) XAxisView* xAxisView;

@property (nonatomic, strong) YAxisView *yAxisView;

@property (nonatomic, assign) CGFloat xAxisViewWidth;

@property (nonatomic, strong) NSMutableArray* selectedBtnArr;

@property (nonatomic, assign) NSInteger statisticsType; //0:分时  1:分包

@property (assign, nonatomic) CGFloat x;

@end


@implementation JDKLineChart

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        [self setupsubViews];
        [self addNotifications];
    }
    return self;
}

- (void)setupsubViews
{
    //_scrollView
    self.scrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(KYAxisViewWidth, 0, self.frame.size.width - KYAxisViewWidth - 10, self.frame.size.height)];
    self.scrollView.showsHorizontalScrollIndicator = NO;
    self.scrollView.delegate = self;
    [self addSubview:self.scrollView];
    
    //Y轴
    self.yAxisView = [[YAxisView alloc]initWithFrame:CGRectMake(0, 0, KYAxisViewWidth, self.frame.size.height)];
    [self.yAxisView updataYAxiswithMax:300 WithMin:0 withUnit:@"KB/s"];
    [self addSubview:self.yAxisView];
    
    //X轴 背景网络
    self.xAxisViewWidth = self.scrollView.frame.size.width;
    self.xAxisView = [[XAxisView alloc] initWithFrame:CGRectMake(0, 0, self.xAxisViewWidth, self.scrollView.frame.size.height)];
    [self.scrollView addSubview:self.xAxisView];
    
}

//画哪些线
- (void)drawKLineWithLineArr:(NSMutableArray*)arr WithStatisticsType:(NSInteger)type{
    
    //更新数据
    self.selectedBtnArr = [NSMutableArray arrayWithArray:arr];
    
    self.statisticsType = type;
    if (self.statisticsType == 0) {
        [self.yAxisView updataYAxiswithMax:300 WithMin:0 withUnit:@"KB/s"];
    }else{
        [self.yAxisView updataYAxiswithMax:60 WithMin:0 withUnit:@"个/s"];
    }
}

#pragma mark -
#pragma mark - 通知
- (void)addNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(onJDLStatisticsDataEventNotification:) name:@"JDLStatisticsDataEventNotification" object:nil];
}

//统计数据的通知
- (void)onJDLStatisticsDataEventNotification:(NSNotification *)notification
{
    NSMutableArray* KLineArr = [NSMutableArray array];
    
    //获取统计数据
    NSDictionary *dic = [notification userInfo];
    long long data = [dic[@"evMsg"] longLongValue];
    InnerData *inner_Data = (InnerData*)data;
    long long last = inner_Data->statData[999].startTime;
    
    for (int j = 0; j<self.selectedBtnArr.count; j++) {
        JDLineModel* line = [[JDLineModel alloc]init];
        line.lineWidth = 2;
        line.lineColor = [self lineColorWithLine:self.selectedBtnArr[j]];
        line.pointArr = [NSMutableArray array];
        self.x = -1;
        if (last > 0) {
            for (int i = inner_Data->currentStatDataIndex + 1; i < 1000; i++) {  //[current 999]
                self.x ++;
                NSValue *pointValue = [self getPointValueWithInnerData:inner_Data withLine:self.selectedBtnArr[j] withIndex:i];
                [line.pointArr addObject:pointValue];
            }
        }
        for (int i = 0; i <= inner_Data->currentStatDataIndex; i++) {  //[0 current]
            self.x ++;
            NSValue *pointValue = [self getPointValueWithInnerData:inner_Data withLine:self.selectedBtnArr[j] withIndex:i];
            [line.pointArr addObject:pointValue];
        }
        [KLineArr addObject:line];
    }

    [self.xAxisView drawKLineWithLineArr:KLineArr WithStatisticsType:self.statisticsType];

    //修改X轴width 便于滑动
    dispatch_async(dispatch_get_main_queue(), ^{
        self.xAxisViewWidth = self.xAxisView.frame.size.width;
        CGRect frame = self.xAxisView.frame;
        frame.size.width  = self.xAxisViewWidth + (self.frame.size.width - KYAxisViewWidth - 10)/40;
        self.xAxisView.frame = frame;
        self.scrollView.contentSize = self.xAxisView.frame.size;
    });
}

//获取线对应的点集
- (NSValue*)getPointValueWithInnerData:(InnerData*)InnerData withLine:(NSString*)line withIndex:(NSInteger)index
{
    long long y;
    if ([line isEqualToString:@"inTotalBytes"]) {
        y = InnerData->statData[index].inTotalBytes;
    }else if ([line isEqualToString:@"outTotalBytes"]){
        y = InnerData->statData[index].outTotalBytes;
    }else if ([line isEqualToString:@"bufferVideoBytes"]){
        y = InnerData->statData[index].bufferVideoBytes;
    }else if ([line isEqualToString:@"dropBytes"]){
        y = InnerData->statData[index].dropBytes;
    }else if ([line isEqualToString:@"inVideoPackets"]){
        y = InnerData->statData[index].inVideoPackets;
    }else if ([line isEqualToString:@"outVideoPackets"]){
        y = InnerData->statData[index].outVideoPackets;
    }else if ([line isEqualToString:@"bufferVideoPackets"]){
        y = InnerData->statData[index].bufferVideoPackets;
    }else if ([line isEqualToString:@"dropPackets"]){
        y = InnerData->statData[index].dropPackets;
    }else{
        y = 0;
    }
    CGPoint point = CGPointMake(self.x,y);
    NSValue *pointValue = [NSValue valueWithCGPoint:point];
    return pointValue;
}

//创建线颜色
- (UIColor*)lineColorWithLine:(NSString*)line{
    if ([line isEqualToString:@"inTotalBytes"]) {
        return [UIColor redColor];
    }else if ([line isEqualToString:@"outTotalBytes"]){
        return [UIColor orangeColor];
    }else if ([line isEqualToString:@"bufferVideoBytes"]){
        return [UIColor yellowColor];
    }else if ([line isEqualToString:@"dropBytes"]){
        return [UIColor greenColor];
    }else if ([line isEqualToString:@"inVideoPackets"]){
        return [UIColor blueColor];
    }else if ([line isEqualToString:@"outVideoPackets"]){
        return [UIColor purpleColor];
    }else if ([line isEqualToString:@"bufferVideoPackets"]){
        return [UIColor blackColor];
    }else if ([line isEqualToString:@"dropPackets"]){
        return [UIColor whiteColor];
    }else{
        return [UIColor clearColor];
    }
}


@end
