//
//  JDPhotoManager.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/19.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDPhotoManager.h"
#import <Photos/Photos.h>
#import <MobileCoreServices/MobileCoreServices.h>
#import "SDWebImage.h"

@implementation JDPhotoManager

static JDPhotoManager *sharedPhotoManager = nil;

+ (instancetype)sharedPhotoManager
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedPhotoManager = [[self alloc] init];
    });
    return sharedPhotoManager;
}


-(void)ShowSheetToSelectedMediaWithInFatherVC:(UIViewController *)fatherVC FinishPickingMediaBlock:(FinishPickingMediaBlock)succBlock
{
    self.fatherViewController = fatherVC;
    self.succBlock = succBlock;
    UIActionSheet *sheet = [[UIActionSheet alloc] initWithTitle:nil
                                                       delegate:self
                                              cancelButtonTitle:@"取消"
                                         destructiveButtonTitle:nil
                                              otherButtonTitles:@"从手机相册上传",/* @"相机拍照",*/ nil];
    
    [sheet showInView:self.fatherViewController.view];
}


#pragma mark - UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (buttonIndex == 0) {
        [self showPhotoLibrary];
    }else if (buttonIndex == 1) {
      //  [self showCamera];
    }
}


#pragma mark -
#pragma mark - 开始弹出相册
//相机
- (void)showCamera
{
    // 判断有摄像头，并且支持拍照功能
    if ([self isCameraAvailable] && [self doesCameraSupportTakingPhotos]){
        // 初始化图片选择控制器
        UIImagePickerController *pickerController = [[UIImagePickerController alloc] init];
        [pickerController setSourceType:UIImagePickerControllerSourceTypeCamera];// 设置类型
        
        // 设置所支持的类型，设置只能拍照，或则只能录像，或者两者都可以
        NSString *requiredMediaType = ( NSString *)kUTTypeImage;
        NSString *requiredMediaType1 = ( NSString *)kUTTypeMovie;
        NSArray *arrMediaTypes=[NSArray arrayWithObjects:requiredMediaType, requiredMediaType1,nil];
        [pickerController setMediaTypes:arrMediaTypes];
        
        // 设置录制视频的质量
        [pickerController setVideoQuality:UIImagePickerControllerQualityTypeHigh];
        //设置最长摄像时间
        [pickerController setVideoMaximumDuration:10.f];
        
        [pickerController setAllowsEditing:YES];// 设置是否可以管理已经存在的图片或者视频
        [pickerController setDelegate:self];// 设置代理
        [self.fatherViewController presentViewController:pickerController animated:YES completion:nil];
    }
    else {
        UIAlertView * alert = [[UIAlertView alloc] initWithTitle:@"提示"
                                                         message:@"该设备相机不可用"
                                                        delegate:nil
                                               cancelButtonTitle:@"确定"
                                               otherButtonTitles:nil];
        [alert show];
    }
    
}

//图册
- (void)showPhotoLibrary
{
    if ([self isPhotoLibraryAvailable]){
        UIImagePickerController *pickerController = [[UIImagePickerController alloc] init];
        [pickerController setSourceType:UIImagePickerControllerSourceTypePhotoLibrary];// 设置类型
        NSMutableArray *mediaTypes = [[NSMutableArray alloc] init];
        if ([self canUserPickPhotosFromPhotoLibrary]){
            [mediaTypes addObject:( NSString *)kUTTypeImage];
        }
//        if ([self canUserPickVideosFromPhotoLibrary]){
//            [mediaTypes addObject:( NSString *)kUTTypeMovie];
//        }
        
        [pickerController setMediaTypes:mediaTypes];
        [pickerController setDelegate:self];// 设置代理
        [self.fatherViewController presentViewController:pickerController animated:YES completion:nil];
    }
}

#pragma mark -
#pragma mark - UIImagePickerControllerDelegate 代理方法
/*
 *  保存图片后到相册后，调用的相关方法，查看是否保存成功
 */
- (void) imageWasSavedSuccessfully:(UIImage *)paramImage didFinishSavingWithError:(NSError *)paramError contextInfo:(void *)paramContextInfo{
    if (paramError == nil){
        NSLog(@"Image was saved successfully.");
    } else {
        NSLog(@"An error happened while saving the image.");
        NSLog(@"Error = %@",paramError);
    }
}
/*
 *  当得到照片或者视频后，调用该方法
 */
- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info{
    NSLog(@"Picker returned successfully.");
    NSLog(@"%@",info);
    
    NSURL *imageRefURL = [info valueForKey:UIImagePickerControllerReferenceURL];
    NSString *mediaType = [info objectForKey:UIImagePickerControllerMediaType];
    // 判断获取类型：图片
    if ([mediaType isEqualToString:( NSString *)kUTTypeImage]){
        UIImage *theImage = nil;
        
        ALAssetsLibrary* assetLibrary = [[ALAssetsLibrary alloc] init];
        void (^ALAssetsLibraryAssetForURLResultBlock)(ALAsset *) = ^(ALAsset *asset) {
            
            if (asset != nil)
            {
                ALAssetRepresentation *rep = [asset defaultRepresentation];
                Byte *imageBuffer = (Byte*)malloc(rep.size);
                NSUInteger bufferSize = [rep getBytes:imageBuffer fromOffset:0.0 length:rep.size error:nil];
                NSData *imageData = [NSData dataWithBytesNoCopy:imageBuffer length:bufferSize freeWhenDone:YES];
                
                UIImage *image = nil;
                if ([asset representationForUTI:(__bridge NSString *)kUTTypeGIF])
                {
                    image = [UIImage sd_imageWithGIFData:imageData];
                }
                else
                {
                    image = [UIImage sd_imageWithData:imageData];
                }
                
                //图片成功的回调
                if (image && self.succBlock) {
                    self.succBlock(image, nil);
                }
            }
            else
            {
                
            }
        };
        
        [assetLibrary assetForURL:imageRefURL
                      resultBlock:ALAssetsLibraryAssetForURLResultBlock
                     failureBlock:^(NSError *error){
                     }];
        
//        // 判断，图片是否允许修改
//        if ([picker allowsEditing]){
//            //获取用户编辑之后的图像
//            theImage = [info objectForKey:UIImagePickerControllerEditedImage];
//        } else {
//            // 照片的元数据参数
//            theImage = [info objectForKey:UIImagePickerControllerOriginalImage];
//        }
//
//        //图片成功的回调
//        if (self.succBlock) {
//            self.succBlock(theImage, nil);
//        }
    }
    else if ([mediaType isEqualToString:(NSString *)kUTTypeMovie]){
        // 判断获取类型：视频
        //获取视频文件的url
        NSURL* mediaURL = [info objectForKey:UIImagePickerControllerMediaURL];
        //创建ALAssetsLibrary对象并将视频保存到媒体库
        // Assets Library 框架包是提供了在应用程序中操作图片和视频的相关功能。相当于一个桥梁，链接了应用程序和多媒体文件。
       // ALAssetsLibrary *assetsLibrary = [[ALAssetsLibrary alloc] init];
        // 将视频保存到相册中
        /*[assetsLibrary writeVideoAtPathToSavedPhotosAlbum:mediaURL
                                          completionBlock:^(NSURL *assetURL, NSError *error) {
                                              if (!error) {
                                                  NSLog(@"captured video saved with no error.");
                                              }else{
                                                  NSLog(@"error occured while saving the video:%@",error);
                                              }
                                          }];*/
        
        //视频成功的回调
        if (self.succBlock) {
            self.succBlock(nil, mediaURL);
        }
    }
    [picker dismissViewControllerAnimated:YES completion:nil];
}

/*
 *  当用户取消时，调用该方法
 */
- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker{
    [picker dismissViewControllerAnimated:YES completion:nil];
    
}


#pragma mark -
#pragma mark -

/*
 *  判断设备是否有摄像头
 */
- (BOOL) isCameraAvailable{
    return [UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera];
}

/*
 *  前面的摄像头是否可用
 */
- (BOOL) isFrontCameraAvailable{
    return [UIImagePickerController isCameraDeviceAvailable:UIImagePickerControllerCameraDeviceFront];
}


/*
 *  后面的摄像头是否可用
 */
- (BOOL) isRearCameraAvailable{
    return [UIImagePickerController isCameraDeviceAvailable:UIImagePickerControllerCameraDeviceRear];
}


/*
 *  判断是否支持某种多媒体类型：拍照，视频
 */
- (BOOL) cameraSupportsMedia:(NSString *)paramMediaType sourceType:(UIImagePickerControllerSourceType)paramSourceType{
    __block BOOL result = NO;
    if ([paramMediaType length] == 0){
        NSLog(@"Media type is empty.");
        return NO;
    }
    NSArray *availableMediaTypes =[UIImagePickerController availableMediaTypesForSourceType:paramSourceType];
    [availableMediaTypes enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL*stop) {
        NSString *mediaType = (NSString *)obj;
        if ([mediaType isEqualToString:paramMediaType]){
            result = YES;
            *stop= YES;
        }
    }];
    return result;
}


/*
 *  检查摄像头是否支持录像
 */
- (BOOL) doesCameraSupportShootingVideos{
    return [self cameraSupportsMedia:(NSString *)kUTTypeMovie sourceType:UIImagePickerControllerSourceTypeCamera];
}


/*
 *  检查摄像头是否支持拍照
 */
- (BOOL) doesCameraSupportTakingPhotos{
    return [self cameraSupportsMedia:(NSString *)kUTTypeImage sourceType:UIImagePickerControllerSourceTypeCamera];
}

#pragma mark - 相册文件选取相关

/*
 *  相册是否可用
 */
- (BOOL) isPhotoLibraryAvailable{
    return [UIImagePickerController isSourceTypeAvailable: UIImagePickerControllerSourceTypePhotoLibrary];
}


/*
 *  是否可以在相册中选择视频
 */
- (BOOL) canUserPickVideosFromPhotoLibrary{
    return [self cameraSupportsMedia:( NSString *)kUTTypeMovie sourceType:UIImagePickerControllerSourceTypePhotoLibrary];
}


/*
 *  是否可以在相册中选择图片
 */
- (BOOL) canUserPickPhotosFromPhotoLibrary{
    return [self cameraSupportsMedia:( NSString *)kUTTypeImage sourceType:UIImagePickerControllerSourceTypePhotoLibrary];
}



@end
