//
//  JDCloudUtil.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudUtil.h"
#import <ifaddrs.h>
#import <arpa/inet.h>

static NSString * const JDCloudUtilImageBundleName = @"JDCloudImageSource.bundle";
static NSString * const JDCloudUtilImageBundle     = @"JDCloudImageSource";
static NSString * const JDCloudUtilLanguageBundle  = @"JDCloudLanguageSource";
static const CGFloat JDCloudUtilTitleNomalTextSize = 28;
static NSString * const ALYPVVersion  = @"3.4.0";

@implementation JDCloudUtil

+ (NSString*)getSDKVersion{
    return ALYPVVersion;
}

+ (NSBundle *)resourceBundle {
    NSBundle *resourceBundle = [NSBundle bundleWithPath:[[NSBundle mainBundle] pathForResource:JDCloudUtilImageBundle ofType:@"bundle"]];
    if (!resourceBundle) {
        resourceBundle = [NSBundle mainBundle];
    }
    return resourceBundle;
}

+ (NSBundle *)languageBundle {
    NSBundle *resourceBundle = [NSBundle mainBundle];
    return resourceBundle;
}

+ (UIImage *)imageWithNameInBundle:(NSString *)nameInBundle {
    return [UIImage imageNamed:[NSString stringWithFormat:@"%@",nameInBundle]];
}



+ (BOOL)isInterfaceOrientationPortrait {
    UIInterfaceOrientation o = [[UIApplication sharedApplication] statusBarOrientation];
    return o == UIInterfaceOrientationPortrait;
}

+ (void)setFullOrHalfScreen {
    BOOL isFull = [self isInterfaceOrientationPortrait];
    if ([[UIDevice currentDevice] respondsToSelector:@selector(setOrientation:)]) {
        SEL selector = NSSelectorFromString(@"setOrientation:");
        NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:[UIDevice instanceMethodSignatureForSelector:selector]];
        [invocation setSelector:selector];
        [invocation setTarget:[UIDevice currentDevice]];
        int val = isFull ? UIInterfaceOrientationLandscapeRight:UIInterfaceOrientationPortrait;
        
        [invocation setArgument:&val atIndex:2];
        [invocation invoke];
    }
    [[UIApplication sharedApplication]setStatusBarOrientation:UIInterfaceOrientationLandscapeRight animated:YES];
}

+ (NSString *)timeformatFromSeconds:(NSInteger)seconds {
    //format of hour
    NSString *str_hour = [NSString stringWithFormat:@"%02ld", (long) seconds / 3600];
    //format of minute
    NSString *str_minute = [NSString stringWithFormat:@"%02ld", (long) (seconds % 3600) / 60];
    //format of second
    NSString *str_second = [NSString stringWithFormat:@"%02ld", (long) seconds % 60];
    //format of time
    NSString *format_time = nil;
    if (seconds / 3600 <= 0) {
        format_time = [NSString stringWithFormat:@"00:%@:%@", str_minute, str_second];
    } else {
        format_time = [NSString stringWithFormat:@"%@:%@:%@", str_hour, str_minute, str_second];
    }
    return format_time;
}

+ (void)drawFillRoundRect:(CGRect)rect radius:(CGFloat)radius color:(UIColor *)color context:(CGContextRef)context {
    CGContextSetAllowsAntialiasing(context, TRUE);
    CGContextSetFillColor(context, CGColorGetComponents(color.CGColor));
    //    CGContextSetRGBFillColor(context, red, green, blue, alpha);
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, CGRectGetMinX(rect), CGRectGetMidY(rect));
    CGContextAddArcToPoint(context, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMidX(rect), CGRectGetMinY(rect), radius);
    CGContextAddArcToPoint(context, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMidY(rect), radius);
    CGContextAddArcToPoint(context, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMidX(rect), CGRectGetMaxY(rect), radius);
    CGContextAddArcToPoint(context, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMidY(rect), radius);
    CGContextClosePath(context);
    CGContextFillPath(context);
}



+ (float)convertPixelToPoint:(float)pixel {
    if (pixel < 0) {
        return 0;
    }
    return pixel / 2;
}

+ (float)nomalTextSize {
    return JDCloudUtilTitleNomalTextSize / 2.0;
}

//"fd_definition" = "流畅";
//"ld_definition" = "标清";
//"sd_definition" = "高清";
//"hd_definition" = "超清";
//"2k_definition" = "2K";
//"4k_definition" = "4K";
//"od_definition" = "OD";
//获取所有已知清晰度泪飙
+ (NSArray<NSString *> *)allQualities {
    return @[@"流畅",@"标清",@"高清",@"超清",@"2K",@"4K",@"原画"];
}

+ (void)setPlayFinishTips:(NSString *)des{
    JDClOUDVODVIEW_PLAYFINISH = des;
}

+ (NSString *)playFinishTips{
    return JDClOUDVODVIEW_PLAYFINISH;
}

+ (void)setNetworkTimeoutTips:(NSString *)des{
    JDClOUDVODVIEW_NETWORKTIMEOUT = des;
}

+ (NSString *)networkTimeoutTips{
    return JDClOUDVODVIEW_NETWORKTIMEOUT;
}

+ (void)setNetworkUnreachableTips:(NSString *)des{
    JDClOUDVODVIEW_NETWORKUNREACHABLE = des;
}

+ (NSString *)networkUnreachableTips{
    return JDClOUDVODVIEW_NETWORKUNREACHABLE;
}

+ (void)setLoadingDataErrorTips:(NSString *)des{
    JDClOUDVODVIEW_LOADINGDATAERROR = des;
}

+ (NSString*)loadingDataErrorTips{
    return JDClOUDVODVIEW_LOADINGDATAERROR;
}

+ (void)setSwitchToMobileNetworkTips:(NSString *)des{
    JDClOUDVODVIEW_USEMOBILENETWORK = des;
}
+ (NSString *)switchToMobileNetworkTips{
    return JDClOUDVODVIEW_USEMOBILENETWORK;
}

@end
