//
//  JDPushFlowHighSetViewController.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/6.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDPushFlowHighSetViewController.h"
#import "PushFlowParamModel.h"
#import "PushFlowParamTableViewCell.h"
#import "JDStartPushFlowViewController.h"
#import "JDScanViewController.h"
#import "AppDelegate.h"

@interface JDPushFlowHighSetViewController ()<UITableViewDelegate,UITableViewDataSource,UIGestureRecognizerDelegate>

@property (nonatomic, strong) UITableView *paramTableView;              //TableView
@property (nonatomic, strong) NSMutableArray *dataArray;                //TableView数据
@property (nonatomic, strong) NSArray* headerTitleArr;                  //TableView头数据
@property (nonatomic, strong) UIButton* liveButton;                     //开始直播

@property (nonatomic, strong) ConfigPushFlowParamModel *configPushFlowParamModel;    //直播推流参数model

@end

@implementation JDPushFlowHighSetViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"高级设置";
    
    [self setupSubviews];
    [self setupParamData];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.hidden = NO;
    [self.view endEditing:YES];
}

//传过来的model
- (void)resultConfigPushFlowParamModel:(ConfigPushFlowParamModel *)model{
    self.configPushFlowParamModel = model;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    
}

#pragma mark -
#pragma mark - 初始化子试图
//初始化子试图
- (void)setupSubviews
{ 
    [self addSubViews];         //增加子试图
    [self addGesture];          //增加手势
    [self addNotifications];    //键盘通知
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupParamData
{
    self.headerTitleArr = @[@"",@"视频参数",@"音频参数",@"设备设置"];
    [self initPushFlowParamModel];         //初始化cell的model
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //二维码
        {
            //1、判断相机权限是否打开
            AVAuthorizationStatus status = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
            if (status == AVAuthorizationStatusRestricted || status == AVAuthorizationStatusDenied)
            {
                // 无权限
                [[[JDActionController alloc]init] showAlertControllerWithTitle:@"“京东视频云”想访问您的相机" message:@"京东视频云需要访问你的相机权限，开启后才能扫描二维码" preferredStyle:UIAlertControllerStyleAlert otherButtonTitles:@[@"不允许",@"好"] superViewController:self actionControllerBlock:^(NSString *buttonTitle) {
                    if ([buttonTitle isEqualToString:@"好"]){
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
                    }
                }];
                return;
            }
            
            JDScanViewController* VC = [[JDScanViewController alloc]init];
            VC.style = [JDScanViewController scanViewUIStyle];
            VC.succBlock = ^(NSString *scanResultStr) {
                self.configPushFlowParamModel.pushUrl = scanResultStr;
                [self reloadPushFlowParamModel];
                [self.paramTableView reloadData];
            };
            [self.navigationController pushViewController:VC animated:YES];
        }
            break;
        case 102:   //开始直播
        {
            if(![self checkConfigValueIsReasonable]){
                return;
            }
            AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
            if(app.floatPushStreamView == nil){
                if(self.configPushFlowParamModel.captureSource == 0){
                    JDStartPushFlowViewController* VC = [[JDStartPushFlowViewController alloc]init];
                    [VC resultConfigPushFlowParamModel:self.configPushFlowParamModel];
                    [self.navigationController pushViewController:VC animated:YES];
                }
                else{
                    if (@available(iOS 11.0, *)){
                        UIPasteboard* pb = [UIPasteboard generalPasteboard];
                        pb.string = [self.configPushFlowParamModel toJsonString];
                        UIAlertView * alert = [[UIAlertView alloc] initWithTitle:@"京东视频云"
                                                                         message:@"请先到控制中心，长按启动屏幕录制"
                                                                        delegate:nil
                                                               cancelButtonTitle:@"确定"
                                                               otherButtonTitles:nil];
                        [alert show];
                    }else{
                        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"录屏功能只支持iOS11级以上，请升级后使用！"];
                    }
                }
            }
            else{
                [app.floatPushStreamView resumVC];
            }
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - TableViewDataSources
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 4;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section == 0) {
        return 1;
    }else if (section == 1){
        if (self.configPushFlowParamModel.captureSource == 1) { //视频来源
            return 6;  //录屏
        }else{
            return 8;   //摄像头
        }
    }else if (section == 2){
        return 4;
    }else{
        if (self.configPushFlowParamModel.captureSource == 1) { //视频来源
            return 2;  //录屏
        }else{
           return 6;   //摄像头
        }
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    // section 0 ：推流地址、
    // section 1 ：推流分辨率、采集分辨率、预览分辨率、码率、码率调节、帧率、关键帧、编码方式、（录制时去掉采集分辨率、预览分辨率）
    // section 2 ：采样率、码率、编码方式、声道、
    // section 4 ：设备模式、镜头模式、后台推流、重连时长、画中画、重连次数（录制时设备模式:横、竖，去掉镜头模式、后台推流、画中画）
    
    PushFlowParamModel *model = self.dataArray[indexPath.row];
    if (indexPath.section == 0) {
        model = self.dataArray[0];
    }
    else if (indexPath.section == 1){
        if (self.configPushFlowParamModel.captureSource == 1) { //录制 section = 1 去掉采集分辨率、预览分辨率
            if(indexPath.row == 0){
                model = self.dataArray[1];
            }else{
                model = self.dataArray[3 + indexPath.row];
            }
        }else{ //摄像头
            model = self.dataArray[1 + indexPath.row];
        }
    }
    else if (indexPath.section == 2)
    {
        model = self.dataArray[9 + indexPath.row];
    }
    else{
        if (self.configPushFlowParamModel.captureSource == 1) { //录制  section = 4 去掉设备模式、镜头模式、后台推流、画中画
            model = self.dataArray[17 + indexPath.row];
        }else{  //摄像头
            model = self.dataArray[13 + indexPath.row];
        }
    }

    if (model) {
        PushFlowParamTableViewCell *cell = [[PushFlowParamTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
        [cell configureCellModel:model withHight:40 from:@"JDPushFlowHighSetViewController"];
        return cell;
    }
    return nil;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
    return self.headerTitleArr[section];
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section
{
    if (section == 3) {
        return 10;
    }
    return 0.1f;
}

//section底部视图
- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section
{
    UIView *view=[[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 0)];
    view.backgroundColor = [UIColor clearColor];
    return view;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
   
    if (section == 0) {
         return 0.1;
     }
    return 44;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 40;
}

- (nullable UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    if (section != 0) {
        UIView* view = [[UIView alloc]initWithFrame: CGRectMake(0, 0, ScreenWidth, 44)];
        view.backgroundColor = [UIColor clearColor];
        
        UILabel* label = [[UILabel alloc]initWithFrame:CGRectMake(10, 0, 100, 44)];
        label.textColor = RGBCOLOR(161, 163, 174);
        label.text = self.headerTitleArr[section];
        label.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [view addSubview:label];
        return view;
    }else{
        UIView* view = [[UIView alloc]initWithFrame: CGRectMake(0, 0, ScreenWidth, 44)];
        view.backgroundColor = [UIColor clearColor];
        return view;
    }
}

#pragma mark -
#pragma mark - 功能函数
- (void)addSubViews {
    //TableView
    self.paramTableView = [[UITableView alloc] initWithFrame:CGRectMake(0, 0, ScreenWidth, ScreenHeight-60) style:UITableViewStyleGrouped];
    self.paramTableView.delegate = (id)self;
    self.paramTableView.dataSource = (id)self;
    self.paramTableView.separatorStyle = NO;
    self.paramTableView.backgroundColor = [UIColor clearColor];
    self.paramTableView.contentSize = CGSizeMake(ScreenWidth, 1800);
    self.paramTableView.showsVerticalScrollIndicator = NO;
    self.paramTableView.showsHorizontalScrollIndicator = NO;
    [self.view addSubview:self.paramTableView];
    
    self.liveButton =  [UIButton buttonWithType:(UIButtonTypeCustom)];
    self.liveButton.frame = CGRectMake(0, ScreenHeight - 60 - SafeAreaBottomHeight, ScreenWidth, 60 + SafeAreaBottomHeight);
    [self.liveButton setTitleEdgeInsets:UIEdgeInsetsMake(0, 0, self.liveButton.titleLabel.frame.origin.y + SafeAreaBottomHeight, 0)];
    [self.liveButton setTitle:@"开始直播" forState:UIControlStateNormal];
    self.liveButton.backgroundColor = RGBCOLOR(39, 47, 80);
    self.liveButton.titleLabel.font = [UIFont systemFontOfSize:17 * SCALE_FONT];
    self.liveButton.tag  = 102;
    [self.liveButton setTitleColor:[UIColor whiteColor] forState:(UIControlStateNormal)];
    [self.liveButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.view addSubview:self.liveButton];
}

//初始化Cell的model
- (void)initPushFlowParamModel{
    
    //采集支持 是否支持1080p
    NSArray* _collectionResolutionRatioArr = @[@"360P",@"540P",@"720P",@"1080P"];
    if (![JDLive isCanSetSessionPreset:AVCaptureSessionPreset1920x1080 withCameraPosition:AVCaptureDevicePositionFront]) {
        _collectionResolutionRatioArr = @[@"360P",@"540P",@"720P"];
    }
    
    //推流分辨率。推流和预览加1080p
    NSArray* _pushFlowResolutionRatioArr = @[@"360P",@"540P",@"720P",@"1080P"];
    //码率
    NSArray* _codeRatioArr = @[@"1000",@"800",@"600",@"400"];
    //码率调节
    NSArray* _codeRatioAdjustArr = @[@"恒定",@"动态"];
    //关键帧
    NSArray* _IFrameArr = @[@"1s",@"2s",@"3s",@"4s",@"5s"];
    //编码方式
    NSArray* _codeTypeArr = @[@"自适应",@"软编",@"硬编"];
    //采样率
    NSArray* _samplingRatioArr = @[@"32kHZ",@"44.1kHz",@"48kHz"];
    //音频编码方式
    NSArray* _audioCodeTypeArr = @[@"AAC_LC",@"HE_AAC",@"HEAAC_V2",@"AAC_LD"];
    //音频码率
    NSArray* _audioCodeRatioArr = @[@"32",@"48",@"64",@"128"];
    //声道
    NSArray* _soundTrackArr = @[@"单声道",@"双声道"];
    //屏幕方向
    NSArray* _screenDirectionArr = [NSArray array];
    _screenDirectionArr = @[@"横屏",@"竖屏",@"动态"];
    
    /*
    if (self.configPushFlowParamModel.captureSource == 1) { //视频来源 录制
        _screenDirectionArr = @[@"横屏",@"竖屏"];
    }else{ //摄像头
        _screenDirectionArr = @[@"横屏",@"竖屏",@"动态"];
    }*/
    //摄像头模式
    //NSArray* _cameraTypeArr = @[@"前置镜头",@"后置镜头",@"USB镜头"];
    NSArray* _cameraTypeArr = @[@"前置镜头",@"后置镜头",];
    
    //创建Cell的模型
    PushFlowParamModel *pushFlowAddress = [[PushFlowParamModel alloc] init];
    pushFlowAddress.title = @"推流地址";
    pushFlowAddress.infoText = @"二维码";
    pushFlowAddress.placeHolder = self.configPushFlowParamModel.pushUrl;
    pushFlowAddress.reuseId = PushFlowParamModelReuseCellInputButton;
    pushFlowAddress.inputAndButtonBlock = ^(NSString *message,UIButton* sender){
        if (message.length > 1) {
            self.configPushFlowParamModel.pushUrl = message;
        }else{
            if (sender) {
                sender.tag = 100;
                [self buttonClickAction:sender];
            }
        }
        PushFlowParamModel *model = self.dataArray[0];
        model.placeHolder =  self.configPushFlowParamModel.pushUrl;
    };
    
    PushFlowParamModel *pushFlowResolutionRatio = [[PushFlowParamModel alloc] init];
    pushFlowResolutionRatio.title = @"推流分辨率";
    pushFlowResolutionRatio.segmentTitleArray = _pushFlowResolutionRatioArr;
    pushFlowResolutionRatio.defaultValue = self.configPushFlowParamModel.streamResolution;
    pushFlowResolutionRatio.reuseId = PushFlowParamModelReuseCellSegment;
    pushFlowResolutionRatio.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.streamResolution = value;
        self.configPushFlowParamModel.pictureQuality = value;
      
        PushFlowParamModel *model = self.dataArray[1];
        model.defaultValue = self.configPushFlowParamModel.streamResolution;
    };
    
    PushFlowParamModel *collectionResolutionRatio = [[PushFlowParamModel alloc] init];
    collectionResolutionRatio.title = @"采集分辨率";
    collectionResolutionRatio.segmentTitleArray = _collectionResolutionRatioArr;
    collectionResolutionRatio.defaultValue = self.configPushFlowParamModel.captureResolution;
    collectionResolutionRatio.reuseId = PushFlowParamModelReuseCellSegment;
    collectionResolutionRatio.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.captureResolution = value;
        
        PushFlowParamModel *model = self.dataArray[2];
        model.defaultValue = self.configPushFlowParamModel.captureResolution;
    };
    
    PushFlowParamModel *previewResolutionRatio = [[PushFlowParamModel alloc] init];
    previewResolutionRatio.title = @"预览分辨率";
    previewResolutionRatio.segmentTitleArray = _pushFlowResolutionRatioArr;
    previewResolutionRatio.defaultValue = self.configPushFlowParamModel.previewResolution;
    previewResolutionRatio.reuseId = PushFlowParamModelReuseCellSegment;
    previewResolutionRatio.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.previewResolution = value;\
        
        PushFlowParamModel *model = self.dataArray[3];
        model.defaultValue = self.configPushFlowParamModel.previewResolution;
    };
    
    PushFlowParamModel *codeRatio = [[PushFlowParamModel alloc] init];
    codeRatio.title = @"码率";
    codeRatio.segmentTitleArray = _codeRatioArr;
    codeRatio.defaultValue = self.configPushFlowParamModel.bitrateType;
    codeRatio.reuseId = PushFlowParamModelReuseCellSegment;
    codeRatio.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.bitrateType = value;
        PushFlowParamModel *model = self.dataArray[4];
        model.defaultValue = self.configPushFlowParamModel.bitrateType;
    };
    
    PushFlowParamModel *codeRatioAdjust = [[PushFlowParamModel alloc] init];
    codeRatioAdjust.title = @"码率调节";
    codeRatioAdjust.segmentTitleArray = _codeRatioAdjustArr;
    codeRatioAdjust.defaultValue = self.configPushFlowParamModel.bitrateAdjustMode;
    codeRatioAdjust.reuseId = PushFlowParamModelReuseCellSegment;
    codeRatioAdjust.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.bitrateAdjustMode = value;
        
        PushFlowParamModel *model = self.dataArray[5];
        model.defaultValue = self.configPushFlowParamModel.bitrateAdjustMode;
    };
    
    PushFlowParamModel *frameRatio = [[PushFlowParamModel alloc] init];
    frameRatio.title = @"帧率";
    frameRatio.placeHolder = self.configPushFlowParamModel.fpsStr;
    frameRatio.infoText = @"fps";
    frameRatio.reuseId = PushFlowParamModelReuseCellInput;
    frameRatio.stringBlock = ^(NSString *message){
        self.configPushFlowParamModel.fpsStr = message;
        
        PushFlowParamModel *model = self.dataArray[6];
        model.placeHolder = self.configPushFlowParamModel.fpsStr;
    };
    
    PushFlowParamModel *IFrame = [[PushFlowParamModel alloc] init];
    IFrame.title = @"关键帧";
    IFrame.segmentTitleArray = _IFrameArr;
    IFrame.defaultValue = self.configPushFlowParamModel.keyInterval;
    IFrame.reuseId = PushFlowParamModelReuseCellSegment;
    IFrame.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.keyInterval = value;
        
        PushFlowParamModel *model = self.dataArray[7];
        model.defaultValue = self.configPushFlowParamModel.keyInterval;
    };
    
    PushFlowParamModel *codeType = [[PushFlowParamModel alloc] init];
    codeType.title = @"编码方式";
    codeType.segmentTitleArray = _codeTypeArr;
    codeType.defaultValue =  self.configPushFlowParamModel.videoCodecType;
    codeType.reuseId = PushFlowParamModelReuseCellSegment;
    codeType.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.videoCodecType = value;
        
        PushFlowParamModel *model = self.dataArray[8];
        model.defaultValue = self.configPushFlowParamModel.videoCodecType;
    };
    
    PushFlowParamModel *samplingRatio = [[PushFlowParamModel alloc] init];
    samplingRatio.title = @"采样率";
    samplingRatio.segmentTitleArray = _samplingRatioArr;
    samplingRatio.defaultValue = self.configPushFlowParamModel.audioSampleRate;
    samplingRatio.reuseId = PushFlowParamModelReuseCellSegment;
    samplingRatio.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.audioSampleRate = value;
        PushFlowParamModel *model = self.dataArray[9];
        model.defaultValue = self.configPushFlowParamModel.audioSampleRate;
    };
    
    PushFlowParamModel *audioCodeRatio = [[PushFlowParamModel alloc] init];
    audioCodeRatio.title = @"码率";
    audioCodeRatio.segmentTitleArray = _audioCodeRatioArr;
    if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 128) {
        audioCodeRatio.defaultValue = 3;
    }
    else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 64){
        audioCodeRatio.defaultValue = 2;
    }else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 48){
        audioCodeRatio.defaultValue = 1;
    }else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 32){
        audioCodeRatio.defaultValue = 0;
    }else{
        audioCodeRatio.defaultValue = 0;
    }
    audioCodeRatio.reuseId = PushFlowParamModelReuseCellSegment;
    audioCodeRatio.segmentBlock = ^(int value) {
        PushFlowParamModel *model = self.dataArray[10];
        model.defaultValue = value;
        if (value == 3) {
            self.configPushFlowParamModel.audioBitrateStr = @"128";
        }else if (value == 2){
            self.configPushFlowParamModel.audioBitrateStr = @"64";
        }else if (value == 1){
            self.configPushFlowParamModel.audioBitrateStr = @"48";
        }else if (value == 0){
            self.configPushFlowParamModel.audioBitrateStr = @"32";
        }else{
            self.configPushFlowParamModel.audioBitrateStr = @"32";
        }
    };
    
    PushFlowParamModel *audioCodeType = [[PushFlowParamModel alloc] init];
    audioCodeType.title = @"编码方式";
    audioCodeType.segmentTitleArray = _audioCodeTypeArr;
    audioCodeType.defaultValue = self.configPushFlowParamModel.audioCodecType;
    audioCodeType.reuseId = PushFlowParamModelReuseCellSegment;
    audioCodeType.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.audioCodecType = value;
        PushFlowParamModel *model = self.dataArray[11];
        model.defaultValue = self.configPushFlowParamModel.audioCodecType;
    };
    
    PushFlowParamModel *soundTrack = [[PushFlowParamModel alloc] init];
    soundTrack.title = @"声道";
    soundTrack.segmentTitleArray = _soundTrackArr;
    soundTrack.defaultValue = self.configPushFlowParamModel.audioChannelMode == JDLPushAudioChannel_1 ? 0 : 1;
    soundTrack.reuseId = PushFlowParamModelReuseCellSegment;
    soundTrack.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.audioChannelMode = value == 0 ? JDLPushAudioChannel_1 : JDLPushAudioChannel_2;
        PushFlowParamModel *model = self.dataArray[12];
        model.defaultValue = self.configPushFlowParamModel.audioChannelMode;
    };
    
    PushFlowParamModel *screenDirection = [[PushFlowParamModel alloc] init];
    screenDirection.title = @"设备模式";
    screenDirection.segmentTitleArray = _screenDirectionArr;
    screenDirection.defaultValue = self.configPushFlowParamModel.deviceOrientationMode;
    screenDirection.reuseId = PushFlowParamModelReuseCellSegment;
    screenDirection.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.deviceOrientationMode = value;
        PushFlowParamModel *model = self.dataArray[13];
        model.defaultValue = self.configPushFlowParamModel.deviceOrientationMode;
    };
    
    PushFlowParamModel *cameraType = [[PushFlowParamModel alloc] init];
    cameraType.title = @"镜头模式";
    cameraType.segmentTitleArray = _cameraTypeArr;
    cameraType.defaultValue = self.configPushFlowParamModel.cameraType;
    cameraType.reuseId = PushFlowParamModelReuseCellSegment;
    cameraType.segmentBlock = ^(int value) {
        self.configPushFlowParamModel.cameraType = value;
        PushFlowParamModel *model = self.dataArray[14];
        model.defaultValue = self.configPushFlowParamModel.cameraType;
    };
    
    PushFlowParamModel *backgroundPush = [[PushFlowParamModel alloc] init];
    backgroundPush.title = @"后台推流";
    backgroundPush.defaultSwitchValue = self.configPushFlowParamModel.isBackgroundPush;
    backgroundPush.reuseId = PushFlowParamModelReuseCellSwitch;
    backgroundPush.switchBlock = ^(BOOL open) {
        self.configPushFlowParamModel.isBackgroundPush = open;
        PushFlowParamModel *model = self.dataArray[15];
        model.defaultSwitchValue = self.configPushFlowParamModel.isBackgroundPush;
    };
    
    PushFlowParamModel *ispPicAndPicPush = [[PushFlowParamModel alloc] init];
    ispPicAndPicPush.title = @"画中画";
    ispPicAndPicPush.defaultSwitchValue = self.configPushFlowParamModel.isPicAndPicPush;
    ispPicAndPicPush.reuseId = PushFlowParamModelReuseCellSwitch;
    ispPicAndPicPush.switchBlock = ^(BOOL open) {
        self.configPushFlowParamModel.isPicAndPicPush = open;
        PushFlowParamModel *model = self.dataArray[16];
        model.defaultSwitchValue = self.configPushFlowParamModel.isPicAndPicPush;
    };
    
    PushFlowParamModel *relinkTime = [[PushFlowParamModel alloc] init];
    relinkTime.title = @"重连时长";
    relinkTime.infoText = @"ms";
    relinkTime.placeHolder = self.configPushFlowParamModel.reconnectIntervalStr;
    relinkTime.reuseId = PushFlowParamModelReuseCellInput;
    relinkTime.stringBlock = ^(NSString *message){
        self.configPushFlowParamModel.reconnectIntervalStr = message;
        PushFlowParamModel *model = self.dataArray[17];
        model.placeHolder = self.configPushFlowParamModel.reconnectIntervalStr ;
    };
    
    PushFlowParamModel *relinkNum = [[PushFlowParamModel alloc] init];
    relinkNum.title = @"重连次数";
    relinkNum.infoText = @"次";
    relinkNum.placeHolder = self.configPushFlowParamModel.reconnectCountStr;
    relinkNum.reuseId = PushFlowParamModelReuseCellInput;
    relinkNum.stringBlock = ^(NSString *message){
        self.configPushFlowParamModel.reconnectCountStr = message;
        PushFlowParamModel *model = self.dataArray[18];
        model.placeHolder = self.configPushFlowParamModel.reconnectCountStr;
    };
    
    //推流地址、
    //推流分辨率、采集分辨率、预览分辨率、码率、码率调节、帧率、关键帧、编码方式、（录制时去掉采集分辨率、预览分辨率）
    //采样率、码率、编码方式、声道、
    //设备模式、镜头模式、后台推流、画中画、重连时长、重连次数（录制时去掉设备模式、镜头模式、后台推流、画中画）
    self.dataArray = [NSMutableArray arrayWithObjects:pushFlowAddress,pushFlowResolutionRatio,collectionResolutionRatio,previewResolutionRatio,codeRatio,codeRatioAdjust,frameRatio,IFrame,codeType,samplingRatio,audioCodeRatio,audioCodeType,soundTrack,screenDirection,cameraType,backgroundPush,ispPicAndPicPush,relinkTime,relinkNum,nil];
}

//当页面上cell发生变化时，更新页面cell显示
- (void)reloadPushFlowParamModel
{
    for (PushFlowParamModel *model in self.dataArray){
        if ([model.title isEqualToString:@"推流地址"]){
            model.placeHolder =  self.configPushFlowParamModel.pushUrl;
        }
        else if ([model.title isEqualToString:@"推流分辨率"]) {
            model.defaultValue = self.configPushFlowParamModel.streamResolution;
        }
        else if ([model.title isEqualToString:@"采集分辨率"]) {
            model.defaultValue = self.configPushFlowParamModel.captureResolution;
        }
        else if ([model.title isEqualToString:@"预览分辨率"]) {
            model.defaultValue = self.configPushFlowParamModel.previewResolution;
        }
        else if ([model.title isEqualToString:@"码率"]) {
            if(model.segmentTitleArray.count == 4){ //视频码率
                model.defaultValue = self.configPushFlowParamModel.bitrateType;
            }else{ //音频
                if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 128) {
                    model.defaultValue = 3;
                }else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 64){
                    model.defaultValue = 2;
                }else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 48){
                    model.defaultValue = 1;
                }else if ([self.configPushFlowParamModel.audioBitrateStr intValue] == 32){
                    model.defaultValue = 0;
                }else{
                    model.defaultValue = 0;
                }
            }
        }
        else if ([model.title isEqualToString:@"码率调节"]) {
            model.defaultValue = self.configPushFlowParamModel.bitrateAdjustMode;
        }
        else if ([model.title isEqualToString:@"帧率"]) {
            model.placeHolder = self.configPushFlowParamModel.fpsStr;
        }
        else if ([model.title isEqualToString:@"关键帧"]) {
            model.defaultValue = self.configPushFlowParamModel.keyInterval;
        }
        else if ([model.title isEqualToString:@"编码方式"]) {
            if(model.segmentTitleArray.count == 3){ //视频码率
                model.defaultValue = self.configPushFlowParamModel.videoCodecType;
            }else{ //音频
                 model.defaultValue = self.configPushFlowParamModel.audioCodecType;
            }
        }
        else if ([model.title isEqualToString:@"采样率"]) {
            model.defaultValue = self.configPushFlowParamModel.audioSampleRate;
        }
        else if ([model.title isEqualToString:@"声道"]) {
            model.defaultValue = self.configPushFlowParamModel.audioChannelMode == JDLPushAudioChannel_1 ? 0 : 1;
        }
        else if ([model.title isEqualToString:@"设备模式"]) {
            model.defaultValue = self.configPushFlowParamModel.deviceOrientationMode;
        }
        else if ([model.title isEqualToString:@"镜头模式"]) {
            model.defaultValue = self.configPushFlowParamModel.cameraType;
        }
        else if ([model.title isEqualToString:@"后台推流"]) {
            model.defaultSwitchValue = self.configPushFlowParamModel.isBackgroundPush;
        }
        else if ([model.title isEqualToString:@"画中画"]) {
            model.defaultSwitchValue = self.configPushFlowParamModel.isPicAndPicPush;
        }
        else if ([model.title isEqualToString:@"重连时长"]) {
            model.placeHolder = self.configPushFlowParamModel.reconnectIntervalStr;
        }
        else if ([model.title isEqualToString:@"重连次数"]) {
            model.placeHolder = self.configPushFlowParamModel.reconnectCountStr;
        }else{}
    }
}

//点击开始推流前校验
- (BOOL)checkConfigValueIsReasonable
{
    // 定义归档文件路径
    NSString *pathdata = [NSString stringWithFormat:@"%@/Documents/JDCloud_Live_Config",NSHomeDirectory()];
    // 调用对象的归档方法 保存偏好设置
    [NSKeyedArchiver archiveRootObject:self.configPushFlowParamModel toFile:pathdata];
    
    //1.校验推流地址
    NSURL* pushUrl = [NSURL URLWithString:self.configPushFlowParamModel.pushUrl];
    if(!pushUrl || pushUrl == nil){
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"推流地址有误，请仔细检查推流地址并重新填写。"];
        return NO;
    }
    //2.校验 采集分辨率 >= 推流分辨率
    if (self.configPushFlowParamModel.captureResolution < self.configPushFlowParamModel.streamResolution) {
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"推流分辨率应不大于采集分辨率"];
        return NO;
    }
    //3、编码方式选择AAC_LD时，单声道，码率只能64
    if (self.configPushFlowParamModel.audioCodecType == JDLPushAudioEncoderMode_AAC_LD && self.configPushFlowParamModel.audioChannelMode == JDLPushAudioChannel_1 && (![self.configPushFlowParamModel.audioBitrateStr isEqualToString:@"64"])) {
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"编码方式选择AAC_LD，声道为单声道时，音频码率只能选择64"];
        return NO;
    }
    //4、编码方式选择AAC_LD时，双声道，码率只能128
    if (self.configPushFlowParamModel.audioCodecType == JDLPushAudioEncoderMode_AAC_LD && self.configPushFlowParamModel.audioChannelMode == JDLPushAudioChannel_2 && (![self.configPushFlowParamModel.audioBitrateStr isEqualToString:@"128"])) {
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"编码方式选择AAC_LD，声道为双声道时，音频码率只能选择128"];
        return NO;
    }
    //5、判断相机权限是否打开
    AVAuthorizationStatus status = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
    if (status == AVAuthorizationStatusRestricted || status == AVAuthorizationStatusDenied)
    {
        // 无权限
        [[[JDActionController alloc]init] showAlertControllerWithTitle:@"“京东视频云”想访问您的相机" message:@"京东视频云需要访问你的相机权限，开启后才能开始直播" preferredStyle:UIAlertControllerStyleAlert otherButtonTitles:@[@"不允许",@"好"] superViewController:self actionControllerBlock:^(NSString *buttonTitle) {
            if ([buttonTitle isEqualToString:@"好"]){
                [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
            }
        }];
        return NO;
    }
    return YES;
}

#pragma mark -
#pragma mark - 手势
- (void)addGesture{
    //点击手势 为了收键盘
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapGesture:)];
    tap.delegate = self;
    [self.paramTableView addGestureRecognizer:tap];
}

//手势点击事件
- (void)tapGesture:(UITapGestureRecognizer *)gesture{
    [self.paramTableView endEditing:YES];
}

//tableView开始滑动，键盘收起
-(void)scrollViewWillBeginDragging:(UIScrollView *)scrollView{
    [self.view endEditing:YES];
}

#pragma mark -
#pragma mark - 键盘通知
- (void)addNotifications {
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
    
}

- (void)keyboardWillShow:(NSNotification *)sender {
    
    CGRect keyboardFrame = [sender.userInfo[UIKeyboardFrameEndUserInfoKey] CGRectValue];
    
    //第一响应者 判断响应的哪个TextFeild
    UIWindow *keyWindow = [[UIApplication sharedApplication] keyWindow];
    UIView *firstResponder = [keyWindow performSelector:@selector(firstResponder)];
    
    //计算第一响应者相对于父视图的坐标
    CGRect newTFRect = [firstResponder convertRect:firstResponder.bounds toView:self.view];
    
    //在键盘下面的上移
    CGFloat dist = keyboardFrame.origin.y - (newTFRect.origin.y + newTFRect.size.height);
    if (dist < 0) {
        [UIView animateWithDuration:0.3f animations:^{
            CGRect frame = self.view.frame;
            frame.origin.y = - ( ScreenHeight - keyboardFrame.origin.y);
            self.view.frame = frame;
        }];
    }
}

- (void)keyboardWillHide:(NSNotification *)sender
{
    [[[UIApplication sharedApplication] keyWindow] endEditing:YES];
    [UIView animateWithDuration:0.3f animations:^{
        self.view.frame = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
