//
//  JDStartPushFlowViewController.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/7.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDStartPushFlowViewController.h"
#import "StartPushFlowView.h"
#import "JDAudioCallViewController.h"
#import "JDPhotoManager.h"
#import "JDPushFlowContextProtocol.h"
#import "NavigationItemTitleView.h"
#import "JDLiveEndViewController.h"
#import "JDCloudVideo.h"
#import "MBProgressHUD+Help.h"
#import "RTCTokenTools.h"

#define weakObj(o) __weak typeof(o) o##Weak = o;

//直播状态变化通知
NSString *const JDLPushStreamStateChangeNotification=@"JDLPushStreamStateChangeNotification";
//直播错误发生通知
NSString *const JDLPushStreamErrorNotification=@"JDLPushStreamErrorNotification";
//直播网络状态改变通知
NSString *const JDLNetworkStatusChangeNotification=@"JDLNetworkStatusChangeNotification";
//直播录制状态改变通知
NSString *const JDLMediaRecordStateChangeNotification=@"JDLMediaRecordStateChangeNotification";
//推流模式改变通知
NSString *const JDLPushStreamModeChangeNotification=@"JDLPushStreamModeChangeNotification";

@interface JDStartPushFlowViewController ()<JDPushFlowContextProtocol,JDLiveDelegate,JDLinkPKDelegate, AVCaptureMetadataOutputObjectsDelegate>

@property (nonatomic, strong) NavigationItemTitleView *navigationItemTitleView; //导航上面按钮View

@property (nonatomic, strong) StartPushFlowView *startPushFlowView; //美颜静音开始推流view

@property (nonatomic, strong) ConfigPushFlowParamModel *configPushFlowParamModel;    //直播推流参数model

@property (nonatomic, strong) UIView* currentView; //当前弹出来的View

@property (nonatomic, assign) NSInteger pushStreamMode; //纯音频推流设置  0普通 1纯音频+背景图

@property (nonatomic, strong) UIView *playerGustureView; //画中画手势view

@property (nonatomic, assign) CGPoint loc_in;//

@property (nonatomic, assign) CGFloat currentVideoZoomFactor; //当前相机焦距缩放因子

@property (nonatomic, assign) CGPoint focusPoint;  //对焦点

@property (nonatomic, strong) CALayer *focusLayer; //对焦layer

@property (nonatomic, assign) BOOL focusLayerIsDisappearing; //对焦layer框 是否消失

@property (nonatomic, strong) AVCaptureDevice *currentdevice; //当前设备

@property (nonatomic, assign) BOOL lastFlashLightState;                    //进入后台前，闪光灯的状态

@property (nonatomic, strong)JDCloudPlayer* iplayer;

@property (nonatomic,copy)NSString* RTC_PUSH_URL;
@property (nonatomic,copy)NSString* RTC_PLAY_URL;

//@property (nonatomic, assign) JDLBGMusicPlayerState lastAudioState;        //进入后台前，音频的状态

@end


@implementation JDStartPushFlowViewController
@synthesize live = _live;
@synthesize linkLive = _linkLive;
@synthesize previewView = _previewView;
@synthesize isNeedFloatPush = _isNeedFloatPush;
@synthesize isOrientationLock = _isOrientationLock;
@synthesize currentSelectAudioIndex = _currentSelectAudioIndex;
@synthesize strMusicUrl = _strMusicUrl;
@synthesize isAutoFocus = _isAutoFocus;
@synthesize isMirroredPush = _isMirroredPush;

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [[UIDevice currentDevice] endGeneratingDeviceOrientationNotifications];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupSubview];
    [self setupData];
    [self dismissDetailVC];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.hidden = YES;
    // 禁止系统屏保
    [[UIApplication sharedApplication] setIdleTimerDisabled: YES];
    //监听进入前台 后台
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(becomeActive)
                                                 name:UIApplicationDidBecomeActiveNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(resignActive)
                                                 name:UIApplicationWillResignActiveNotification
                                               object:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    // 恢复系统屏保
    [[UIApplication sharedApplication] setIdleTimerDisabled: NO];
}

//前一页传过来数据
- (void)resultConfigPushFlowParamModel:(ConfigPushFlowParamModel *)model{
    self.configPushFlowParamModel = model;
}

#pragma mark --
#pragma mark -- 初始化UI
- (void)setupSubview
{
    [self.view addSubview:self.previewView];
    [self.view addSubview:self.startPushFlowView];
    [self.view addSubview:self.navigationItemTitleView];
    
    //手势调整焦距
    UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc]initWithTarget:self action:@selector(pinchDetected:)];
    [self.view addGestureRecognizer:pinch];
    
    if (![UIDevice currentDevice].generatesDeviceOrientationNotifications) {
        [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    }
    
    //若系统锁定打开，是收不到通知
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleDeviceOrientationChange:)
                                                 name:UIDeviceOrientationDidChangeNotification object:nil];
}

#pragma mark-
#pragma mark- 初始化数据Live
-(void)setupData
{
    _isOrientationLock = NO;
    _currentSelectAudioIndex = 0;
    _isMirroredPush = NO;
    _isAutoFocus = YES;
    self.focusLayerIsDisappearing = NO;
    self.lastFlashLightState = NO;
    
    JDLConfig* config = [JDLConfig defaultConfig];
    
    ConfigPushFlowParamModel *cm = self.configPushFlowParamModel;
    config.captureResolution = cm.captureResolution;
    config.pushResolution = cm.streamResolution;
    config.videoPerformanceMode = cm.performanceMode;
    config.videoLiveScene = cm.liveScence;
    NSArray *bitrateArray = @[@1000, @800, @600, @400];
    int bitrate = [bitrateArray[cm.bitrateType] intValue];
    if(self.configPushFlowParamModel.bitrateAdjustMode == 1){
        config.videoBitrate = bitrate;
        config.videoBitrateMin = bitrate * 0.5;
        config.videoBitrateMin = 0;
        config.videoBitrateMax = bitrate * 1.5;
    }
    else{
        config.videoBitrate = bitrate;
        config.videoBitrateMin = bitrate;
        config.videoBitrateMax = bitrate;
    }
    config.fps = [cm.fpsStr intValue];
    NSArray<NSNumber*> *keyIntervalArray = @[@1, @2, @3, @4, @5];
    config.keyIntervalTime = [keyIntervalArray[cm.keyInterval] intValue];
    
    config.videoEncoderMode = JDLPushVideoEncoderMode_AUTO;
    if (cm.videoCodecType == 1){
        if(cm.videoCodecFormat == 0){
            config.videoEncoderMode = JDLPushVideoEncoderMode_X264;
        }
        else if(cm.videoCodecFormat == 1){
            config.videoEncoderMode = JDLPushVideoEncoderMode_X265;
        }
    }
    else if (cm.videoCodecType == 2){
        if(cm.videoCodecFormat == 0){
            config.videoEncoderMode = JDLPushVideoEncoderMode_VT264;
        }
        else if(cm.videoCodecFormat == 1){
            config.videoEncoderMode = JDLPushVideoEncoderMode_VT265;
        }
    }
    
    NSArray<NSNumber*> *audioSampleRateArray = @[@32000, @44100, @48000];
    config.audioSampleRate = [[audioSampleRateArray objectAtIndex:cm.audioSampleRate] intValue];
    config.audioBitrate = [cm.audioBitrateStr intValue];
    config.audioEncoderMode = cm.audioCodecType;
    config.audioChannel = cm.audioChannelMode;
    config.deviceOrientationMode = cm.deviceOrientationMode;
    //if(cm.cameraType < 2){
        config.cameraType = cm.cameraType;
    //}
    
    config.isBackgroundPush = cm.isBackgroundPush;
    config.reconnectInterval = [cm.reconnectIntervalStr intValue]/1000.0;
    config.maxReconnectCount = [cm.reconnectCountStr intValue];
    self.live = [[JDLive alloc] initWithConfig:config];
    //self.linkLive = [[JDLinkPK alloc] initWithLive:_live JDLinkPKConfig:nil];
    self.linkLive.delegate = self;
    [_live setLiveDelegate:self];
    NSURL* url = [NSURL URLWithString:cm.pushUrl];
    if (!url || url == nil) {
         [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"推流地址错误，请仔细检查推流地址并重新填写。"];
    }else{
         [_live setPushStreamUrl:[NSURL URLWithString:cm.pushUrl]];
    }
    [_live startPreview:self.previewView];
    if (cm.isPicAndPicPush) {
        [self setupVideoGesture];
        _isOrientationLock = YES;
    }
    
     __weak typeof(self) weakSelf = self;
    [_live setBGMusicFinishBlock:^(){
        if(weakSelf.strMusicUrl && ![weakSelf.strMusicUrl isEqualToString:@""]){
            [weakSelf.live startBGMusic:weakSelf.strMusicUrl];
        }
    }];
    
    if ([_live getCurrentDevicePosition] == AVCaptureDevicePositionFront) {
        [_live setPushStreamHFlip:!self.isMirroredPush];
    }else{
        [_live setPushStreamHFlip:NO];
    }
    
}
#pragma mark - 上下文协议 LinkLiveDelegate
-(void)onJDLinkPK:(JDLinkPK*)livePK withEventId:(int)eventId withParam:(NSDictionary*)param
{
    NSLog(@"onLinkLive: eventId = %d ,param = %@",eventId,param);
    switch (eventId) {
        //连麦PK成功
        case 2000:
        {
            self.previewView.frame = CGRectMake(0, SafeAreaNavTwoHeight, ScreenWidth, ScreenWidth*16/9/2);
            [self.live adjustPreviewSize];
            
            [MBProgressHUD removeHUDFromView:[UIApplication sharedApplication].keyWindow];
            [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"连麦PK成功"];
        }
            break;
            
        default:
            break;
    }
}

-(void)onJDLinkPK:(JDLinkPK*)livePK withErrorCode:(int)errorCode withMessage:(NSString*)message
{
    NSLog(@"onLinkLive: errorCode = %d ,message = %@",errorCode,message);
    [MBProgressHUD removeHUDFromView:[UIApplication sharedApplication].keyWindow];
    [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:[NSString stringWithFormat:@"连麦错误：%@",message]];
    
    [self onLinkRoomAction2:nil];
    
}

-(NSString*)getTokenWithRoomId:(NSNumber*)roomId userId:(NSString*)userId nonce:(NSString*)nonce timestamp:(NSString*)timestamp
{
    return nil;
}

#pragma mark -
#pragma mark - 上下文协议 JDPushFlowContextProtocol

- (void)onLinkRoomAction1:(NSDictionary*)dic
{
    weakObj(self);
    NSString* roomId = dic[@"TF1"];
    NSString* localUserId = dic[@"TF2"];
    NSString* remoteUserId = nil;
    [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeIndeterminate withText:@"开始连麦PK"];
    JDLinkPKConfig* pkConfig = [[JDLinkPKConfig alloc] init];
    pkConfig.envType = 1;
    pkConfig.roomId = @([roomId integerValue]);
    pkConfig.userId = localUserId;
    pkConfig.appId = @"192bc3400174019265a7b1ad1ea7c6c7";
    pkConfig.nonce = @"AK-816a9a11-6ee2-4a68-a9e0-8b06f3abcd4f";
    pkConfig.timestamp = @"1628145604000";
    pkConfig.token = [RTCTokenTools getTokenWithRoomId:roomId appId:pkConfig.appId appKey:@"SadW4EIcFmhmA7ixgK39MNegUFj0LnAkYEPlxlykexVezqsXS2Q1VOMed88ES4GxTP0Jiqv3pR/bCNE1lcrpA==" timestamp:[pkConfig.timestamp integerValue] userId:pkConfig.userId nonce:pkConfig.nonce];
    
    self.linkLive = [[JDLinkPK alloc] initWithLive:_live JDLinkPKConfig:pkConfig];
    self.linkLive.delegate = self;
    [self.linkLive startPK:remoteUserId];
    
//    weakObj(self)
//    NSLog(@"onLinkRoomAction");
//    if(self.iplayer == nil){
//        self.iplayer = [[JDCloudPlayer alloc] init];
//        JDCloudOptionModel* optionModel = [[JDCloudOptionModel alloc] init];
//        optionModel.isSetAudioSessionCategoryPlayAndRecord = YES;
//        NSURL* url = [NSURL URLWithString:@"https://abnormal-video-list.oss.cn-north-1.jcloudcs.com/yds01.mp4"];
//        [_iplayer setVideoWithURL:url withPlayerOption:optionModel];
//        [_iplayer prepareToPlay];
//        _iplayer.isOnlyAudio = YES;
//        _iplayer.shouldAutoplay = YES;
//        [_live AddMixVideoStreamWithIndex:0];
//        [_live AddMixAudioStreamWithIndex:0];
//        _iplayer.videoRawDataBlock = ^(CMSampleBufferRef sampleBuffer) {
//            [selfWeak.live inputMixVideoSampleBuffer:sampleBuffer streamIndex:0];
//        };
//        _iplayer.audioRawDataBlock = ^(CMSampleBufferRef sampleBuffer) {
//            [selfWeak.live inputMixAudioSampleBuffer:sampleBuffer streamIndex:0];
//        };
//
//
//    }
//    else{
//        [_iplayer stop];
//        [_iplayer shutdown];
//        self.iplayer = nil;
//        [_live removeVideoStreamWithIndex:0];
//        [_live removeAudioStreamWithIndex:0];
//
//    }
}

- (void)onLinkRoomAction2:(NSDictionary*)dic
{
    [self.linkLive stopPK];
    self.linkLive = nil;
    self.previewView.frame = CGRectMake(0, SafeAreaNavTwoHeight, ScreenWidth, ScreenWidth*16/9);
    [self.live adjustPreviewSize];
}


//显示底部弹出view
- (void)showBottomPopupView:(UIView*)bottomView
{
    if(self.currentView){
        [self.currentView removeFromSuperview];
    }
    self.currentView = bottomView;
    [self.view addSubview:self.currentView];
}

//关闭底部弹出view
- (void)closeBottomPopupView
{
    [self.currentView removeFromSuperview];
}
    
//推到下一个VC
- (void)pushViewController:(UIViewController*)VC
{
    [self.navigationController pushViewController:VC animated:YES];
}
    
//关闭VC
- (void)popViewController
{
    [self.navigationController popViewControllerAnimated:YES];
}

static __strong UIViewController* s_save_vc = nil;
static __strong UINavigationController* s_save_nav = nil;
//临时隐藏VC
- (void)hideViewController
{
    s_save_vc = self;
    s_save_nav = self.navigationController;
    [self.navigationController popViewControllerAnimated:NO];
}

//显示VC
- (void)showViewController
{
    [s_save_nav pushViewController:s_save_vc animated:NO];
    s_save_vc = nil;
}

//销毁VC
- (void)releaseViewController
{
    s_save_vc = nil;
    s_save_nav = nil;
}

#pragma mark -
#pragma mark - JDLiveDelegate
/**
 推流状态发生改变
 */
- (void)onPushStreamStateChange:(JDLPushStreamState)pushStreamState
{
    [[NSNotificationCenter defaultCenter] postNotificationName:JDLPushStreamStateChangeNotification object:[NSNumber numberWithInt:pushStreamState] userInfo:nil];
}

/**
 推流错误回调
 */
- (void)onPushStreamError:(JDLStreamErrorCode)errorCode
{
    [[NSNotificationCenter defaultCenter] postNotificationName:JDLPushStreamErrorNotification object:[NSNumber numberWithInt:errorCode] userInfo:nil];
}

/**
 网络健康状态发生改变
 */
- (void)onNetworkStatusChange:(JDLNetworkStateCode)networkStateCode
{
    [[NSNotificationCenter defaultCenter] postNotificationName:JDLNetworkStatusChangeNotification object:[NSNumber numberWithInt:networkStateCode] userInfo:nil];
}

/**
 视频录制状态改变
 */
-(void)onMediaRecordStateChange:(JDLMediaRecordState)mediaRecordState
{
    [[NSNotificationCenter defaultCenter] postNotificationName:JDLMediaRecordStateChangeNotification object:[NSNumber numberWithInt:mediaRecordState] userInfo:nil];
}

/**
 推流模式改变
 */
-(void)onPushStreamModeChange:(JDLPushStreamMode)pushStreamMode
{
    [[NSNotificationCenter defaultCenter] postNotificationName:JDLPushStreamModeChangeNotification object:[NSNumber numberWithInt:pushStreamMode] userInfo:nil];
}

#pragma mark -
#pragma mark - 其他功能函数
//销毁中间完成控制器JDLiveEndViewController
- (void)dismissDetailVC
{
    NSMutableArray *marr = [[NSMutableArray alloc]initWithArray:self.navigationController.viewControllers];
    for (UIViewController *vc in marr) {
        if ([vc isKindOfClass:[JDLiveEndViewController class]]) {
            [marr removeObject:vc];
            break;
        }
    }
    self.navigationController.viewControllers = marr;
}

//进入前台
- (void)becomeActive{
    //根据进入后台前状态 闪光灯赋值
    AVCaptureDevice *device = [_live getCurrentCaptureCamera];
    if (self.lastFlashLightState && device.torchMode == AVCaptureTorchModeOff) {
        [self.live toggleFlashLight];
    }

    /*
    //根据进入后台前状态 音频赋值
    if (self.lastAudioState == JDLBGMusicPlayerState_Playing) {
        [self.live resumeBGMusic];
    }*/
    
}

//进入后台
- (void)resignActive{
    //闪光灯状态
    AVCaptureDevice *device = [_live getCurrentCaptureCamera];
    if([self.live isFlashLightSupported] && device.torchMode == AVCaptureTorchModeOn){
        self.lastFlashLightState = YES;
    }else{
        self.lastFlashLightState = NO;
    }

    /*
    //不是后台推流 音频暂停
    self.lastAudioState = [self.live getBGMusicState];
    if (!self.configPushFlowParamModel.isBackgroundPush) {
        [self.live pauseBGMusic];
    }*/
}

//设置画中画点击互换手势
- (void)setupVideoGesture
{
//    NSString *picInPicURL = HTTP_Video_PICINPICPUSH_URL;
//    NSURL *url = [NSURL URLWithString:picInPicURL];
//    JDCloudOptionModel *optionModel = [[JDCloudOptionModel alloc] init];
//    optionModel.videoFormat =  JDCloudPlayerVideoFormatMP4;
//    optionModel.isSetAudioSessionCategoryPlayAndRecord = YES;
//    [_live setPlayUrl:url playerOptionModel:optionModel];
//    weakObj(self);
//    _live.playerSizeCallback = ^(CGSize size){
//        if (size.width > size.height) {
//            CGFloat width = 200;
//            CGFloat height = width * size.height/size.width;
//            [selfWeak.live setPlayerViewRect:CGRectMake(selfWeak.previewView.frame.size.width - width, 64 + SafeAreaBottomHeight, width, height)];
//        } else {
//            CGFloat height = 200;
//            CGFloat width = height * size.width/size.height;
//            [selfWeak.live setPlayerViewRect:CGRectMake(selfWeak.previewView.frame.size.width - width, 64 + SafeAreaBottomHeight, width, height)];
//        }
//    };
//
//    _live.playerInPushLiveFinishedBlock = ^(BOOL isVodVideo) {
////        if (isVodVideo) {
////            [selfWeak.live replay];
////        } else {
//            [selfWeak.live stopPlayerVideo];
////        }
//    };
}

//收键盘
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}

#pragma mark -
#pragma mark - 推流方向处理
- (void)updatePushStreamOrientation
{
    UIDeviceOrientation deviceOrientation = [UIDevice currentDevice].orientation;
    switch (deviceOrientation) {
            
        case UIDeviceOrientationLandscapeLeft:
        case UIDeviceOrientationLandscapeRight:
        case UIDeviceOrientationPortrait:
        {
            if (!_isOrientationLock && self.configPushFlowParamModel.deviceOrientationMode == 2) {
                //处理屏幕旋转
                [self.live setVideoPushStreamOrientation:(UIInterfaceOrientation)deviceOrientation];
            }
        }
            break;
        default:
            break;
    }
}
//设备方向改变的处理
- (void)handleDeviceOrientationChange:(NSNotification *)notification{
    
    [self updatePushStreamOrientation];
}

-(void)setIsOrientationLock:(BOOL)bLock
{
    _isOrientationLock = bLock;
    [self updatePushStreamOrientation];
    
}

#pragma mark -
#pragma mark - Crazy
//设置其他功能View
- (StartPushFlowView *)startPushFlowView {
    
    if (!_startPushFlowView) {
        _startPushFlowView = [[StartPushFlowView alloc] initWithFrame:CGRectMake(0, SafeAreaNavTwoHeight, ScreenWidth, ScreenWidth*16/9) withContext:self];
        
        _startPushFlowView.backgroundColor = [UIColor clearColor];
    }
    return _startPushFlowView;
}

//设置预览屏幕
- (UIView *)previewView {
    
    if (!_previewView) {
        _previewView = [[UIView alloc] init];
        _previewView.backgroundColor = [UIColor clearColor];
        _previewView.frame = CGRectMake(0, SafeAreaNavTwoHeight, ScreenWidth, ScreenWidth*16/9);
    }
    return _previewView;
}

//导航栏
- (NavigationItemTitleView *)navigationItemTitleView
{
    if (!_navigationItemTitleView) {
        _navigationItemTitleView = [[NavigationItemTitleView alloc] initWithContext:self];
        _navigationItemTitleView.translatesAutoresizingMaskIntoConstraints = false;
    }
    return _navigationItemTitleView;
}

//左边返回按钮隐藏
- (void)initNavgationcontrollerLeftButton{
    self.navigationItem.leftBarButtonItem = nil;
    self.navigationItem.hidesBackButton = YES;
}

#pragma mark -
#pragma mark - 对焦及焦距
//焦距缩放
- (void)pinchDetected:(UIPinchGestureRecognizer *)recognizer
{
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.currentVideoZoomFactor = [self.live getVideoZoomFactor];
    }
    CGFloat zoomFactor = self.currentVideoZoomFactor * recognizer.scale;//当前触摸缩放因子*坐标比例
    [self.live setVideoZoomFactor:zoomFactor];
}

//点击手动对焦
- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint current = [touch locationInView:self.view];
    
    CGPoint point = [self convertToPoint:current];
    [self cameraViewFocusBegin:self.view pointInDevice:point pointInView:current];

    [self.live setCaptureFocusPoint:point FocusMode:AVCaptureFocusModeContinuousAutoFocus];
    [self.live setCaptureExposureAtPoint:point];
    [self.live setCaptureWhiteBalance];
}

//聚焦开始
- (void)cameraViewFocusBegin:(UIView *)cameraView pointInDevice:(CGPoint)pointInDevice pointInView:(CGPoint)pointInView {
    if ([self.live getCurrentDevicePosition] == AVCaptureDevicePositionFront) return;
    if (self.focusLayerIsDisappearing) return;
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(focusLayerDisappearAnimation) object:nil];
    self.focusPoint = pointInDevice;
    [self focusLayerAnimation:pointInView];
    
}

//聚焦结束
- (void)cameraViewFocusEnd:(UIView *)cameraView pointInDevice:(CGPoint)pointInDevice {
    BOOL isSamePoint = __CGPointEqualToPoint(self.focusPoint, pointInDevice);
    BOOL isZeroPoint = ((0 == self.focusPoint.x) && (0 == self.focusPoint.y));
    if (isSamePoint && !isZeroPoint) {
        [self performSelector:@selector(focusLayerDisappearAnimation) withObject:nil afterDelay:1.5];
    }
    NSLog(@"对焦完毕时的point----%@-------%@", NSStringFromCGPoint(pointInDevice), NSStringFromCGPoint(self.focusPoint));
}

//对焦开始动画
- (void)focusLayerAnimation:(CGPoint)point {
    [self.focusLayer removeFromSuperlayer];
    [self.focusLayer removeAllAnimations];
    [CATransaction begin];
    [CATransaction setDisableActions:YES];
    self.focusLayer.position = point;
    [CATransaction commit];
    [self.view.layer addSublayer:self.focusLayer];
    
    CABasicAnimation *baseAnimation = [CABasicAnimation animationWithKeyPath:@"transform"];
    baseAnimation.fromValue = [NSValue valueWithCATransform3D:CATransform3DMakeScale(2.0, 2.0, 1.0)];
    baseAnimation.toValue = [NSValue valueWithCATransform3D:CATransform3DMakeScale(1.0, 1.0, 1.0)];
    baseAnimation.timingFunction = [CAMediaTimingFunction functionWithName: kCAMediaTimingFunctionEaseOut];
    baseAnimation.duration = 0.25;
    baseAnimation.removedOnCompletion = NO;
    baseAnimation.fillMode = kCAFillModeForwards;
    [self.focusLayer addAnimation:baseAnimation forKey:@"focusAnimation"];
    
    [self performSelector:@selector(focusLayerDisappearAnimation) withObject:nil afterDelay:2.5];
}

//对焦完成动画
- (void)focusLayerDisappearAnimation {
    if (!self.focusLayer) return;
    self.focusLayerIsDisappearing = YES;
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(focusLayerDisappearAnimation) object:nil];
    [self.focusLayer removeAllAnimations];
    CABasicAnimation *baseAnimation = [CABasicAnimation animationWithKeyPath:@"opacity"];
    baseAnimation.fromValue = [NSNumber numberWithFloat:1.0];
    baseAnimation.toValue = [NSNumber numberWithFloat:0.0];
    baseAnimation.timingFunction = [CAMediaTimingFunction functionWithName: kCAMediaTimingFunctionEaseOut];
    baseAnimation.duration = 0.25;
    baseAnimation.removedOnCompletion = NO;
    baseAnimation.fillMode = kCAFillModeForwards;
    [self.focusLayer addAnimation:baseAnimation forKey:@"focusCompleteAnimation"];
    [self performSelector:@selector(animationDidStop) withObject:nil afterDelay:0.01];
}

//对焦动画结束
- (void)animationDidStop {
    self.focusLayerIsDisappearing = NO;
    [self.focusLayer removeFromSuperlayer];
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(focusLayerDisappearAnimation) object:nil];
}

//crazy 对焦框
- (CALayer *)focusLayer {
    if (!_focusLayer) {
        _focusLayer = [CALayer layer];
        _focusLayer.cornerRadius = 2;
        _focusLayer.borderWidth = 1;
        _focusLayer.borderColor = [UIColor colorWithRed:(249/255.0) green:(203/255.0) blue:(52/255.0) alpha:1.0].CGColor;
        _focusLayer.anchorPoint = CGPointMake(0.5, 0.5);
        _focusLayer.bounds = CGRectMake(0, 0, 75, 75);
    }
    return _focusLayer;
}

//对焦 将UI的坐标转换成相机坐标
- (CGPoint)convertToPoint:(CGPoint)pt
{
    CGSize fsz = self.view.frame.size;
    CGSize sz = [self.live getCaptureDimensions];
    CGPoint point = pt;
    CGFloat ar = sz.height / sz.width;
    CGFloat vr = fsz.width / fsz.height;
    CGFloat xc = .5f;
    CGFloat yc = .5f;
    
    if (vr > ar) {
        CGFloat y2 = fsz.height;
        CGFloat x2 = fsz.height * ar;
        CGFloat x1 = fsz.width;
        CGFloat bb = (x1 - x2) / 2;
        if (point.x >= bb && point.x <= bb + x2) {
            xc = point.y / y2;
            yc = 1.f - ((point.x - bb) / x2);
        }
    }else {
        CGFloat y2 = fsz.width / ar;
        CGFloat y1 = fsz.height;
        CGFloat x2 = fsz.width;
        CGFloat bb = (y1 - y2) / 2;
        if (point.y >= bb && point.y <= bb + y2) {
            xc = ((point.y - bb) / y2);
            yc = 1.f - (point.x / x2);
        }
    }
    return CGPointMake(xc, yc);
}


#pragma mark -
#pragma mark - 暂时不用勿删
//设置画中画点击互换手势
- (void)setupVideoGestureTwo
{
    //    _playerGustureView = [[UIView alloc] initWithFrame:CGRectZero];
    //    _playerGustureView.backgroundColor = [UIColor redColor];
    //    _playerGustureView.alpha = 0.3;
    //
    //    UITapGestureRecognizer *singleClick = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(singleClickGuestTap:)];
    //
    //    [_playerGustureView addGestureRecognizer:singleClick];
    //    [self.view addSubview:_playerGustureView];
    //    weakObj(self);
    //    _live.playerRectCallback = ^(CGRect rect){
    //        selfWeak.playerGustureView.frame = rect;
    //    };
    //    weakObj(_live);
    //    _live.playerSizeCallback = ^(CGSize size) {
    //        if (size.width > size.height) {
    //            CGPoint leftTopPoint = CGPointMake(self.view.frame.size.width - 170, 64 + SafeAreaBottomHeight);
    //            _liveWeak.playerLeftTop = leftTopPoint;
    //            _liveWeak.playerSize = CGSizeMake(170, 170 * size.height/size.width);
    //            selfWeak.playerGustureView.frame = CGRectMake(leftTopPoint.x, leftTopPoint.y, 170, 170 * size.height/size.width);
    //        } else {
    //            CGPoint leftTopPoint = CGPointMake(self.view.frame.size.width - 170 * size.width/size.height, 64 + SafeAreaBottomHeight);
    //            _liveWeak.playerLeftTop = leftTopPoint;
    //            _liveWeak.playerSize = CGSizeMake(170 * size.width/size.height, 170);
    //            selfWeak.playerGustureView.frame = CGRectMake(leftTopPoint.x, leftTopPoint.y, 170 * size.width/size.height, 170);
    //        }
    //    };
    //    NSURL *url = [NSURL URLWithString:@"http://weblive.hebtv.com/live/hbws_bq/index.m3u8"];
//    NSString *picInPicURL = HTTP_Video_PICINPICPUSH_URL;
//    NSURL *url = [NSURL URLWithString:picInPicURL];
//    JDCloudOptionModel *optionModel = [[JDCloudOptionModel alloc] init];
//    optionModel.videoFormat =  JDCloudPlayerVideoFormatRtmp;
//    [_live setPlayUrl:url playerOptionModel:optionModel];
//    //    [_live setRtmpMaxLiveBufferTime:5];
//    weakObj(self);
//    _live.playerSizeCallback = ^(CGSize size){
//        if (size.width > size.height) {
//            CGFloat width = 200;
//            CGFloat height = width * size.height/size.width;
//            [selfWeak.live setPlayerViewRect:CGRectMake(selfWeak.previewView.frame.size.width - width, 64 + SafeAreaBottomHeight, width, height)];
//        } else {
//            CGFloat height = 200;
//            CGFloat width = height * size.width/size.height;
//            [selfWeak.live setPlayerViewRect:CGRectMake(selfWeak.previewView.frame.size.width - width, 64 + SafeAreaBottomHeight, width, height)];
//        }
//    };
//
//    _live.playerInPushLiveFinishedBlock = ^(BOOL isVodVideo) {
//        if (isVodVideo) {
//            [selfWeak.live replay];
//        } else {
//            [selfWeak.live stopPlayerVideo];
//        }
//    };
}

//-(void)singleClickGuestTap:(UITapGestureRecognizer *)gesture
//{
//    [_live switchPlayerAndCapture];
//}
//
//-(void)movePlayer:(UIPanGestureRecognizer *)gesture
//{
//    int width = [UIScreen mainScreen].bounds.size.width;
//    int height = [UIScreen mainScreen].bounds.size.height;
//
//    CGFloat playerGustureViewWidth = _playerGustureView.frame.size.width;
//    CGFloat playerGustureViewHeight = _playerGustureView.frame.size.height;
//
//
//    CGPoint point = [gesture locationInView:self.view];
//    if (gesture.state == UIGestureRecognizerStateBegan) {
//        _loc_in = [gesture locationInView:_playerGustureView];
//    }
//    // 坐标矫正，避免画面超出屏幕_playerGustureView
//    CGFloat x;
//    CGFloat y;
//    if (playerGustureViewWidth + point.x - _loc_in.x >= width) {
//        x = width - playerGustureViewWidth * 0.5;
//    }else if (point.x - _loc_in.x <= 0) {
//        x = playerGustureViewWidth * 0.5;
//    }else {
//        x = playerGustureViewWidth * 0.5 - _loc_in.x + point.x;
//    }
//
//    if (playerGustureViewHeight - _loc_in.y + point.y >= height) {
//        y = height - playerGustureViewHeight * 0.5;
//    }else if (point.y - _loc_in.y  - _navigationItemTitleView.frame.size.height <= 0){
//        y = playerGustureViewHeight * 0.5  + _navigationItemTitleView.frame.size.height;
//    }else {
//        y = playerGustureViewHeight * 0.5 - _loc_in.y + point.y;
//    }
//
//    [_live setVideoPlayerRect:CGRectMake(x - playerGustureViewWidth * 0.5,y - playerGustureViewHeight * 0.5, playerGustureViewWidth, playerGustureViewHeight)];
//    _playerGustureView.frame = CGRectMake(x - playerGustureViewWidth * 0.5,y - playerGustureViewHeight * 0.5, playerGustureViewWidth, playerGustureViewHeight);
//
//
//}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end

