//
//  AudioChangeToneView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/9/27.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "AudioChangeToneView.h"
#import "MoreSettingView.h"

@interface AudioChangeToneView()

@property (nonatomic, strong) NSArray<UIButton *> *totalLevelButtons;        //总共多少个等级
@property (nonatomic, strong) UIButton *selectedButton;                      //选中的Button

@property (nonatomic, weak) id<JDPushFlowContextProtocol> context;

@end

@implementation AudioChangeToneView

- (instancetype)initWithContext:(id<JDPushFlowContextProtocol>)context
{
    self = [super initWithFrame:CGRectMake(0, ScreenHeight - 140 - SafeAreaBottomHeight, ScreenWidth, 140 + SafeAreaBottomHeight)];
    if (self) {
        self.context = context;
        [self setupSubviews];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子视图
- (void)setupSubviews
{
    //背景色
    self.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);
    
    [self addSubViews];
    
    /** 创建几个等级button -3 -2 -1 0 1 2 3 **/
    self.totalLevelButtons  = [self creatButtonsWithCount:7];
    
    //选中的button
    int index = [_context.live getAudioEffectCustomFlag] + 3;
    if(index>6){
        index =3;
    }
    self.selectedButton = self.totalLevelButtons[index];
    self.selectedButton.selected = YES;
}

//子试图
- (void)addSubViews
{
    //title
    UILabel* label = [[UILabel alloc]initWithFrame:CGRectMake(0, 17, ScreenWidth, 21)];
    label.textAlignment = NSTextAlignmentCenter;
    label.font = [UIFont systemFontOfSize:15 * SCALE_FONT];
    label.textColor = [UIColor whiteColor];
    label.text = @"音频升降调";
    [self addSubview:label];
    
    //返回
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setFrame:CGRectMake(0, 16, 44, 44)];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateNormal];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateHighlighted];
    [leftBtn addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:leftBtn];
    [leftBtn sizeToFit];
}

//button
- (NSArray<UIButton *> *)creatButtonsWithCount:(NSInteger)count {
    
    CGFloat buttonWidth = (ScreenWidth - 30)/7;
    
    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:count];
    for (NSInteger index = 0; index < count; index++) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(15 + index * buttonWidth, 65, buttonWidth, 34);
        [button setTitle:@(index-3).stringValue forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [button setTitleColor:[JDStaticTools  colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        button.backgroundColor = RGBACOLOR(0, 0, 0, 0.8);
        [button addTarget:self action:@selector(levelButtonOnClick:) forControlEvents:UIControlEventTouchUpInside];
        button.tag = index;
        button.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        
        [array addObject:button];
        [self addSubview:button];
    }
    return [array copy];
}

#pragma mark -
#pragma mark - 点击事件
- (void)levelButtonOnClick:(UIButton *)sender
{
    self.selectedButton.selected = NO;
    self.selectedButton = sender;
    self.selectedButton.selected = YES;

    [_context.live setAudioEffectCustomFlag:(int)sender.tag - 3];
}

#pragma mark -
#pragma mark - 返回
//返回
- (void)backAction
{
    [_context closeBottomPopupView];
    
    //更多页面出现
    MoreSettingView* _moreSetView = [[MoreSettingView alloc] initWithContext:self.context];
    [_context showBottomPopupView:_moreSetView];
}

@end
