//
//  FloatPushStreamView.m
//  jdcloud_ios_live_demo
//
//  Created by 熊勇 on 2018/10/12.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "FloatPushStreamView.h"
#import "AppDelegate.h"

@interface FloatPushStreamView()
{
    CGPoint _loc_in;
}

@property (nonatomic, weak) id<JDPushFlowContextProtocol> context;

@end


@implementation FloatPushStreamView

- (instancetype)initWithContext:(id<JDPushFlowContextProtocol>)context
{
    int w = ScreenWidth/4;
    int h = w*16/9;
    self = [super initWithFrame:CGRectMake(ScreenWidth - (w + 20), ScreenHeight - (h + 20), w,  h)];
    if (self) {
        self.context = context;
        [self setupSubviews];
        [self.context.live startFloatingPreview:self];
    }
    return self;
}

- (void)showInWindow
{
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    app.floatPushStreamView = self;
    [app.window addSubview:self];
    [self.context hideViewController];
}

- (void)resumVC
{
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    [self.context.live closeFloatingPreview];
    [self removeFromSuperview];
    [self.context showViewController];
    app.floatPushStreamView = nil;
}

- (void)setupSubviews
{
    self.backgroundColor = [UIColor blackColor];
    
    //返回
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    int w = self.frame.size.width;
    [leftBtn setFrame:CGRectMake(w-16, 0, 16, 16)];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"img_delete"] forState:UIControlStateNormal];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"img_delete"] forState:UIControlStateHighlighted];
    [leftBtn addTarget:self action:@selector(onBtnClose:) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:leftBtn];
    [leftBtn sizeToFit];
    
    UIPanGestureRecognizer *panGes = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panAction:)];
    [self addGestureRecognizer:panGes];
    
    UITapGestureRecognizer* tapGes = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapAction:)];
    [self addGestureRecognizer:tapGes];
    
}

- (void)tapAction:(UITapGestureRecognizer *)ges
{
    [self resumVC];
}

- (void)panAction:(UIPanGestureRecognizer *)ges
{
    UIView* _preView = self;
    int width = ScreenWidth;
    int height = ScreenHeight;
    CGPoint loc = [ges locationInView:self.superview];
    if (ges.state == UIGestureRecognizerStateBegan) {
        _loc_in = [ges locationInView:_preView];
    }
    
    // 坐标矫正，避免画面超出屏幕
    CGFloat x;
    CGFloat y;
    if (_preView.frame.size.width - _loc_in.x + loc.x >= width){
        x = width - _preView.frame.size.width * 0.5;
    }else if (loc.x - _loc_in.x <= 0) {
        x = _preView.frame.size.width * 0.5;
    }else {
        x = _preView.frame.size.width * 0.5 - _loc_in.x + loc.x;
    }
    
    if (_preView.frame.size.height - _loc_in.y + loc.y >= height) {
        y = height - _preView.frame.size.height * 0.5;
    }else if (loc.y - _loc_in.y <= 0){
        y = _preView.frame.size.height * 0.5;
    }else {
        y = _preView.frame.size.height * 0.5 - _loc_in.y + loc.y;
    }
    
    [UIView animateWithDuration:0 animations:^{
        _preView.center = CGPointMake(x, y);
    }];
}

-(void)closeWindow
{
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    [self.context releaseViewController];
    [self removeFromSuperview];
    [self.context showViewController];
    app.floatPushStreamView = nil;
}

-(IBAction)onBtnClose:(id)sender
{
    [self closeWindow];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
