//
//  MoreSettingView.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/10.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "MoreSettingView.h"
#import "JDActionController.h"
#import "WaterMarkSetView.h"
#import "SoundMixingView.h"
#import "BeautifySetLevelView.h"
#import "AudioChangeToneView.h"
#import "ChangeVoiceView.h"
#import "JDAudioCallViewController.h"
#import "MBProgressHUD+Help.h"
#import "BrushView.h"
#import "BrushControlView.h"

@interface MoreSettingView()<UIActionSheetDelegate>
{
    NSArray<UIButton *> *_buttons;
    NSArray<UISwitch *> *_switchs;
    NSArray *_btnTitleArr;
    NSArray *_switchTitleArr;
    
}

@property (nonatomic, assign) int pushStreamMode;//0普通 1纯音频+背景图

@property (nonatomic, weak) id<JDPushFlowContextProtocol> context;

@end

@implementation MoreSettingView

- (instancetype)initWithContext:(id<JDPushFlowContextProtocol>)context
{
    self = [super initWithFrame:CGRectMake(0, ScreenHeight - 330 - SafeAreaBottomHeight, ScreenWidth, 330 + SafeAreaBottomHeight)];
    if (self) {
        self.context = context;
        [self setupData];
        [self setupSubviews];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupData{
    
     self.pushStreamMode = 0;
    
    _btnTitleArr = @[@"纯音频推流",@"水印",@"音频调用",@"音频升降调",@"混音",@"贴纸",@"变声"];
    _switchTitleArr = @[@{@"title":@"镜像推流",@"state":_context.isMirroredPush?@"YES":@"NO"},
                        @{@"title":@"音频降噪",@"state":([_context.live getAudioNoiseHandleParam]!=0)?@"YES":@"NO"},
                        @{@"title":@"自动对焦",@"state":_context.isAutoFocus?@"YES":@"NO"},
                        @{@"title":@"悬浮窗推流",@"state":_context.isNeedFloatPush?@"YES":@"NO"},
                        @{@"title":@"耳返",@"state":[_context.live getAudioFoldbackEnable]?@"YES":@"NO"},
                        @{@"title":@"旋转锁定",@"state":_context.isOrientationLock?@"YES":@"NO"}
                        ];
}

#pragma mark -
#pragma mark - 初始化子视图
- (void)setupSubviews
{
    //背景色
    self.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);
    
    [self addSubViews];

    _buttons  = [self creatButtonsWithCount:_btnTitleArr.count];
    _switchs  = [self creatSwitchsWithCount:_switchTitleArr.count];
}

- (void)addSubViews {
    
    //title
    UILabel* label = [[UILabel alloc]initWithFrame:CGRectMake(0, 17, ScreenWidth, 21)];
    label.textAlignment = NSTextAlignmentCenter;
    label.font = [UIFont systemFontOfSize:15 * SCALE_FONT];
    label.textColor = [UIColor whiteColor];
    label.text = @"更多设置";
    [self addSubview:label];
    
    //返回
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setFrame:CGRectMake(5, 16, 44, 44)];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateNormal];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateHighlighted];
    [leftBtn addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:leftBtn];
    [leftBtn sizeToFit];
    
}
//button
- (NSArray<UIButton *> *)creatButtonsWithCount:(NSInteger)count {
    
    CGFloat width = (ScreenWidth - 30)/4;
    
    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:count];
    
    for (NSInteger index = 0; index < count; index++)
    {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(15 + width * (int)(index%4), 60 + (int)(index/4) * 50, width, 20);
        button.tag = 200 + index;
       // [button setContentHorizontalAlignment:1];
        [button setTitle:_btnTitleArr[index] forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [button setTitleColor:[JDStaticTools  colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        button.titleLabel.font = [UIFont systemFontOfSize:14.0 * SCALE_FONT];
        [button addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
        
        [array addObject:button];
        [self addSubview:button];
    }
    return [array copy];
}

//switch
- (NSArray<UISwitch *> *)creatSwitchsWithCount:(NSInteger)count {

    NSMutableArray<UISwitch *> *array = [NSMutableArray arrayWithCapacity:count];
    
    for (NSInteger index = 0; index < count; index++){
        UILabel* titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(19 + (index % 2) * (ScreenWidth/2 - 10), 160 + (int)(index/2)* 40, 80, 46)];
        titleLabel.textAlignment = NSTextAlignmentLeft;
        titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        titleLabel.textColor = [UIColor whiteColor];
        titleLabel.text = _switchTitleArr[index][@"title"];
        titleLabel.lineBreakMode = NSLineBreakByWordWrapping;
        titleLabel.numberOfLines = 0;
        [self addSubview:titleLabel];
        
        BOOL isSelect = NO;
        if ([_switchTitleArr[index][@"state"] isEqualToString:@"YES"]) {
            isSelect = YES;
        }
        UISwitch* switcher = [[UISwitch alloc] initWithFrame:CGRectMake(titleLabel.frame.origin.x + 90, 158 + (int)(index/2)* 43, 46, 24)];
        [switcher setOn:isSelect];
        switcher.tag = 300 + index;
        switcher.center = CGPointMake(switcher.center.x, titleLabel.center.y);
        [switcher addTarget:self action:@selector(switchAction:) forControlEvents:UIControlEventValueChanged];
        switcher.onTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
        switcher.tintColor = [UIColor grayColor];
        [array addObject:switcher];
        [self addSubview:switcher];
        
    }
    return [array copy];
}

#pragma mark -
#pragma mark - 点击事件
//更多中-Button
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 200: //纯音频推流
        {
            [_context closeBottomPopupView];
            
            if([_context.live getPushStreamMode] != JDLPushStreamMode_Normal){
                return;
            }
            
            [[[JDActionController alloc]init] showAlertControllerWithTitle:@"纯音频推流" message:@"是否设置封面？" preferredStyle:UIAlertControllerStyleAlert otherButtonTitles:@[@"不设置",@"是"] superViewController:(UIViewController*)self.context actionControllerBlock:^(NSString *buttonTitle) {
                if ([buttonTitle isEqualToString:@"是"]){
                    
                    //先判断相册读取权限
                    ALAuthorizationStatus author = [ALAssetsLibrary authorizationStatus];
                    if (author != ALAuthorizationStatusAuthorized){
                        // 无权限
                        [[[JDActionController alloc]init] showAlertControllerWithTitle:@"“京东视频云”想访问您的相册" message:@"京东视频云需要访问你的相册权限，开启后才能选择照片" preferredStyle:UIAlertControllerStyleAlert otherButtonTitles:@[@"不允许",@"好"] superViewController:(UIViewController*)self.context actionControllerBlock:^(NSString *buttonTitle) {
                            if ([buttonTitle isEqualToString:@"好"]){
                                [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
                            }
                        }];
                    }else{
                        [SharePhotoManager ShowSheetToSelectedMediaWithInFatherVC:(UIViewController*)self.context FinishPickingMediaBlock:^(UIImage *assetImage, NSURL *assetURL) {
                            if (assetImage != nil &&  assetImage.images != nil)//动画
                            {
                                 [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"纯音频推流封面的设置暂时不支持GIF格式"];
                            }
                            else if (assetImage != nil &&  assetImage.images == nil) //一张图
                            {
                                self.pushStreamMode = 1;
                                __weak typeof(self) weakSelf = self;
                                [weakSelf.context.live startBGImagePushStream:assetImage];
                            }
                            else{
                                return ;
                            }
                        }];
                    }
                }
                else if ([buttonTitle isEqualToString:@"不设置"]){
                    if(self.pushStreamMode == 0){
                        __weak typeof(self) weakSelf = self;
                        [weakSelf.context.live startBGImagePushStream:nil];
                        self.pushStreamMode = 1;
                    }
                    else{
                        self.pushStreamMode = 0;
                        __weak typeof(self) weakSelf = self;
                        [weakSelf.context.live resumeNormalPushStream];
                    }
                }
            }];
        }
            break;
        case 201: //水印
        {
            [_context closeBottomPopupView];
            WaterMarkSetView* view = [[WaterMarkSetView alloc]init];
            view.context = self.context;
            [_context showBottomPopupView:view];
            
            //调用WaterMarkSetView的buttonClickAction方法，默认设置成图片水印
            UIButton *btn = [view viewWithTag:300];
            if (btn != nil && [btn isKindOfClass:[UIButton class]] && [view respondsToSelector:@selector(buttonClickAction:)])
            {
                [view performSelector:@selector(buttonClickAction:) withObject:btn afterDelay:0];
            }
        }
            break;
        case 202: //音频调用
        {
            [_context closeBottomPopupView];
            
             __weak typeof(self) weakSelf = self;
            JDAudioCallViewController* VC = [[JDAudioCallViewController alloc]init];
            VC.currentSelectIndex = _context.currentSelectAudioIndex;
            VC.sureAction = ^(NSString *url,int currentSelectIndex) {
                weakSelf.context.currentSelectAudioIndex = currentSelectIndex;
                //添加背景音乐，url是本地文件路径
                weakSelf.context.strMusicUrl = url;
                if(url && ![url isEqualToString:@""]){
                    [weakSelf.context.live startBGMusic:url];
                }
                else{
                    [weakSelf.context.live stopBGMusicWithComplete:^(){
                    }];
                }
            };
            [_context pushViewController:VC];
        }
            break;
        case 203: //音频升降调
        {
            [_context closeBottomPopupView];
            AudioChangeToneView* view  = [[AudioChangeToneView alloc] initWithContext:self.context];
            [_context showBottomPopupView:view];
        }
            break;
        case 204: //混音
        {
            [_context closeBottomPopupView];
            SoundMixingView* _soundMixingView  = [[SoundMixingView alloc] initWithContext:self.context];
            [_context showBottomPopupView:_soundMixingView];
        }
            break;
        case 205: //贴纸
        {
            
        }
            break;
        case 206: //变声
        {
            [_context closeBottomPopupView];
            ChangeVoiceView* view  = [[ChangeVoiceView alloc] initWithContext:self.context];
            [_context showBottomPopupView:view];
        }
            break;
        case 207: //画笔推流
        {
            [_context closeBottomPopupView];
            
            if([_context.live getPushStreamMode] != JDLPushStreamMode_Normal){
                return;
            }
            
            [[NSNotificationCenter defaultCenter] postNotificationName:@"JDLBrushViewStartNotification" object:nil];
        }
            break;
        default:
            break;
    }
}

//更多中-Switch
- (void)switchAction:(UISwitch*)sender
{
    UISwitch *switcher = (UISwitch*)sender;
    BOOL isOn = [switcher isOn];
    switch (switcher.tag) {
        case 300: //镜像推流
        {
            _context.isMirroredPush = isOn;
            if ([_context.live getCurrentDevicePosition] == AVCaptureDevicePositionFront) {
                [_context.live setPushStreamHFlip:!_context.isMirroredPush];
            }else{
                [_context.live setPushStreamHFlip:NO];
            }
        }
            break;
        case 301: //音频降噪
        {
            [_context.live setAudioNoiseHandleParam:isOn?3:0];
        }
            break;
        case 302: //自动对焦
        {
            _context.isAutoFocus = YES;
        }
            break;
        case 303: //悬浮窗推流
        {
            self.context.isNeedFloatPush = isOn;
        }
            break;
        case 304: //耳返
        {
            if([_context.live isHeadsetPlugIn]){
                [_context.live setAudioFoldbackEnable:isOn];
            }
            else{
                [switcher setOn:NO];
                 [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"请先插入耳机！"];
            }
            
        }
            break;
        case 305: //推流锁定
        {
            _context.isOrientationLock = isOn;
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - 功能函数
//获取view的控制器对象
- (UIViewController *)getCurrentViewControllerWithView:(UIView *)view{
    UIResponder *next = [view nextResponder];
    do {
        if ([next isKindOfClass:[UIViewController class]]) {
            return (UIViewController *)next;
        }
        next = [next nextResponder];
    } while (next != nil);
    return nil;
}

#pragma mark -
#pragma mark - 返回
//返回
- (void)backAction
{
    [_context closeBottomPopupView];
}


@end
