//
//  SoundMixingView.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/13.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "SoundMixingView.h"
#import "MoreSettingView.h"
#import "JDSlider.h"

@interface SoundMixingView ()

@property (nonatomic, weak) id<JDPushFlowContextProtocol> context;

@end


@implementation SoundMixingView

- (instancetype)initWithContext:(id<JDPushFlowContextProtocol>)context
{
    self = [super initWithFrame:CGRectMake(0, ScreenHeight - 150 - SafeAreaBottomHeight, ScreenWidth, 150 + SafeAreaBottomHeight)];
    if (self) {
        self.context = context;
        [self setupSubviews];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子视图
- (void)setupSubviews
{
    //背景色
    self.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);
    
    [self addSubViews];
}

//子试图
- (void)addSubViews
{
    //title
    UILabel* label = [[UILabel alloc]initWithFrame:CGRectMake(0, 17, ScreenWidth, 21)];
    label.textAlignment = NSTextAlignmentCenter;
    label.font = [UIFont systemFontOfSize:15 * SCALE_FONT];
    label.textColor = [UIColor whiteColor];
    label.text = @"混音";
    [self addSubview:label];
    
    //返回
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setFrame:CGRectMake(0, 16, 44, 44)];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateNormal];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateHighlighted];
    [leftBtn addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:leftBtn];
    [leftBtn sizeToFit];
    
    UILabel* bgVolumeLabel = [JDStaticTools setupLabelframe:CGRectMake(15, 50, ScreenWidth, 40) text:@"背景音量" fontSize:14. textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentLeft];
    [self addSubview:bgVolumeLabel];
    
    UILabel* microVolumelabel = [JDStaticTools setupLabelframe:CGRectMake(15, 90, ScreenWidth, 40) text:@"麦克音量" fontSize:14. textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentLeft];
    [self addSubview:microVolumelabel];
    
    JDSlider* bgVolumeSlider = [[JDSlider alloc] initWithFrame:CGRectMake(92, 50, ScreenWidth - 100 - 30 , 40)];
    [bgVolumeSlider addTarget:self action:@selector(silderValueDidChanged:) forControlEvents:UIControlEventValueChanged];
    bgVolumeSlider.minimumTrackTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
    [bgVolumeSlider setThumbImage:[UIImage imageNamed:@"smallDots"] forState:UIControlStateNormal];
    bgVolumeSlider.minimumValue = 0.0;
    bgVolumeSlider.maximumValue = 2.0;
    bgVolumeSlider.value = [_context.live getAudioMixVolumeWithTrackId:1];
    bgVolumeSlider.tag = 500;
    [self addSubview:bgVolumeSlider];
    
    JDSlider* microVolumeSlider = [[JDSlider alloc] initWithFrame:CGRectMake(92, 90, ScreenWidth - 100 - 30, 40)];
    [microVolumeSlider addTarget:self action:@selector(silderValueDidChanged:) forControlEvents:UIControlEventValueChanged];
    microVolumeSlider.minimumTrackTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
    [microVolumeSlider setThumbImage:[UIImage imageNamed:@"smallDots"] forState:UIControlStateNormal];
    microVolumeSlider.minimumValue = 0.0;
    microVolumeSlider.maximumValue = 2.0;
    microVolumeSlider.value = [_context.live getAudioMixVolumeWithTrackId:0];
    microVolumeSlider.tag = 501;
    [self addSubview:microVolumeSlider];
}

#pragma mark -
#pragma mark - 点击事件
//silde变化
- (void)silderValueDidChanged:(JDSlider *)sender
{
    JDSlider *slider = (JDSlider*)sender;
    float translucentValue = slider.value;
    switch (slider.tag) {
        case 500: //背景音量
        {
            [_context.live setAudioMixEnable:YES withTrackId:1];
            [_context.live setAudioMixVolume:translucentValue withTrackId:1];
        }
            break;
        case 501: //麦克音量
        {
            [_context.live setAudioMixEnable:YES withTrackId:0];
            [_context.live setAudioMixVolume:translucentValue withTrackId:0];
        }
            break;
        default:
            break;
    }
}

//返回
- (void)backAction
{
    [_context closeBottomPopupView];
    
    //更多页面出现
    MoreSettingView* _moreSetView = [[MoreSettingView alloc] initWithContext:self.context];
    [_context showBottomPopupView:_moreSetView];
}

@end
