//
//  WaterMarkEditView.m
//  jdcloud_ios_live_demo
//
//  Created by 杜治全 on 2018/10/15.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "WaterMarkEditView.h"
#import "UIView+Additions.h"

@interface WaterMarkEditView ()
{
    UIView*         _borderView;                //用来显示边框或样式背景
    UIButton*       _deleteBtn;                 //删除铵钮
    UIButton*       _comfirmBtn;                //确认铵钮
    UIButton*       _scaleRotateBtn;            //单手操作放大，旋转按钮
    UIView*         _contentView;               //容器视图，放imageView或者textField或者label
    
    CGRect          _initFrame;
}
@end

@implementation WaterMarkEditView

- (id)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]) {
        _initFrame = frame;
        
        _contentView = [[UIView alloc] init];
        //_contentView.userInteractionEnabled = YES;
        
        _borderView = [UIView new];
        _borderView.layer.borderWidth = 1;
        _borderView.layer.borderColor = RGBCOLOR(0x79, 0x79, 0x79).CGColor;
        //_borderView.userInteractionEnabled = YES;
        _borderView.backgroundColor = RGBCOLOR(0x45, 0x45, 0x45);
        [self addSubview:_borderView];
        
        _deleteBtn = [UIButton new];
        [_deleteBtn setImage:[UIImage imageNamed:@"img_delete"] forState:UIControlStateNormal];
        [_deleteBtn addTarget:self action:@selector(onDeleteBtnClicked:) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:_deleteBtn];
        
        _comfirmBtn = [UIButton new];
        [_comfirmBtn setImage:[UIImage imageNamed:@"img_comfirm"] forState:UIControlStateNormal];
        [_comfirmBtn addTarget:self action:@selector(onComfirmBtnClicked:) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:_comfirmBtn];
        
        _scaleRotateBtn = [UIButton new];
        [_scaleRotateBtn setImage:[UIImage imageNamed:@"img_rotate"] forState:UIControlStateNormal];
        UIPanGestureRecognizer* panGensture = [[UIPanGestureRecognizer alloc] initWithTarget:self action: @selector (handlePanSlide:)];
        [self addSubview:_scaleRotateBtn];
        [_scaleRotateBtn addGestureRecognizer:panGensture];
        
        [_borderView  addSubview:_contentView];
        
        UIPanGestureRecognizer* selfPanGensture = [[UIPanGestureRecognizer alloc] initWithTarget:self action: @selector (handlePanSlide:)];
        [self addGestureRecognizer:selfPanGensture];
        
        UIPinchGestureRecognizer* pinchGensture = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinch:)];
        [self addGestureRecognizer:pinchGensture];

        UIRotationGestureRecognizer* rotateGensture = [[UIRotationGestureRecognizer alloc] initWithTarget:self action:@selector(handleRotate:)];
        [self addGestureRecognizer:rotateGensture];
        
        _rotateAngle = 0.f;
        _alphaValue = 1.0;
    }
    
    return self;
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    
    if (self.width == 0 || self.height == 0) return;
    
    CGPoint center = [self convertPoint:self.center fromView:self.superview];
    
    _borderView.bounds = CGRectMake(0, 0, self.bounds.size.width - 25, self.bounds.size.height - 25);
    _borderView.center = center;
    
    _contentView.frame = CGRectMake(0, 0, _borderView.bounds.size.width, _borderView.bounds.size.height);
    
    _deleteBtn.center = CGPointMake(_borderView.x, _borderView.y);
    _deleteBtn.bounds = CGRectMake(0, 0, 50, 50);
    
    _comfirmBtn.center = CGPointMake(_borderView.right, _borderView.y);
    _comfirmBtn.bounds = CGRectMake(0, 0, 50, 50);
    
    _scaleRotateBtn.center = CGPointMake(_borderView.right, _borderView.bottom);
    _scaleRotateBtn.bounds = CGRectMake(0, 0, 50, 50);
}

- (CGRect)pasterFrameOnView:(UIView *)view
{
    //计算旋转前的宽高
    CGRect rc1 = [self convertRect:_deleteBtn.frame toView:view];
    CGRect rc2 = [self convertRect:_comfirmBtn.frame toView:view];
    CGRect rc3 = [self convertRect:_scaleRotateBtn.frame toView:view];
    
    CGPoint pt1 = CGPointMake(rc1.origin.x + rc1.size.width/2, rc1.origin.y + rc1.size.height/2);
    CGPoint pt2 = CGPointMake(rc2.origin.x + rc2.size.width/2, rc2.origin.y + rc2.size.height/2);
    CGPoint pt3 = CGPointMake(rc3.origin.x + rc3.size.width/2, rc3.origin.y + rc3.size.height/2);

    double dx1 = pt1.x - pt2.x;
    double dy1 = pt1.y - pt2.y;
    double width = pow(dx1*dx1 + dy1*dy1, 0.5);
    
    double dx2 = pt2.x - pt3.x;
    double dy2 = pt2.y - pt3.y;
    double height = pow(dx2*dx2 + dy2*dy2, 0.5);

    //计算编辑后的frame
    CGRect frame = CGRectMake(_borderView.x, _borderView.y, _borderView.bounds.size.width, _borderView.bounds.size.height);
    CGRect resultRect = CGRectZero;
    
    if (![view.subviews containsObject:self]) {
        [view addSubview:self];
        CGRect rc = [self convertRect:frame toView:view];
        [self removeFromSuperview];
        resultRect = rc;
    }
    
    resultRect = [self convertRect:frame toView:view];
    
    //根据旋转前的宽高计算旋转前的frame
    resultRect = CGRectInset(resultRect, (resultRect.size.width - width)/2, (resultRect.size.height - height)/2);
    
    //旋转角反向
    _rotateAngle = -_rotateAngle;
    
    return resultRect;
}

#pragma mark - GestureRecognizer handle
- (void)onTap:(UITapGestureRecognizer*)recognizer
{
    if (self.delegate && [self.delegate respondsToSelector:@selector(onEditViewComfirm:)])
    {
        [self.delegate onEditViewComfirm:self];
    }
}

- (void)handlePanSlide:(UIPanGestureRecognizer*)recognizer
{
    //拖动
    if (recognizer.view == self) {
        CGPoint translation = [recognizer translationInView:self.superview];
        CGPoint center = CGPointMake(recognizer.view.center.x + translation.x,
                                     recognizer.view.center.y + translation.y);
        if (center.x < 0) {
            center.x = 0;
        }
        else if (center.x > self.superview.width) {
            center.x = self.superview.width;
        }
        
        if (center.y < 0) {
            center.y = 0;
        }
        else if (center.y > self.superview.height) {
            center.y = self.superview.height;
        }
        
        recognizer.view.center = center;
        
        [recognizer setTranslation:CGPointZero inView:self.superview];
    }
    else if (recognizer.view == _scaleRotateBtn)
    {
        CGPoint translation = [recognizer translationInView:self];
        
        //放大
        if (recognizer.state == UIGestureRecognizerStateChanged)
        {
            CGFloat deltaX = translation.x;
            CGFloat deltaY = translation.x * self.bounds.size.height / self.bounds.size.width;
            self.bounds = CGRectMake(0, 0, self.bounds.size.width + deltaX, self.bounds.size.height + deltaY);
        }
        [recognizer setTranslation:CGPointZero inView:self];
        
        //旋转
        CGPoint newCenter = CGPointMake(recognizer.view.center.x + translation.x, recognizer.view.center.y + translation.y);
        CGPoint anthorPoint = _contentView.center;
        CGFloat height = newCenter.y - anthorPoint.y;
        CGFloat width = newCenter.x - anthorPoint.x;
        CGFloat angle1 = atan(height / width);
        height = recognizer.view.center.y - anthorPoint.y;
        width = recognizer.view.center.x - anthorPoint.x;
        CGFloat angle2 = atan(height / width);
        CGFloat angle = angle1 - angle2;
        
        self.transform = CGAffineTransformRotate(self.transform, angle);
        _rotateAngle += angle;
    }
}

//双手指放大
- (void)handlePinch:(UIPinchGestureRecognizer*)recognizer
{
    self.bounds = CGRectMake(0, 0, self.bounds.size.width * recognizer.scale, self.bounds.size.height * recognizer.scale);
    recognizer.scale = 1;
}

////双手指旋转
- (void)handleRotate:(UIRotationGestureRecognizer*)recognizer
{
    recognizer.view.transform = CGAffineTransformRotate(recognizer.view.transform, recognizer.rotation);
    
    _rotateAngle += recognizer.rotation;
    recognizer.rotation = 0;
}

- (void)onDeleteBtnClicked:(UIButton*)sender
{
    [self removeFromSuperview];
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(onRemoveEditView:)])
    {
        [self.delegate onRemoveEditView:self];
    }
}

- (void)onComfirmBtnClicked:(UIButton*)sender
{
    if (self.delegate && [self.delegate respondsToSelector:@selector(onEditViewComfirm:)])
    {
        [self.delegate onEditViewComfirm:self];
    }
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
