//
//  WaterMarkSetView.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/11.
//  Copyright © 2018年 pengdandan. All rights reserved.
//  水印设置

#import "WaterMarkSetView.h"
#import "PictureWaterMarkEditView.h"
#import "TextWaterMarkEditView.h"
#import "TimestampWaterMarkEditView.h"
#import "MoreSettingView.h"
#import "SDWebImage.h"
#import "JDSlider.h"

@interface WaterMarkSetView() <WaterMarkEditViewDelegate>

@property (nonatomic, strong) NSArray<UIButton *> *waterTypeButtons;         //水印类型图片文字时间
@property (nonatomic, strong) UIButton *selectedTypeButton;                  //选中的水印类型btn

@property (nonatomic, strong) UIView* uploadView;                           //本地上传View
@property (nonatomic, strong) UIView* colorView;                            //颜色选择View
@property (nonatomic, strong) JDSlider* alphaSlider;                        //透明度Slider

@end

@implementation WaterMarkSetView


- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:CGRectMake(0, ScreenHeight - 240 - SafeAreaBottomHeight, ScreenWidth, 240 + SafeAreaBottomHeight)];
    if (self) {
        [self setupSubviews];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子视图
- (void)setupSubviews
{
    //背景色
    self.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);
    
    [self addSubviews];
    
    self.uploadView.hidden = NO;
    self.colorView.hidden = YES;
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
  //  CGRect rect = self.context.previewView.bounds;

    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
            

        case 300: //图片
        {
            if (_selectedTypeButton != button || _waterMarkEditView == nil)
            {
                //屏幕增加图片
                [self removeCurrentEditView];
                _alphaSlider.value = 1.0f;

                _waterMarkEditView = [[PictureWaterMarkEditView alloc] initWithFrame:[self getInitPictureEditViewRect]];
                _waterMarkEditView.delegate = self;
                [[self getCurrentViewControllerWithView:sender].view addSubview:_waterMarkEditView];
            }
            
            self.selectedTypeButton.selected = NO;
            button.selected = YES;
            self.selectedTypeButton = button;
            self.uploadView.hidden = NO;
            self.colorView.hidden = YES;
        }
            break;
        case 301: //文字
        {
            if (_selectedTypeButton != button || _waterMarkEditView == nil)
            {
                [self removeCurrentEditView];
                _alphaSlider.value = 1.0f;

                TextWaterMarkEditView *textWaterMarkEditView = [[TextWaterMarkEditView alloc] initWithFrame:[self getInitTextEditViewRect]];
                textWaterMarkEditView.delegate = self;
                [textWaterMarkEditView setText:@"试试"];
                [textWaterMarkEditView setTextFont:[UIFont systemFontOfSize:14.0]];
                
                
                _waterMarkEditView = textWaterMarkEditView;
                [[self getCurrentViewControllerWithView:sender].view addSubview:_waterMarkEditView];
            }

            self.selectedTypeButton.selected = NO;
            button.selected = YES;
            self.selectedTypeButton = button;
            self.uploadView.hidden = YES;
            self.colorView.hidden = NO;
        }
            break;
        case 302: //时间
        {
            if (_selectedTypeButton != button || _waterMarkEditView == nil)
            {
                [self removeCurrentEditView];
                _alphaSlider.value = 1.0f;
                
                TimestampWaterMarkEditView *timestampWaterMarkEditView = [[TimestampWaterMarkEditView alloc] initWithFrame:[self getInitTimeEditViewRect]];
                timestampWaterMarkEditView.delegate = self;
                [timestampWaterMarkEditView setTextFont:[UIFont systemFontOfSize:14.0]];
                [timestampWaterMarkEditView setTextColor:[UIColor whiteColor]];
                
                NSDateFormatter* fmt = [[NSDateFormatter alloc] init];
                fmt.dateFormat = @"yyyy-MM-dd HH:mm:ss";
                fmt.locale = [[NSLocale alloc] initWithLocaleIdentifier:@"zh_CN"];
                NSString* dateString = [fmt stringFromDate:[NSDate date]];
                [timestampWaterMarkEditView setText:dateString];

                _waterMarkEditView = timestampWaterMarkEditView;
                [[self getCurrentViewControllerWithView:sender].view addSubview:_waterMarkEditView];
            }

            self.selectedTypeButton.selected = NO;
            button.selected = YES;
            self.selectedTypeButton = button;
            self.uploadView.hidden = YES;
            self.colorView.hidden = NO;
            
            //测试代码，去掉水印
            for (int i = 10; i < 20; ++i)
            {
                //[_context.live removeWater:i];
            }
        }
            break;
        case 303: //本地上传
        {
            __weak typeof(self) weakSelf = self;
            [SharePhotoManager ShowSheetToSelectedMediaWithInFatherVC:[self getCurrentViewControllerWithView:self] FinishPickingMediaBlock:^(UIImage *assetImage, NSURL *assetURL) {
                
                
                if (weakSelf.waterMarkEditView && [weakSelf.waterMarkEditView isKindOfClass:[PictureWaterMarkEditView class]])
                {
                    PictureWaterMarkEditView *editView = (PictureWaterMarkEditView*)weakSelf.waterMarkEditView;
                    if (assetImage != nil &&  assetImage.images == nil)
                    {
                        [editView setImageList:@[assetImage] duration:1.0];
                    }
                    else if (assetImage != nil &&  assetImage.images != nil)//动画
                    {
                        [editView setImageList:assetImage.images duration:assetImage.duration];
                    }
                }

            }];
        }
            break;
            
        default:
            break;
    }
}

//颜色选择button
- (void)colorButtonClickAction:(UIButton *)sender
{
    NSString *text = _selectedTypeButton.titleLabel.text;
    if ([text isEqualToString:@"文字"] || [text isEqualToString:@"时间"])
    {
        NSArray* colorArr = @[[UIColor whiteColor],[UIColor redColor],[UIColor orangeColor],[UIColor yellowColor],[UIColor greenColor],[UIColor blueColor],[UIColor purpleColor],[UIColor blackColor]];
        long index = sender.tag - 400;
        UIColor *color = [colorArr objectAtIndex:index];

        TextWaterMarkEditView *editView = (TextWaterMarkEditView*)_waterMarkEditView;
        [editView setTextColor:color];
    }
}

//透明度silde变化
- (void)silderValueDidChanged:(JDSlider *)sender
{
    if (_waterMarkEditView)
    {
        JDSlider* slider = (JDSlider *)sender;
        [_waterMarkEditView setAlpha:slider.value];
        _waterMarkEditView.alphaValue = slider.value;
    }
}

- (void)saveWaterMark
{
    if (_waterMarkEditView)
    {
        CGRect waterMarkEditViewFrame = [_waterMarkEditView pasterFrameOnView:_context.previewView];
        
        CGFloat w = _context.previewView.bounds.size.width;
        CGFloat h = _context.previewView.bounds.size.height;
        CGAffineTransform t = CGAffineTransformScale(CGAffineTransformIdentity, 1.0/w, 1.0/h);
        
        [_waterMarkEditView setAlpha:1.0];
        CGFloat alpha = [_waterMarkEditView alphaValue];
        
        CGAffineTransform transform = _waterMarkEditView.transform;
        CGFloat angle = _waterMarkEditView.rotateAngle;
        
        CGRect rect = CGRectApplyAffineTransform(waterMarkEditViewFrame, t);
        
        if ([_waterMarkEditView isKindOfClass:[PictureWaterMarkEditView class]])
        {
            PictureWaterMarkEditView *editView = (PictureWaterMarkEditView*)_waterMarkEditView;
            
            if (editView.imageList)
            {
                [self.context.live addWaterWithImageList:editView.imageList duration:editView.duration rect:rect rotation:angle alpha:alpha rotationMode:kJDLNoRotation];
            }
            else if (editView.imageView.image)
            {
                UIImage *image = editView.imageView.image;
                [self.context.live addWaterWithImage:image rect:rect rotation:angle alpha:alpha rotationMode:kJDLNoRotation];
            }
        }
        else if ([_waterMarkEditView isKindOfClass:[TextWaterMarkEditView class]])
        {
            TextWaterMarkEditView *editView = (TextWaterMarkEditView*)_waterMarkEditView;
            
            [self.context.live addTextWithView:editView.textField rect:rect rotation:angle alpha:alpha rotationMode:kJDLNoRotation];
        }
        else if ([_waterMarkEditView isKindOfClass:[TimestampWaterMarkEditView class]])
        {
            TimestampWaterMarkEditView *editView = (TimestampWaterMarkEditView*)_waterMarkEditView;
            
            [self.context.live addTimestampWithLabel:editView.label rect:rect rotation:angle alpha:alpha rotationMode:kJDLNoRotation];
        }
    }
}

//返回
- (void)backAction
{
//    //把水印保存到流里
//    [self saveWaterMark];
    
    [_context closeBottomPopupView];
    
    //更多页面出现
    MoreSettingView* _moreSetView = [[MoreSettingView alloc] initWithContext:self.context];
    [_context showBottomPopupView:_moreSetView];
}

- (void)onEditViewComfirm:(WaterMarkEditView *)editView
{
    //把水印保存到流里
    [self saveWaterMark];
    
    //去除编辑框
    [self removeCurrentEditView];
}

- (void)onRemoveEditView:(WaterMarkEditView *)editView
{
    //去除编辑框
    [self removeCurrentEditView];
}

- (void)dealloc
{
    [self removeCurrentEditView];
}

#pragma mark -
#pragma mark - 功能函数
//增加子试图
- (void)addSubviews
{
    //返回
    UIButton* leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [leftBtn setFrame:CGRectMake(0, 16, 44, 44)];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateNormal];
    [leftBtn setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateHighlighted];
    [leftBtn addTarget:self action:@selector(backAction) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:leftBtn];
    [leftBtn sizeToFit];
    
    UILabel* navLabel = [JDStaticTools setupLabelframe:CGRectMake(0, 0, ScreenWidth, 60) text:@"水印设置" fontSize:15 * SCALE_FONT textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentCenter];
    [self addSubview:navLabel];
    
    UILabel* waterMarkTypelabel = [JDStaticTools setupLabelframe:CGRectMake(15, 60, ScreenWidth, 34) text:@"水印类型" fontSize:14 * SCALE_FONT textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentLeft];
    [self addSubview:waterMarkTypelabel];
    
    UILabel* translucentlabel = [JDStaticTools setupLabelframe:CGRectMake(15, 150, ScreenWidth, 60) text:@"透明度" fontSize:14 * SCALE_FONT textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentLeft];
    [self addSubview:translucentlabel];
    
    //创建水印类型
    [self creatWaterTypeButtons];
    
    //颜色选择View
    [self setColorView];
    
    //本地上传
    _uploadView = [[UIView alloc]initWithFrame:CGRectMake(0, 110, ScreenWidth, 60)];
    _uploadView.backgroundColor = [UIColor clearColor];
    [self addSubview:_uploadView];
    
    UIButton* uploadBtn = [JDStaticTools setupButtonframe:CGRectMake(92, 0, 70, 34) title:@"本地上传" fontSize:14 * SCALE_FONT textColor:[UIColor grayColor] normalImageName:@"按钮_白底" hightLightedImageName:@"按钮_白底" tag:303];
    [uploadBtn addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    [_uploadView addSubview:uploadBtn];
    
    //透明度slider
    _alphaSlider = [[JDSlider alloc] initWithFrame:CGRectMake(92, 150, ScreenWidth - 100 - 30 + 8, 60)];
    [_alphaSlider addTarget:self action:@selector(silderValueDidChanged:) forControlEvents:UIControlEventValueChanged];
    _alphaSlider.minimumTrackTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
    [_alphaSlider setThumbImage:[UIImage imageNamed:@"smallDots"] forState:UIControlStateNormal];
    _alphaSlider.value = 1.0;
    [self addSubview:_alphaSlider];
}

//设置颜色选择view
- (void)setColorView
{
    self.colorView = [[UIView alloc]initWithFrame:CGRectMake(0, 110, ScreenWidth, 60)];
    self.colorView.backgroundColor = [UIColor clearColor];
    [self addSubview:self.colorView];
    
    UILabel* translucentlabel = [JDStaticTools setupLabelframe:CGRectMake(15, 10, ScreenWidth, 21) text:@"颜色" fontSize:14 * SCALE_FONT textColor:[UIColor whiteColor] textAlignment:NSTextAlignmentLeft];
    [self.colorView addSubview:translucentlabel];
    
    //颜色的8个按钮
    [self colorButtons];
}

//创建水印类型button
- (NSArray<UIButton *> *)creatWaterTypeButtons
{
    NSArray* waterMaskTypeTitleArr = @[@"图片",@"文字",@"时间"];
    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:3];
    for (NSInteger index = 0; index < 3; index++) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(92 + index * 90 , 60, 70, 34);
        button.tag = 300 + index;
        button.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [button setTitle:waterMaskTypeTitleArr[index] forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [button setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];

        [button setBackgroundImage:[UIImage imageNamed:@"按钮_白框"] forState:UIControlStateNormal];
        [button setBackgroundImage:[UIImage imageNamed:@"按钮_蓝框"] forState:UIControlStateSelected];
        [button addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
        
        [array addObject:button];
        [self addSubview:button];
    }
    return [array copy];
}

//创建颜色button
- (NSArray<UIButton *> *)colorButtons
{
    //颜色
    NSArray* colorArr = @[[UIColor whiteColor],[UIColor redColor],[UIColor orangeColor],[UIColor yellowColor],[UIColor greenColor],[UIColor blueColor],[UIColor purpleColor],[UIColor blackColor]];
    //button个数
    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:8];
    for (NSInteger index = 0; index < 8; index++) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(92 + index * 27, 10, 22, 22);
        button.layer.cornerRadius = 11;
        button.clipsToBounds = YES;
        button.tag = 400 + index;
        [button setBackgroundImage:[JDStaticTools imageWithColor:[colorArr objectAtIndex:index]] forState:UIControlStateNormal];
        [button addTarget:self action:@selector(colorButtonClickAction:) forControlEvents:UIControlEventTouchUpInside];

        [array addObject:button];
        [self.colorView addSubview:button];
    }
    return [array copy];
}

//获取view的控制器对象
- (UIViewController *)getCurrentViewControllerWithView:(UIView *)view{
    UIResponder *next = [view nextResponder];
    do {
        if ([next isKindOfClass:[UIViewController class]]) {
            return (UIViewController *)next;
        }
        next = [next nextResponder];
    } while (next != nil);
    return nil;
}

- (CGRect)getInitPictureEditViewRect
{
    CGRect rect1 = [[self getCurrentViewControllerWithView:self] view].bounds;
    CGRect rect2 = self.bounds;
    
    CGRect rect = rect1;
    rect.size.height = rect1.size.height - rect2.size.height;
    
    CGFloat width = 190;
    CGFloat heith = 240;
    CGRect resultRect = CGRectMake(rect.origin.x + (rect.size.width - width) / 2, rect.origin.y + (rect.size.height - heith) / 2, width, heith);
    
    return resultRect;
}

- (CGRect)getInitTextEditViewRect
{
    NSString *defaultStr = @"在这里输入文字";
    CGSize size = [defaultStr sizeWithFont:[UIFont systemFontOfSize:14]];
    
    CGRect rect1 = [[self getCurrentViewControllerWithView:self] view].bounds;
    CGRect rect2 = self.bounds;
    
    CGRect rect = rect1;
    rect.size.height = rect1.size.height - rect2.size.height;
    
    CGFloat width = size.width + 50;
    CGFloat heith = size.height  + 50;
    CGRect resultRect = CGRectMake(rect.origin.x + (rect.size.width - width) / 2, rect.origin.y + (rect.size.height - heith) / 2, width, heith);
    
    return resultRect;
}

- (CGRect)getInitTimeEditViewRect
{
    CGRect rect1 = [[self getCurrentViewControllerWithView:self] view].bounds;
    CGRect rect2 = self.bounds;
    
    CGRect rect = rect1;
    rect.size.height = rect1.size.height - rect2.size.height;
    
    CGFloat width = 210;
    CGFloat heith = 80;
    CGRect resultRect = CGRectMake(rect.origin.x + (rect.size.width - width) / 2, rect.origin.y + (rect.size.height - heith) / 2, width, heith);

    return resultRect;
}


- (void)removeCurrentEditView
{
    if (_waterMarkEditView)
    {
        [_waterMarkEditView removeFromSuperview];
        _waterMarkEditView = nil;
    }
}

@end
