//
//  PushFlowParamTableViewCell.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/5.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "PushFlowParamTableViewCell.h"

@interface PushFlowParamTableViewCell ()<UITextFieldDelegate>

@property (nonatomic, strong) UILabel *titleLabel;          //title
@property (nonatomic, strong) UITextField *inputTextF;      //输入框
@property (nonatomic, strong) UITextField *bgTextF;         //输入框背景

@property (nonatomic, strong) UILabel *infoLabel;           //单位
@property (nonatomic, strong) UISwitch *switcher;           //开关
@property (nonatomic, strong) UISegmentedControl *segment;  //分段
@property (nonatomic, strong) UIImageView* bgImageV;        //背景图
@property (nonatomic, strong) UIButton* button;             //Button

@property (nonatomic, strong) PushFlowParamModel *cellModel;

@end

@implementation PushFlowParamTableViewCell

- (void)awakeFromNib {
    [super awakeFromNib];
}

#pragma mark -
#pragma mark - 初始化Cell
//初始化cell
- (void)configureCellModel:(PushFlowParamModel *)cellModel withHight:(CGFloat)hight from:(NSString*)from
{
    self.backgroundColor = [UIColor clearColor];
    self.selectionStyle = UITableViewCellSelectionStyleNone;
    self.contentView.frame = CGRectMake(0, 0, ScreenWidth, hight);

    self.cellModel = cellModel;
    
    [self setupTitleView];
    
    //选择的样式
    if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInput]) {  //只有输入框 或者后面有单位
        [self setupInputView];
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInputButton]){ //输入框 后面button
        [self setupInputView];
        [self setupButtonView];
    }
    else if([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellSwitch]) { //开关
        [self setupSwitchView];
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellSegment]) { //分段选择 或者后面有单位
        [self setupSegmentView];
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellButton]) { //只有button
         [self setupButtonView];
    }
    else {
        
    }
}

//开始布局cell中控件位置
- (void)layoutSubviews {
    
    [super layoutSubviews];
    CGFloat midY  = CGRectGetMidY(self.contentView.bounds);
    // CGFloat midX  = CGRectGetMidX(self.contentView.bounds);
    CGFloat width = CGRectGetWidth(self.contentView.bounds);
    CGFloat titleWidth = 85;
    self.titleLabel.frame = CGRectMake(10, 0, titleWidth, CGRectGetHeight(self.contentView.bounds));
    if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInput]) {
        self.bgTextF.hidden = YES;
        self.inputTextF.borderStyle = UITextBorderStyleRoundedRect;
        self.inputTextF.frame = CGRectMake(titleWidth + 5, midY - 15, (width - 120)/4, 30);
        self.infoLabel.frame = CGRectMake(CGRectGetWidth(self.inputTextF.frame) + CGRectGetMinX(self.inputTextF.frame) + 5, midY - 10, 40, 20);
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellSwitch]) {
        self.switcher.frame = CGRectMake(titleWidth + 5, midY - 16, 40, 30);
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellSegment]) {
        self.segment.frame = CGRectMake(titleWidth + 5, midY - 15, CGRectGetWidth(self.segment.frame), 30);
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInputButton]) {
        self.bgTextF.frame = CGRectMake(titleWidth + 5, midY - 15, width - 100, 30);
        self.inputTextF.frame = CGRectMake(titleWidth + 5, midY - 15, width - 135, 30);
        if ([self.cellModel.infoText isEqualToString:@"二维码"]) {
            self.button.frame = CGRectMake(self.bgTextF.frame.size.width + self.bgTextF.frame.origin.x - 33, midY - 15, 30, 30);
            [self.button setBackgroundImage:[UIImage imageNamed:@"扫一扫"] forState:UIControlStateNormal];
        }
    }else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellButton]) { //只有button
        self.button.frame = CGRectMake(0, 0, width, CGRectGetHeight(self.contentView.bounds));
    }
    else {
        
    }
}

#pragma mark - Cell中title
- (void)setupTitleView
{
    self.titleLabel = [[UILabel alloc] init];
    self.titleLabel.textAlignment = NSTextAlignmentLeft;
    self.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
    self.titleLabel.textColor = [UIColor whiteColor];
    self.titleLabel.text = self.cellModel.title;
    self.titleLabel.lineBreakMode = NSLineBreakByWordWrapping;
    self.titleLabel.numberOfLines = 0;
    self.titleLabel.adjustsFontSizeToFitWidth = YES;
    [self.contentView addSubview:self.titleLabel];
}

#pragma mark - Cell中Button
- (void)setupButtonView
{
    self.button = [UIButton buttonWithType:(UIButtonTypeCustom)];
    [self.button addTarget:self action:@selector(buttonClickAction:) forControlEvents:(UIControlEventTouchUpInside)];
    [self.button setTitleColor:[UIColor whiteColor] forState:(UIControlStateNormal)];
    self.button.titleLabel.font = [UIFont systemFontOfSize:15.f];
    self.button.layer.cornerRadius = 5;
    [self.button setTitle:self.cellModel.infoText forState:(UIControlStateNormal)];
    [self.button setImage:self.cellModel.image forState:UIControlStateNormal];
    [self.contentView addSubview:self.button];
   // self.button.backgroundColor = [UIColor yellowColor];
}

//Button按钮点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    [self.inputTextF resignFirstResponder];
    if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInputButton]) {
        self.cellModel.inputAndButtonBlock(nil,sender);
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellButton]) {
        self.cellModel.buttonBlock(sender);
    }
}

#pragma mark - Cell中输入框
- (void)setupInputView
{
    self.bgTextF = [[UITextField alloc]init];
    self.bgTextF.borderStyle = UITextBorderStyleRoundedRect;
    //self.bgTextF.backgroundColor = [UIColor redColor];
    [self.contentView addSubview:self.bgTextF];
    
    self.inputTextF = [[UITextField alloc] init];
    self.inputTextF.textColor = [UIColor grayColor];
    self.inputTextF.textAlignment = 1;
    self.inputTextF.font = [UIFont systemFontOfSize:14];
    self.inputTextF.keyboardType = UIKeyboardTypeDefault;
    self.inputTextF.text = self.cellModel.placeHolder;
    self.inputTextF.returnKeyType = UIReturnKeyDone;
    self.inputTextF.delegate = (id)self;
    self.inputTextF.borderStyle = UITextBorderStyleNone;
    [self.contentView addSubview:self.inputTextF];
  //  self.inputTextF.backgroundColor = [UIColor greenColor];
    
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.textAlignment = 0;
    self.infoLabel.font = [UIFont systemFontOfSize:13 * SCALE_FONT];
    self.infoLabel.text = self.cellModel.infoText;
    self.infoLabel.textColor = [UIColor whiteColor];
    [self.contentView addSubview:self.infoLabel];
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    
    if (!textField.text.length) {
        textField.text = textField.placeholder;
    }
    if ([self.cellModel.title isEqualToString:@"帧率"]) {
        if (textField.text.intValue > 60) {
            textField.text = @"60";
        }else if (textField.text.intValue < 10){
            textField.text = @"10";
        }
    }
    if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInputButton]) {
        if (self.cellModel.inputAndButtonBlock) {
             self.cellModel.inputAndButtonBlock(textField.text,nil);
        }
    }
    else if ([self.cellModel.reuseId isEqualToString:PushFlowParamModelReuseCellInput]) {
        if (self.cellModel.stringBlock) {
            self.cellModel.stringBlock(textField.text);
        }
    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];//释放第一响应者
    if (!textField.text.length) {
        textField.text = textField.placeholder;
    }
    return YES;
}

#pragma mark - Cell中开关
- (void)setupSwitchView {
    
    self.switcher = [[UISwitch alloc] init];
    [self.switcher setOn:self.cellModel.defaultSwitchValue];
    [self.switcher addTarget:self action:@selector(switchAction:) forControlEvents:UIControlEventValueChanged];
    self.switcher.onTintColor = RGBCOLOR(78, 185, 248);
    self.switcher.tintColor = [UIColor whiteColor];
    [self.contentView addSubview:self.switcher];
}

- (void)switchAction:(id)sender {
    
    UISwitch *switcher = (UISwitch*)sender;
    BOOL isButtonOn = [switcher isOn];
    self.cellModel.switchBlock(isButtonOn);
}


#pragma mark - Cell中分段选择
- (void)setupSegmentView {
    
    if (self.segment) {
        [self.segment removeFromSuperview];
    }
    
    NSMutableArray* arr = [NSMutableArray array];
    for (int i = 0; i< self.cellModel.segmentTitleArray.count; i++) {
        [arr addObject:self.cellModel.segmentTitleArray[i]];
    }

    self.segment = [[UISegmentedControl alloc] initWithItems:arr];
    [self.segment addTarget:self action:@selector(segmentValueDidChanged:) forControlEvents:(UIControlEventValueChanged)];
    self.segment.selectedSegmentIndex = self.cellModel.defaultValue;
    self.segment.tintColor = RGBCOLOR(162, 177, 191);
    self.segment.layer.cornerRadius = 3;
    self.segment.clipsToBounds = YES;

    
    if (arr.count >= 5) {
        self.segment.frame = CGRectMake(0, 0, ScreenWidth - 100, 30);
    }else{
        self.segment.frame = CGRectMake(0, 0, (ScreenWidth - 100)/4*arr.count, 30);
    }
    
    [self.segment setBackgroundImage:[JDStaticTools imageWithColor:[UIColor whiteColor]] forState:UIControlStateNormal barMetrics:UIBarMetricsDefault];
    [self.segment setBackgroundImage:[JDStaticTools imageWithColor: [JDStaticTools  colorWithHexString:kJDVCThemeColor]] forState:UIControlStateSelected barMetrics:UIBarMetricsDefault];
    
    
    NSDictionary *attr = [NSDictionary dictionaryWithObjectsAndKeys:[UIFont systemFontOfSize:13.f], NSFontAttributeName,[UIColor grayColor],NSForegroundColorAttributeName,nil];
    [self.segment setTitleTextAttributes:attr
                                forState:UIControlStateNormal];
    
    NSDictionary *attr2 = [NSDictionary dictionaryWithObjectsAndKeys:[UIFont systemFontOfSize:13.f], NSFontAttributeName,[UIColor whiteColor],NSForegroundColorAttributeName,nil];
    [self.segment setTitleTextAttributes:attr2
                                forState:UIControlStateSelected];
    
    [self.contentView addSubview:self.segment];
    
    self.infoLabel = [[UILabel alloc] init];
    self.infoLabel.textAlignment = 0;
    self.infoLabel.font = [UIFont systemFontOfSize:12];
    self.infoLabel.text = self.cellModel.infoText;
    self.infoLabel.textColor = [UIColor whiteColor];
    [self.contentView addSubview:self.infoLabel];

}
//分段控件选择结果回调
- (void)segmentValueDidChanged:(UISegmentedControl *)sender
{
    [[[UIApplication sharedApplication] keyWindow] endEditing:YES];
    
    int value = (int)sender.selectedSegmentIndex;
    self.cellModel.segmentBlock(value);
}


@end
