//
//  JDPlayURLVideoViewController.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/18.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDPlayURLVideoViewController.h"
#import "JDScanViewController.h"

static CGFloat inputTextFHeight = 30;                                        //inputTextF高度
static CGFloat inputTextFY = 80;                                             //inputTextFY

@interface JDPlayURLVideoViewController ()<UITextFieldDelegate,UIGestureRecognizerDelegate>

@property (nonatomic, strong) UITextField *inputTextF;                       //输入框
@property (nonatomic, strong) UITextField *backgroundTF;                     //输入框的背景
@property (nonatomic, strong) UILabel *textFieldTitleLabel;                  //输入框前面Label
@property (nonatomic, strong) UIButton* QBCodeButton;                        //二维码按钮
@property (nonatomic, strong) UIButton* playURLVideoButton;                  //播放按钮

@end

@implementation JDPlayURLVideoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.title = @"播放URL视频";
    [self setupSubviews];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.hidden = NO;
    self.view.backgroundColor = [UIColor whiteColor];
}

- (void)back
{
    if (self.backAction) {
        self.backAction(self.inputTextF.text);
    }
    [self.navigationController popViewControllerAnimated:YES];
}
 

#pragma mark -
#pragma mark - 初始化子试图
- (void)setupSubviews
{
    [self initNavgationcontrollerLeftButton];
    
    [self.view addSubview:self.backgroundTF];
    [self.view addSubview:self.inputTextF];
    [self.view addSubview:self.playURLVideoButton];
    [self.view addSubview:self.textFieldTitleLabel];
    [self.view addSubview:self.QBCodeButton];
    self.view.backgroundColor = [UIColor whiteColor];
    
    //增加手势
    [self addGesture];
}

#pragma mark -
#pragma mark - Lazy init
//输入框背景
- (UITextField *)backgroundTF
{
    if(!_backgroundTF){
        _backgroundTF = [[UITextField alloc] initWithFrame:CGRectMake(10, inputTextFY + SafeAreaTopHeight, ScreenWidth - 20, inputTextFHeight)];
        _backgroundTF.textColor = [UIColor blackColor];
        _backgroundTF.borderStyle = UITextBorderStyleRoundedRect;
    }
    return _backgroundTF;
}

//输入框
- (UITextField *)inputTextF
{
    if(!_inputTextF){
        _inputTextF = [[UITextField alloc] initWithFrame:CGRectMake(50, inputTextFY + SafeAreaTopHeight, ScreenWidth - 95, inputTextFHeight)];
        _inputTextF.textColor = [UIColor blackColor];
        _inputTextF.textAlignment = 1;
        _inputTextF.font = [UIFont systemFontOfSize:14];
        _inputTextF.keyboardType = UIKeyboardTypeDefault;
        _inputTextF.placeholder = @"请输入URL播放视频";
        _inputTextF.returnKeyType = UIReturnKeyDone;
        _inputTextF.delegate = (id)self;
        _inputTextF.borderStyle = UITextBorderStyleNone;
    }
    return _inputTextF;
}

//输入框前面Label
- (UILabel *)textFieldTitleLabel{
    if (!_textFieldTitleLabel) {
        _textFieldTitleLabel = [JDStaticTools setupLabelframe:CGRectMake(16, inputTextFY + SafeAreaTopHeight, ScreenWidth, inputTextFHeight) text:@"URL" fontSize:15. textColor:[UIColor blackColor] textAlignment:NSTextAlignmentLeft];
    }
    return _textFieldTitleLabel;
}

//二维码按钮
- (UIButton *)QBCodeButton
{
    if (!_QBCodeButton) {
        _QBCodeButton = [JDStaticTools setupButtonframe:CGRectMake(self.backgroundTF.frame.size.width - 23, inputTextFY+ SafeAreaTopHeight , 28 , 28) title:@"" fontSize:15. textColor:[UIColor whiteColor] normalImageName:@"扫一扫"  hightLightedImageName:@"扫一扫" tag:100];
        [_QBCodeButton setTitleEdgeInsets:UIEdgeInsetsMake(0, 0, _playURLVideoButton.titleLabel.frame.origin.y + SafeAreaBottomHeight, 0)];
        _QBCodeButton.layer.cornerRadius = 5;
        [_QBCodeButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _QBCodeButton;
}

//播放按钮
- (UIButton *)playURLVideoButton{
    if (!_playURLVideoButton) {
        _playURLVideoButton = [JDStaticTools setupButtonframe:CGRectMake(0, ScreenHeight - 60 - SafeAreaBottomHeight, ScreenWidth, 60 + SafeAreaBottomHeight) title:@"播放" fontSize:16. textColor:[UIColor whiteColor] normalImageName:nil hightLightedImageName:nil tag:103];
        _playURLVideoButton.backgroundColor = RGBCOLOR(39, 47, 80);
        [_playURLVideoButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playURLVideoButton;
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //二维码
        {
            //1、判断相机权限是否打开
            AVAuthorizationStatus status = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
            if (status == AVAuthorizationStatusRestricted || status == AVAuthorizationStatusDenied)
            {
                // 无权限
                [[[JDActionController alloc]init] showAlertControllerWithTitle:@"“京东视频云”想访问您的相机" message:@"京东视频云需要访问你的相机权限，开启后才能扫描二维码" preferredStyle:UIAlertControllerStyleAlert otherButtonTitles:@[@"不允许",@"好"] superViewController:self actionControllerBlock:^(NSString *buttonTitle) {
                    if ([buttonTitle isEqualToString:@"好"]){
                        [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
                    }
                }];
                return;
            }
            
            JDScanViewController* VC = [[JDScanViewController alloc]init];
            VC.style = [JDScanViewController scanViewUIStyle];
            VC.succBlock = ^(NSString *scanResultStr) {
                self.inputTextF.text = scanResultStr;
            };
            self.view.backgroundColor = [UIColor blackColor];
            [self.navigationController pushViewController:VC animated:YES];
        }
            break;
        case 103:   //播放URL视频
        {
            [self back];
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - UITextFieldDelegate
- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];//释放第一响应者
    return YES;
}

#pragma mark -
#pragma mark - Gesture
- (void)addGesture
{
    //点击
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapGesture:)];
    tap.delegate = self;
    [self.view addGestureRecognizer:tap];
}

//手势点击事件
- (void)tapGesture:(UITapGestureRecognizer *)gesture
{
    //取得所点击的点的坐标
    CGPoint point = [gesture locationInView:self.view];
    
    // 判断该点在不在区域内
    if (!CGRectContainsPoint(self.inputTextF.frame, point))
    {
       [self.view resignFirstResponder];//释放第一响应者
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
