//
//  JDVideoPlayMainViewController.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/17.
//  Copyright © 2018年 pengdandan. All rights reserved.
//  播放器首页

#import "JDVideoPlayMainViewController.h"
#import "VideoPlayListModel.h"
#import "JDPlayURLVideoViewController.h"
#import "JDVideoPlayContextProtocol.h"
#import "JDCloudMainVideoPlayView.h"
#import "MoreBottomAlertView.h"
#import "JDCloudPlayerTopView.h"
#import "JDCloudOnlyAudioPlayView.h"
#import "VideoListMainView.h"
#import "VodListView.h"
#import "LiveListView.h"
#import "LocalVideoView.h"
#import "LogListView.h"
#import <sys/utsname.h>
//#import "JDVCommon/JDCloudLogger.h"
#import "JDCloudPlayLogModel.h"

@interface JDVideoPlayMainViewController ()<JDVideoPlayContextProtocol,UIScrollViewDelegate,VideoListMainViewButtonClickDelegate>

@property (nonatomic, strong) JDCloudMainVideoPlayView*  mainPlayView;   //带UI的播放器
@property (nonatomic, strong) VideoListMainView* videoListMainView;      //直播、点播、本地视频、日志的Button的View
@property (nonatomic, strong) VodListView* vodListView;                  //点播View
@property (nonatomic, strong) LiveListView* liveListView;                //直播View
@property (nonatomic, strong) LocalVideoView* localVideoView;            //本地视频View
@property (nonatomic, strong) LogListView* logListView;                  //日志View
@property (nonatomic, strong) NSMutableArray *childViews;                //直播、点播、本地视频、日志的View
@property (nonatomic, strong) UIScrollView *contentScrollView;           //直播、点播、本地视频、日志滑动SV
@property (nonatomic, strong) UIButton* resetURLButton;                  //重设URL按钮
@property (nonatomic, strong) UIView* currentView;                       //当前弹出来的View

@property (nonatomic, assign) JDCloudPlayerState lastPlayerState;        //页面即将消失时，最后的播放状态

@end

@implementation JDVideoPlayMainViewController
@synthesize jdCloudPlayer = _jdCloudPlayer;
@synthesize infoModel = _infoModel;
@synthesize lastSelecterVideoListType = _lastSelecterVideoListType;
@synthesize lastSelecterVideoRow = _lastSelecterVideoRow;


- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupSubviews];
    [self setupData];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.hidden = YES;
    [self becomeActive];
    //退到后台后，播放器处理
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(becomeActive)
                                                 name:UIApplicationDidBecomeActiveNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(resignActive)
                                                 name:UIApplicationWillResignActiveNotification
                                               object:nil];
    
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupData
{
    //默认选中点播
    [self.videoListMainView buttonClickActionWithButtonTag:101];
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)setupSubviews
{
    [self.view addSubview:self.videoListMainView];
    [self.view addSubview:self.contentScrollView];
    [self.view addSubview:self.resetURLButton];
    [self.view addSubview:self.mainPlayView];

    // 禁止额外滚动区域
    self.automaticallyAdjustsScrollViewInsets = NO;
    
    //滑动上面加的子试图
    self.childViews = [NSMutableArray array];
    [self.childViews addObject:self.liveListView];
    [self.childViews addObject:self.vodListView];
    [self.childViews addObject:self.localVideoView];
    [self.childViews addObject:self.logListView];
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
    [[UIApplication sharedApplication] setStatusBarOrientation:orientation animated:YES];
    switch (orientation) {
        case UIInterfaceOrientationUnknown:
        case UIInterfaceOrientationPortraitUpsideDown:
            break;
        case UIInterfaceOrientationPortrait:
        {
            if(IPhoneX){
                CGRect frame = CGRectMake(0, VIEWSAFEAREAINSETS(self.view).top, ScreenWidth, ScreenWidth * 9 / 16.0);
                self.mainPlayView.frame = frame;
            }else{
                CGRect frame = CGRectMake(0, 0, ScreenWidth, ScreenWidth * 9 / 16.0);
                self.mainPlayView.frame = frame;
            }
        }
            break;
        case UIInterfaceOrientationLandscapeLeft:
        case UIInterfaceOrientationLandscapeRight:
        {
            CGRect frame = self.mainPlayView.frame;
            self.mainPlayView.frame = frame;
        }
            break;
            
        default:
            break;
    }
}


#pragma mark -
#pragma mark - 点击事件
- (void)videoListMainViewButtonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //直播
        {
            self.resetURLButton.hidden = NO;
            [self addViewToContentScrollView:button.tag - 100];
            [self.liveListView viewWillAppear];
        }
            break;
        case 101:   //点播
        {
            self.resetURLButton.hidden = NO;
            [self addViewToContentScrollView:button.tag - 100];
            [self.vodListView viewWillAppear];
        }
            break;
        case 102:   //本地视频
        {
            self.resetURLButton.hidden = YES;
            [self addViewToContentScrollView:button.tag - 100];
            [self.localVideoView viewWillAppear];
        }
            break;
        case 103:   //播放日志
        {
            self.resetURLButton.hidden = YES;
            [self addViewToContentScrollView:button.tag - 100];
        }
            break;
        default:
            break;
    }
}

//URL播放按钮点击
- (void)resetURLButtonClickAction:(UIButton *)button
{
    
    JDPlayURLVideoViewController* VC = [[JDPlayURLVideoViewController alloc]init];
    VC.backAction = ^(NSString *QR_URLStr) {
        //新的URL，播放视频
        if(QR_URLStr.length>0){
            
            VideoPlayListModel* model = [[VideoPlayListModel alloc] init];
            model.videoListType = JDCloudPlayerVideoListTypeScan;
            model.title = @"";
            model.imageUrl = nil;
            NSMutableDictionary* dic = [NSMutableDictionary dictionary];
            [dic setValue:@"标清" forKey:@"name"];
            [dic setValue:@"SD" forKey:@"code"];
            [dic setValue:@"mp4" forKey:@"format"];
            [dic setValue:QR_URLStr forKey:@"playUrl"];
            model.videoPlayURLArr = [NSMutableArray arrayWithObject:dic];
            model.selectedPlayerDic = model.videoPlayURLArr[0];
            self.infoModel = model;
            [self.mainPlayView playWithPlayerInfoModel:model];
            //刷新最后选中
            self.lastSelecterVideoRow = 0;
            self.lastSelecterVideoListType = JDCloudPlayerVideoListTypeScan;
            [self.liveListView viewWillAppear];
            [self.vodListView viewWillAppear];
            [self.localVideoView viewWillAppear];
        }
        //视频名字为空
        for (UIView* viewTwo in self.mainPlayView.subviews) {
            if ([viewTwo isKindOfClass:[JDCloudPlayerTopView class]]) {
                for (UILabel* label in viewTwo.subviews) {
                    if ([label isKindOfClass:[UILabel class]]) {
                        label.text = @"";
                    }
                }
            }
        }
    };
   
    [self pushViewController:VC];
}

#pragma mark -
#pragma mark - 功能函数

//滚动结束时，将对应的视图控制器的视图添加到内容滚动视图中
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    NSInteger i = self.contentScrollView.contentOffset.x / ScreenWidth;
    [self.videoListMainView buttonClickActionWithButtonTag:100 + i];
}

//显示选择的子视图
- (void)addViewToContentScrollView:(NSInteger)viewIndex
{
    UIView* childView = self.childViews[viewIndex];
    CGFloat x = viewIndex * ScreenWidth;
    childView.frame = CGRectMake(x, 0, ScreenWidth, ScreenHeight);
    [self.contentScrollView addSubview:childView];
    self.contentScrollView.contentOffset = CGPointMake(x, 0);
}


#pragma mark -
#pragma mark - 播放器状态处理
- (void)becomeActive{
    if (self.jdCloudPlayer && self.mainPlayView.networkStatus == JDCloudNetworkStatusReachableViaWiFi) {
        if (self.lastPlayerState == JDCloudPlayerStatePause) {
            [self.jdCloudPlayer pause];
        }else if (self.lastPlayerState == JDCloudPlayerStatePlay){
             [self.jdCloudPlayer resume];
        }else{
            [self.jdCloudPlayer stop];
        }
    }
}

- (void)resignActive{
    if (self.jdCloudPlayer) {
        self.lastPlayerState = self.jdCloudPlayer.playerState;
        if (self.lastPlayerState == JDCloudPlayerStatePlay && !self.jdCloudPlayer.isPlayOnBackGroud) {
            [self.jdCloudPlayer pause];
        }
    }
    
}

- (void)showLoadingView{
    [_mainPlayView showLoadingView];
}

#pragma mark -
#pragma mark - JDVideoPlayContextProtocol
//model数据设着
- (void)playWithPlayerInfoModel:(VideoPlayListModel *)infoModel
{
    self.infoModel = infoModel;
    [self.mainPlayView playWithPlayerInfoModel:infoModel];
    [self.mainPlayView setNeedsLayout];
}

//日志通知
- (void)playHappenEvent:(JDCloudPlayerEvent)event withPlayerInfo:(NSDictionary *)dict
{
    [self.logListView playHappenEvent:event];
    JDCloudPlayLogModel *logModel = [[JDCloudPlayLogModel alloc] initWithEvent:event];
    NSString *time = [logModel timeString];
    NSString *eventStr = [logModel eventDescription];
    NSString *point = [NSString stringWithFormat:@"%@-%@",@"播放器",[logModel eventDescription]];
    NSDictionary *data = [[NSDictionary alloc] initWithObjects:@[eventStr,time,dict] forKeys:@[@"event",@"time",@"playerInfo"]];    
}

//下一集
- (void)videoPlayNextURL{
    if (self.lastSelecterVideoListType == JDCloudPlayerVideoListTypeLive) {
        [self.liveListView videoPlayNextURL];
    }else if (self.lastSelecterVideoListType == JDCloudPlayerVideoListTypeVOD){
        [self.vodListView videoPlayNextURL];
    }else if (self.lastSelecterVideoListType == JDCloudPlayerVideoListTypeLocal){
        [self.localVideoView videoPlayNextURL];
    }else{}
}

- (VideoPlayListModel *)nextPlayInfoModel
{
    if (self.lastSelecterVideoListType == JDCloudPlayerVideoListTypeVOD){
        return [self.vodListView getNextVideoPlayInfoModel];
    }
    return nil;
}

//显示底部弹出view
- (void)showBottomPopupView:(UIView*)bottomView
{
    if(self.currentView){
        [self.currentView removeFromSuperview];
    }
    self.currentView = bottomView;
    [self.view addSubview:self.currentView];
}

//关闭底部弹出view
- (void)closeBottomPopupView
{
    [self.currentView removeFromSuperview];
}

//推到下一个VC
- (void)pushViewController:(UIViewController*)VC
{
    //去其他页面 先暂停播放器
    if (self.jdCloudPlayer) {
        self.lastPlayerState = self.jdCloudPlayer.playerState;
        if (self.lastPlayerState == JDCloudPlayerStatePlay) {
            [self.jdCloudPlayer pause];
        }
    }
    
    [self.navigationController pushViewController:VC animated:YES];
}

//关闭VC
- (void)popViewController
{
    [self releaseViewController];
    [self.navigationController popViewControllerAnimated:YES];
}

static __strong UIViewController* s_save_vc = nil;
static __strong UINavigationController* s_save_nav = nil;

//临时隐藏VC
- (void)hideViewController
{
    s_save_vc = self;
    s_save_nav = self.navigationController;
    [self.navigationController popViewControllerAnimated:NO];
}

//显示VC
- (void)showViewController
{
    [s_save_nav pushViewController:s_save_vc animated:NO];
    [self.mainPlayView insertSubview:self.jdCloudPlayer.playerView atIndex:0];
    s_save_vc = nil;
}

//销毁VC
- (void)releaseViewController
{
    //销毁定时器，不然JDCloudPlayerBottomView不销毁
    [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayRemoveTimeNotification" object:nil];
    [self.jdCloudPlayer shutdown];
    self.mainPlayView.jdCloudPlayer = nil;
    s_save_vc = nil;
    s_save_nav = nil;
}

#pragma mark -
#pragma mark - Lazy init
//点播、直播、日志、本地视频切换
- (VideoListMainView *)videoListMainView{
    if (!_videoListMainView) {
        _videoListMainView = [[VideoListMainView alloc]initWithFrame:CGRectMake(0, ScreenWidth * 9 / 16.0 + SafeAreaTopThreeHeight, ScreenWidth, 50) withContext:self];
        _videoListMainView.backgroundColor = [UIColor clearColor];
        _videoListMainView.delegate = self;
    }
    return _videoListMainView;
}

//播放URL视频
- (UIButton *)resetURLButton{
    if (!_resetURLButton) {
        _resetURLButton = [JDStaticTools setupButtonframe:CGRectMake(0, ScreenHeight - 60 - SafeAreaBottomHeight, ScreenWidth, 60 + SafeAreaBottomHeight) title:@"播放URL视频" fontSize:16. textColor:[UIColor whiteColor] normalImageName:nil hightLightedImageName:nil tag:104];
        _resetURLButton.backgroundColor = RGBCOLOR(39, 47, 80);
        [_resetURLButton addTarget:self action:@selector(resetURLButtonClickAction:) forControlEvents:UIControlEventTouchUpInside];
        _resetURLButton.hidden = YES;
    }
    return _resetURLButton;
}

//加UI的播放器
- (JDCloudMainVideoPlayView *)mainPlayView
{
    if (!_mainPlayView) {
        _mainPlayView = [[JDCloudMainVideoPlayView alloc]initWithFrame:CGRectMake(0, 0, ScreenWidth, ScreenWidth * 9 / 16.0) withContext:self];
        _jdCloudPlayer = _mainPlayView.jdCloudPlayer;
    }
    return _mainPlayView;
}

//点播View
- (VodListView *)vodListView{
    if (!_vodListView) {
        CGFloat videoPlaylistViewY = self.videoListMainView.frame.size.height + self.videoListMainView.frame.origin.y;
        _vodListView = [[VodListView alloc]initWithFrame:CGRectMake(0, videoPlaylistViewY, ScreenWidth, ScreenHeight - videoPlaylistViewY - self.resetURLButton.frame.size.height) withContext:self];
    
    }
    return _vodListView;
}

//直播View
- (LiveListView *)liveListView{
    if (!_liveListView) {
        CGFloat videoPlaylistViewY = self.videoListMainView.frame.size.height + self.videoListMainView.frame.origin.y;
        _liveListView = [[LiveListView alloc]initWithFrame:CGRectMake(0, videoPlaylistViewY, ScreenWidth, ScreenHeight - videoPlaylistViewY - self.resetURLButton.frame.size.height) withContext:self];
    }
    return _liveListView;
}

//本地视频
- (LocalVideoView *)localVideoView{
    if (!_localVideoView) {
        CGFloat videoPlaylistViewY = self.videoListMainView.frame.size.height + self.videoListMainView.frame.origin.y;
        _localVideoView = [[LocalVideoView alloc]initWithFrame:CGRectMake(0, videoPlaylistViewY, ScreenWidth, ScreenHeight - videoPlaylistViewY) withContext:self];
    }
    return _localVideoView;
}

//日志
- (LogListView *)logListView{
    if (!_logListView) {
        CGFloat videoPlaylistViewY = self.videoListMainView.frame.size.height + self.videoListMainView.frame.origin.y;
        _logListView = [[LogListView alloc]initWithFrame:CGRectMake(0, videoPlaylistViewY, ScreenWidth, ScreenHeight - videoPlaylistViewY) withContext:self];
    }
    return _logListView;
}

//点播、直播、日志、本地视频列表滑动的SV
- (UIScrollView *)contentScrollView{
    if (!_contentScrollView) {
        CGFloat videoPlaylistViewY = self.videoListMainView.frame.size.height + self.videoListMainView.frame.origin.y;
        _contentScrollView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, videoPlaylistViewY, ScreenWidth, ScreenHeight - videoPlaylistViewY)];
        _contentScrollView.contentSize = CGSizeMake(ScreenWidth * 4, 0);
        _contentScrollView.showsHorizontalScrollIndicator = NO;
        _contentScrollView.pagingEnabled = YES;
        _contentScrollView.delegate = self;
        _contentScrollView.autoresizesSubviews = NO;
    }
    return _contentScrollView;
}


 #pragma mark - 锁屏功能
 /*
 * 说明：播放器父类是UIView。
        屏幕锁屏方案需要用户根据实际情况，进行开发工作；
        如果viewcontroller在navigationcontroller中，需要添加子类重写navigationgController中的 以下方法，根据实际情况做判定 。
 */
// - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation{
//     if (self.mainPlayView.isLocked) {
//         return toInterfaceOrientation = UIInterfaceOrientationPortrait;
//     } else {
//         return YES;
//     }
//     return (toInterfaceOrientation == UIInterfaceOrientationPortrait);
// }
 
 - (BOOL)shouldAutorotate{
     return !self.mainPlayView.isLocked;
 }

//屏幕旋转完成的状态、更多页面删除、纯音频
- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator{
    for (UIView* view in self.view.subviews) {
        if ([view isKindOfClass:[MoreBottomAlertView class]]) {
            [view removeFromSuperview];
        }else if ([view isKindOfClass:[JDCloudMainVideoPlayView class]]){
            for (UIView* subView in view.subviews) {
                if ([subView isKindOfClass:[JDCloudOnlyAudioPlayView class]]) {
                    [subView removeFromSuperview];
                }
            }
        }
    }
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations{
    if (self.mainPlayView.isLocked) {
        return UIInterfaceOrientationMaskPortrait;
    }else{
        return UIInterfaceOrientationMaskAllButUpsideDown;
    }
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}






























































/*
 
 [self prefersStatusBarHidden];
 [self performSelector:@selector(setNeedsStatusBarAppearanceUpdate)];
 [[NSNotificationCenter defaultCenter] addObserver:self
 selector:@selector(becomeActive)
 name:UIApplicationDidBecomeActiveNotification
 object:nil];
 [[NSNotificationCenter defaultCenter] addObserver:self
 selector:@selector(resignActive)
 name:UIApplicationWillResignActiveNotification
 object:nil];
 
 //#import "AppDelegate.h"
 //#import <sys/utsname.h>
 //#import <MediaPlayer/MediaPlayer.h>
 
if (_volumeView == nil) {
 
 //音量处理
 @property (nonatomic, strong) MPVolumeView *volumeView;
 @property (nonatomic, weak) UISlider *volumeSlider;
 @property (nonatomic, assign) float volume;
 
    _volumeView = [[MPVolumeView alloc] init];
    for (UIView *view in [_volumeView subviews]) {
        if ([view.class.description isEqualToString:@"MPVolumeSlider"]) {
            _volumeSlider = (UISlider *)view;
            break;
        }
    }
    [_volumeView setFrame:CGRectZero];
    [_volumeView setShowsVolumeSlider:NO];
    [_volumeView sizeToFit];
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    NSString *platform =  [self iphoneType];
    CGFloat width = 0;
    CGFloat height = 0;
    CGFloat topHeight = 0;
    UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
    [[UIApplication sharedApplication] setStatusBarOrientation:orientation animated:YES];
    
    
    if (orientation == UIInterfaceOrientationPortrait ) {
        width = ScreenWidth;
        height = ScreenWidth * 9 / 16.0;
        topHeight = 0;
        //        [self refreshUIWhenScreenChanged:false];
    }else{
        width = ScreenWidth;
        height = ScreenHeight;
        topHeight = 0;
        //        [self refreshUIWhenScreenChanged:true];
    }
    CGRect tempFrame = CGRectMake(0,topHeight, width, height);
    //iphone x
    if (![platform isEqualToString:@"iPhone10,3"] && ![platform isEqualToString:@"iPhone10,6"]) {
        switch (orientation) {
            case UIInterfaceOrientationUnknown:
            case UIInterfaceOrientationPortraitUpsideDown:
            {
                
            }
                break;
            case UIInterfaceOrientationPortrait:
            {
                self.mainPlayView.frame = tempFrame;
            }
                break;
            case UIInterfaceOrientationLandscapeLeft:
            case UIInterfaceOrientationLandscapeRight:
            {
                self.mainPlayView.frame = tempFrame;
            }
                break;
                
            default:
                break;
        }
        return;
    }
    
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 110000
    
    switch (orientation) {
        case UIInterfaceOrientationUnknown:
        case UIInterfaceOrientationPortraitUpsideDown:
        {
            if (self.isStatusHidden) {
                CGRect frame = self.mainPlayView.frame;
                frame.origin.x = VIEWSAFEAREAINSETS(self.view).left;
                frame.origin.y = VIEWSAFEAREAINSETS(self.view).top;
                frame.size.width = ScreenWidth-VIEWSAFEAREAINSETS(self.view).left*2;
                frame.size.height = ScreenHeight-VIEWSAFEAREAINSETS(self.view).bottom-VIEWSAFEAREAINSETS(self.view).top;
                self.mainPlayView.frame = frame;
            }else{
                CGRect frame = self.mainPlayView.frame;
                frame.origin.y = VIEWSAFEAREAINSETS(self.view).top;
                //竖屏全屏时 isStatusHidden 来自是否 旋转回调。
                if (self.isStatusHidden) {
                    frame.size.height = ScreenHeight- VIEWSAFEAREAINSETS(self.view).top- VIEWSAFEAREAINSETS(self.view).bottom ;
                }
                self.mainPlayView.frame = frame;
            }
        }
            break;
        case UIInterfaceOrientationPortrait:
        {
            width = ScreenWidth;
            height = ScreenWidth * 9 / 16.0;
            topHeight = 0;
            //            [self refreshUIWhenScreenChanged:false];
            
            CGRect frame = CGRectMake(0, topHeight, width, height);
            frame.origin.y = VIEWSAFEAREAINSETS(self.view).top + 0;
            //竖屏全屏时 isStatusHidden 来自是否 旋转回调。
            if (self.isStatusHidden) {
                frame.size.height = ScreenHeight- VIEWSAFEAREAINSETS(self.view).top- VIEWSAFEAREAINSETS(self.view).bottom;
            }
            self.mainPlayView.frame = frame;
        }
            break;
        case UIInterfaceOrientationLandscapeLeft:
        case UIInterfaceOrientationLandscapeRight:
        {
            CGRect frame = self.mainPlayView.frame;
            frame.origin.x = VIEWSAFEAREAINSETS(self.view).left;
            frame.origin.y = VIEWSAFEAREAINSETS(self.view).top;
            frame.size.width = ScreenWidth-VIEWSAFEAREAINSETS(self.view).left*2;
            frame.size.height = ScreenHeight-VIEWSAFEAREAINSETS(self.view).bottom;
            self.mainPlayView.frame = frame;
        }
            break;
            
        default:
            break;
    }
#else
    
#endif
}

- (void)gestureSetVolumeUp
{
    [_volumeView setHidden:YES];
    self.volume = self.volumeSlider.value;
    if (self.volume < 1) {
        self.volume += 0.2/17.0;
    }
    [self.volumeSlider setValue:self.volume animated:NO];
    [self.volumeSlider sendActionsForControlEvents:UIControlEventAllEvents];
}

- (void)gestureSetVolumeDown{
    [_volumeView setHidden:YES];
    self.volume = self.volumeSlider.value;
    if (self.volume > 0) {
        self.volume -= 0.2/17.0;
    }
    [self.volumeSlider setValue:self.volume animated:NO];
    [self.volumeSlider sendActionsForControlEvents:UIControlEventAllEvents];
}

#pragma mark -
#pragma mark - 播放器状态处理
- (void)becomeActive{
    NSLog(@"播放器状态:%ld",(long)self.jdCloudPlayer.playerState);
    if (self.jdCloudPlayer && self.jdCloudPlayer.playerState == JDCloudPlayerStatePause){
        NSLog(@"");
        //        [self.jdCloudPlayer play];
    }
}

- (void)resignActive{
    if (self.jdCloudPlayer && !self.jdCloudPlayer.isPlayOnBackGroud){
        //        [self.jdCloudPlayer pause];
    }
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    app.allowRotation = YES;
    self.navigationController.navigationBar.hidden = YES;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:YES];
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    app.allowRotation = NO;
}


#define VIEWSAFEAREAINSETS(view) ({UIEdgeInsets i; if(@available(iOS 11.0, *)) {i = view.safeAreaInsets;} else {i = UIEdgeInsetsZero;} i;})


- (void)dealloc
{
    self.mainPlayView = nil;
    [self.mainPlayView.jdCloudPlayer shutdown];
    self.mainPlayView.jdCloudPlayer = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}




[self prefersStatusBarHidden];
[self performSelector:@selector(setNeedsStatusBarAppearanceUpdate)];
[[NSNotificationCenter defaultCenter] addObserver:self
                                         selector:@selector(becomeActive)
                                             name:UIApplicationDidBecomeActiveNotification
                                           object:nil];
[[NSNotificationCenter defaultCenter] addObserver:self
                                         selector:@selector(resignActive)
                                             name:UIApplicationWillResignActiveNotification
                                           object:nil];
[self addPlayNotifications];



//手机类型
- (NSString*)iphoneType {
    struct utsname systemInfo;
    uname(&systemInfo);
    NSString*platform = [NSString stringWithCString: systemInfo.machine encoding:NSASCIIStringEncoding];
    return platform;
=======
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    
>>>>>>> 37857006f3a42f749a1384261847936ff7072387
}
*/

/*
#pragma mark - 锁屏功能
/
 * 说明：播放器父类是UIView。
 屏幕锁屏方案需要用户根据实际情况，进行开发工作；
 如果viewcontroller在navigationcontroller中，需要添加子类重写navigationgController中的 以下方法，根据实际情况做判定 。

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation{
    
    return YES;
}

- (BOOL)shouldAutorotate{
    return YES;
}



- (UIInterfaceOrientationMask)supportedInterfaceOrientations{
    //    return UIInterfaceOrientationMaskPortrait;
    return UIInterfaceOrientationMaskAllButUpsideDown;
}

 */




@end

