//
//  ALPVCenterView.m
//

#import "JDCloudPlayerLoadingView.h"
#import "JDCloudPlayerGifView.h"
#import <Foundation/NSBundle.h>
#import "JDCloudPrivateDefine.h"
#import "JDCloudUtil.h"

static const CGFloat JDCloudPlayerLoadingViewGifViewWidth   = 28;   //gifView 宽度
static const CGFloat JDCloudPlayerLoadingViewGifViewHeight  = 28;   //gifView 高度
static const CGFloat JDCloudPlayerLoadingViewMargin         = 2;    //间隙

@interface JDCloudPlayerLoadingView ()
@property (nonatomic, strong) JDCloudPlayerGifView *gifView;
@property (nonatomic, strong) UILabel *tipLabelView;

@end

@implementation JDCloudPlayerLoadingView

- (JDCloudPlayerGifView *)gifView{
    if (!_gifView) {
        _gifView = [[JDCloudPlayerGifView alloc] init];
        [_gifView setGifImageWithName:@"al_loader"];
    }
    return _gifView;
}

- (UILabel *)tipLabelView{
    if (!_tipLabelView) {
        NSString *str = @"loading";
        _tipLabelView = [[UILabel alloc] init];
        [_tipLabelView setText:str];
        [_tipLabelView setTextColor:kJDCloudColorTextNomal];
        [_tipLabelView setFont:[UIFont systemFontOfSize:[JDCloudUtil nomalTextSize]]];
        [_tipLabelView setTextAlignment:NSTextAlignmentCenter];
    }
    return _tipLabelView;
}


#pragma mark - init
- (instancetype)init {
    return [self initWithFrame:CGRectZero];
}

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self setBackgroundColor:[UIColor clearColor]];
        [self setHidden:YES];
        [self addSubview:self.gifView];
        [self addSubview:self.tipLabelView];
    }
    return self;
}

- (void)layoutSubviews {
    [super layoutSubviews];
    float width = self.bounds.size.width;
    float margin = JDCloudPlayerLoadingViewMargin;
    float textHeight = [JDCloudUtil nomalTextSize];
    float messageViewY = (width - textHeight) / 2;
    self.tipLabelView.frame = CGRectMake(0, messageViewY, width, textHeight);
    float gifWidth = JDCloudPlayerLoadingViewGifViewWidth;
    float gifHeight = JDCloudPlayerLoadingViewGifViewHeight;
    self.gifView.frame = CGRectMake((width - gifWidth) / 2, messageViewY - gifHeight - margin, gifWidth, gifWidth);
    [self.gifView startAnimation];
}

#pragma mark - public method
- (void)show {
    if (![self isHidden]) {
        return;
    }
    [self.gifView startAnimation];
    [self setHidden:NO];
}

- (void)dismiss {
    if ([self isHidden]) {
        return;
    }
    [self.gifView stopAnimation];
    [self setHidden:YES];
}

@end
