//
//  JDCloudPlayerPopLayer.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerPopLayer.h"
#import "JDCloudUtil.h"

static const CGFloat JDCloudPopBackButtonWidth  = 30;  //返回按钮宽度
static const CGFloat JDCloudPopBackButtonHeight = 30;  //返回按钮高度

@interface JDCloudPlayerPopLayer () <JDCloudPlayerErrorViewDelegate>
@property (nonatomic, strong) UIButton *backBtn;            //返回按钮
@property (nonatomic, strong) JDCloudPlayerErrorView *errorView; //错误view

@end
@implementation JDCloudPlayerPopLayer

- (UIButton *)backBtn{
    if (!_backBtn) {
        _backBtn = [[UIButton alloc] init];
        [_backBtn addTarget:self action:@selector(onClick:) forControlEvents:UIControlEventTouchUpInside];
        UIImage *image = [UIImage imageNamed:@"img_delete"];
        [_backBtn setBackgroundImage:image forState:UIControlStateNormal];
        [_backBtn setBackgroundImage:image forState:UIControlStateHighlighted];
    }
    return _backBtn;
}

- (JDCloudPlayerErrorView *)errorView{
    if (!_errorView) {
        _errorView = [[JDCloudPlayerErrorView alloc] init];
    }
    return _errorView;
}

#pragma mark - init
- (instancetype)init {
    return [self initWithFrame:CGRectZero];
}

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = [UIColor blackColor];
        [self addSubview:self.backBtn];
        self.errorView.delegate = self;
    }
    return self;
}

- (void)layoutSubviews{
    [super layoutSubviews];
    self.backBtn.frame = CGRectMake(15, 20,JDCloudPopBackButtonWidth,JDCloudPopBackButtonWidth);
    self.errorView.center = CGPointMake(self.frame.size.width / 2, self.frame.size.height / 2);
}

#pragma makr - 重写setter方法
//- (void)setSkin:(JDCloudPlayerViewSkin)skin{
//    _errorView.skin = skin;
//}

#pragma mark - onClick
- (void)onClick:(UIButton *)btn {
    if (![JDCloudUtil isInterfaceOrientationPortrait]) {
        [JDCloudUtil setFullOrHalfScreen];
    }
    if (self.delegate && [self.delegate respondsToSelector:@selector(onBackClickedWithPopLayer:)]) {
        [self.delegate onBackClickedWithPopLayer:self];
    }
}

#pragma mark - public method
/*
 #define JDCloudVODVIEW_UNKNOWN              @"未知错误"
 #define JDCloudVODVIEW_PLAYFINISH           @"再次观看，请点击重新播放"
 #define JDCloudVODVIEW_NETWORKTIMEOUT       @"当前网络不佳，请稍后点击重新播放"
 #define JDCloudVODVIEW_NETWORKUNREACHABLE   @"无网络连接，检查网络后点击重新播放"
 #define JDCloudVODVIEW_LOADINGDATAERROR     @"视频加载出错，请点击重新播放"
 #define JDCloudVODVIEW_USEMOBILENETWORK     @"当前为移动网络，请点击播放"
 */
- (void)showPopViewWithCode:(JDCloudPlayerPopCode)code popMsg:(NSString *)popMsg {
    if ([_errorView isShowing]) {
        [_errorView dismiss];
    }
//    NSBundle *resourceBundle = [JDCloudUtil languageBundle];
    NSString *tempString = @"unknown";
    JDCloudErrorType type = JDCloudErrorTypeRetry;
    switch (code) {
        case JDCloudPlayerPopCodePlayFinish:
        {
            tempString = [JDCloudUtil playFinishTips];
            if (!tempString) {
                tempString = @"Watch again, please click replay";
            }
            type = JDCloudErrorTypeReplay;
        }
            break;
        case JDCloudPlayerPopCodeNetworkTimeOutError :
        {
            tempString = [JDCloudUtil networkTimeoutTips];
            if (!tempString) {
                tempString = @"The current network is not good. Please click replay later";
            }
            type = JDCloudErrorTypeReplay;
        }
            break;
        case JDCloudPlayerPopCodeUnreachableNetwork:
        {
            tempString = [JDCloudUtil networkUnreachableTips];
            if (!tempString) {
                tempString = @"No network connection, check the network, click replay";
            }
            type = JDCloudErrorTypePause;
        }
            break;
        case JDCloudPlayerPopCodeLoadDataError :
        {
            tempString = [JDCloudUtil loadingDataErrorTips];
            if (!tempString) {
                tempString = @"Video loading error, please click replay";
            }
            type = JDCloudErrorTypeRetry;
        }
            break;
        case JDCloudPlayerPopCodeServerError:
        {
            tempString = popMsg;
            type = JDCloudErrorTypeRetry;
        }
            break;
        case JDCloudPlayerPopCodeUseMobileNetwork:
        {
            
            tempString = [JDCloudUtil switchToMobileNetworkTips];
            if (!tempString) {
                tempString = @"For mobile networks, click play";
            }
            type = JDCloudErrorTypePause;
            
        }
            break;
        default:
            break;
    }
    if(popMsg){
        tempString = popMsg;
    }
    self.errorView.message = tempString;
    self.errorView.type = type;
    [_errorView showWithParentView:self];
}

#pragma mark - JDCloudPVErrorViewDelegate
- (void)onErrorViewClickedWithType:(JDCloudErrorType)type{
    if (self.delegate && [self.delegate respondsToSelector:@selector(showPopViewWithType:)]) {
        [self.delegate showPopViewWithType:type];
    }
}

@end
