//
//  LiveListView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/10/25.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "LiveListView.h"
#import "VideoPlayListModel.h"
#import "MBProgressHUD+Help.h"
#import "JDHttpRequest.h"
#import "VideoPlayListTableViewCell.h"
#import "JDCloudMainVideoPlayView.h"
#import "JDCloudPlayerTopView.h"

@interface LiveListView()<UITableViewDelegate,UITableViewDataSource,HttpRequestDelegate>


@property (nonatomic, strong) UITableView *listTableView;                                 //显示列表的TableView
@property (nonatomic, strong) NSMutableArray <VideoPlayListModel *>*liveListArr;           //点播列表

@property (retain, nonatomic) JDHttpRequest *httpRequst;                                  //http请求类
@property (nonatomic, assign) BOOL isHttpLoading;                                         //http是否在请求中

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;                       //播放器通过上下文传过来

@property (nonatomic, strong) UIView *retryView;                                           //重试视图

@end

@implementation LiveListView

- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context
{
    self = [super initWithFrame:frame];
    if (self) {
        self.context = context;
        [self setupSubviews];
        self.httpRequst = [[JDHttpRequest alloc]init];
        self.httpRequst.delegate = self;
        self.isHttpLoading = NO;
    }
    return self;
}

- (void)dealloc{
    self.httpRequst.delegate = nil;
}

#pragma mark -
#pragma mark - 初始化子试图
//初始化子试图
- (void)setupSubviews
{
     [self addRetryView];
    [self addSubview:self.listTableView];
    [JDStaticTools setExtraCellLineHidden:self.listTableView];
}

- (UITableView*)listTableView{
    if (!_listTableView) {
        _listTableView = [[UITableView alloc]initWithFrame:CGRectMake(0, 0, ScreenWidth, self.frame.size.height)];
        _listTableView.dataSource = self;
        _listTableView.delegate = self;
        _listTableView.backgroundColor = [UIColor clearColor];
        _listTableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        _listTableView.hidden = YES;
    }
    return _listTableView;
}

#pragma mark -
#pragma mark - 初始化数据
//初始化子试图
- (void)setupData
{
    self.retryView.hidden = YES;
    
    //发送请求
    if ([JDStaticTools checkNetWork] > 0){
        self.liveListArr = [NSMutableArray array];
        //开始请求
        if (!self.isHttpLoading) {
            self.isHttpLoading = YES;
            [MBProgressHUD showHUDInView:self HudType:MBProgressHUDModeIndeterminate withText:@""];
            [self.httpRequst sendGetRequstToUrl:@"https://demo-player-openapi.jdcloud.com/v1/lives?" withParam:@"" withTag:100];
            // [self.httpRequst sendGetRequstToUrl:@"http://10.226.139.136:1050/v1/lives?" withParam:@"" withTag:100];
        }
        
    }else{
        [self netWorkFailed];
    }
}

#pragma mark -
#pragma mark - 刷新数据
//刷新数据
- (void)viewWillAppear{
    if(self.liveListArr.count < 1 ){
        [self setupData];
    }else{
        [self.listTableView reloadData];
    }
}

#pragma mark -
#pragma mark - 下一集
- (void)videoPlayNextURL
{
    if (self.context.lastSelecterVideoRow != self.liveListArr.count - 1) { //不是最后一个视频
        NSUInteger i = self.context.lastSelecterVideoRow +1;
        NSIndexPath *nextCellIndexPath = [NSIndexPath indexPathForRow:i inSection:0];
        [self tableView:self.listTableView didSelectRowAtIndexPath:nextCellIndexPath];
        [self.listTableView reloadData];
    }
    else{
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"已经是最后一个视频了～"];
    }
}

#pragma mark -
#pragma mark - 请求解析
//成功 rtmp://pl2.jdcloud.com/sdk/CMCC_new_sdks
- (void)RequestSuccessWithDataDic:(NSDictionary *)data withTag:(int)tag
{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.listTableView.hidden = NO;
        self.retryView.hidden = YES;
        self.isHttpLoading = NO;
        [MBProgressHUD removeHUDFromView:self];
        NSMutableArray * arr = data[@"result"];
        for(int i = 0; i < arr.count; i++){
            VideoPlayListModel* model = [[VideoPlayListModel alloc]init];
            model.videoListType = JDCloudPlayerVideoListTypeLive;
            model.title = arr[i][@"name"];
            model.imageUrl = arr[i][@"imgUrl"];
            model.videoFormat = JDCloudPlayerVideoFormatRtmp;
            //设置videoPlayURLArr
            NSMutableArray* tempArr = [NSMutableArray array];
            NSMutableArray* streamListArr = arr[i][@"streamList"];
            for (int j = 0; j < streamListArr.count; j++) {
                NSMutableDictionary* dic = [NSMutableDictionary dictionary];
                [dic setValue:streamListArr[j][@"name"] forKey:@"name"];
                [dic setValue:streamListArr[j][@"code"] forKey:@"code"];
                [dic setValue:@"rtmp" forKey:@"format"];
                [dic setValue:streamListArr[j][@"playUrlRtmp"] forKey:@"playUrl"];
                [tempArr addObject:dic];
            }
            model.videoPlayURLArr = tempArr;
            [self.liveListArr addObject:model];
        }
        [self.listTableView reloadData];
    });
}



//失败 使用测试数据
- (void)RequestFailedWithErr:(NSError *)error withTag:(int)tag{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.isHttpLoading = NO;
        [MBProgressHUD removeHUDFromView:self];
        self.listTableView.hidden = YES;
        self.retryView.hidden = NO;
    });
}

#pragma mark -
#pragma mark - 功能函数
//网络检查失败处理
- (void)netWorkFailed{
    [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"您的网络好像有问题，请检查。"];
}

//重设视频名字
- (void)setName:(NSString*)name{
    for (UIView* view in self.superview.superview.subviews) {
        if ([view isKindOfClass:[JDCloudMainVideoPlayView class]]) {
            for (UIView* viewTwo in view.subviews) {
                if ([viewTwo isKindOfClass:[JDCloudPlayerTopView class]]) {
                    for (UILabel* label in viewTwo.subviews) {
                        if ([label isKindOfClass:[UILabel class]]) {
                            label.text = name;
                        }
                    }
                }
            }
        }
    }
}


#pragma mark -
#pragma mark - UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.liveListArr.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString* CellIdentifier = @"CellOne";
    VideoPlayListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (!cell) {
        cell = [[[NSBundle mainBundle] loadNibNamed:@"VideoPlayListTableViewCell" owner:self options:nil] objectAtIndex:0];
        cell.backgroundColor = [UIColor clearColor];
        [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    if (indexPath.row < self.liveListArr.count){
        VideoPlayListModel* model = self.liveListArr[indexPath.row];
        if (self.context.lastSelecterVideoListType == JDCloudPlayerVideoListTypeLive) {
            [cell setupVideoPlayListModel:model withSelectedRow:self.context.lastSelecterVideoRow WithCurrRow:indexPath.row];
        }else{
            [cell setupVideoPlayListModel:model withSelectedRow:-1 WithCurrRow:indexPath.row];
        }
    }
    return cell;
}

- (CGFloat) tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 88.;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    self.context.lastSelecterVideoListType = JDCloudPlayerVideoListTypeLive;
    self.context.lastSelecterVideoRow = indexPath.row;
    [self.listTableView reloadData];
    
    //播放器重设播放URL
    VideoPlayListModel* model = self.liveListArr[indexPath.row];
    [self setName:model.title];
    model.selectedPlayerDic = model.videoPlayURLArr[0];
    model.videoFormat = JDCloudPlayerVideoFormatRtmp;
    model.videoListType = JDCloudPlayerVideoListTypeLive;
    [self.context playWithPlayerInfoModel:model];
}

#pragma mark -
#pragma mark - --重试 --
- (void)addRetryView
{
    if (!self.retryView) {
        self.retryView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, ScreenWidth, self.frame.size.height)];
        self.retryView.backgroundColor = [UIColor clearColor];
        [self addSubview:self.retryView];
        
        UILabel *retryLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, self.frame.size.height/2- 30, ScreenWidth, 30)];
        [retryLabel setBackgroundColor:[UIColor clearColor]];
        [retryLabel setFont:[UIFont systemFontOfSize:16]];
        [retryLabel setTextColor:[UIColor whiteColor]];
        [retryLabel setTextAlignment:NSTextAlignmentCenter];
        [retryLabel setText:@"加载失败，请点击屏幕重试！"];
        [self.retryView addSubview:retryLabel];
        
//        UIButton *retryBtn = [JDStaticTools setupButtonframe:CGRectMake(50, 90, ScreenWidth-100, 44) title:@"重 试" fontSize:16. textColor:[UIColor whiteColor] normalImageName:nil hightLightedImageName:nil tag:104];
//        retryBtn.backgroundColor = RGBCOLOR(39, 47, 80);
//        [retryBtn addTarget:self action:@selector(setupData) forControlEvents:UIControlEventTouchUpInside];
//        [self.retryView addSubview:retryBtn];
        
        UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(setupData)];
        [self.retryView addGestureRecognizer:tap];
    }
}

@end
