//
//  LocalVideoView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/10/25.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "LocalVideoView.h"
#import "VideoPlayListModel.h"
#import "VideoPlayListTableViewCell.h"
#import "JDCloudMainVideoPlayView.h"
#import "JDCloudPlayerTopView.h"
#import <Photos/PHAssetResourceManager.h>
#import <Photos/PHFetchOptions.h>
#import <Photos/PHAsset.h>
#import <Photos/PHImageManager.h>
#import <Photos/PHAssetResource.h>
#import "MBProgressHUD+Help.h"

@interface LocalVideoView()<UITableViewDelegate,UITableViewDataSource>

@property (nonatomic, strong) UITableView *listTableView;                                 //显示列表的TableView
@property (nonatomic, strong) PHFetchResult *fetchResults;                                //本地视频列表
@property (nonatomic, strong) AVPlayerItem * _Nullable playerItem;                        //播放器的playerItem

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;                       //播放器通过上下文传过来

@end

@implementation LocalVideoView

- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context
{
    self = [super initWithFrame:frame];
    if (self) {
        self.context = context;
        [self setupSubviews];
        [self setupData];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子试图
//初始化子试图
- (void)setupSubviews
{
    [self addSubview:self.listTableView];
    [JDStaticTools setExtraCellLineHidden:self.listTableView];
}

- (UITableView*)listTableView{
    if (!_listTableView) {
        _listTableView = [[UITableView alloc]initWithFrame:CGRectMake(0, 0, ScreenWidth, self.frame.size.height)];
        _listTableView.dataSource = self;
        _listTableView.delegate = self;
        _listTableView.backgroundColor = [UIColor clearColor];
        _listTableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    }
    return _listTableView;
}

#pragma mark -
#pragma mark - 初始化数据
//初始化子试图
- (void)setupData
{
    if (self.fetchResults.count < 1) {
        PHFetchOptions *fetchOption = [[PHFetchOptions alloc] init];
        fetchOption.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"creationDate" ascending:false]];
        fetchOption.predicate = [NSPredicate predicateWithFormat:@"mediaType = %d",PHAssetMediaTypeVideo];
        self.fetchResults = [PHAsset fetchAssetsWithMediaType:PHAssetMediaTypeVideo options:fetchOption];
    }
}

#pragma mark -
#pragma mark - 刷新数据
//刷新数据
- (void)viewWillAppear{
    if(self.fetchResults.count < 1 ){
        [self setupData];
    }else{
        [self.listTableView reloadData];
    }
}

#pragma mark -
#pragma mark - 下一集
- (void)videoPlayNextURL
{
    if (self.context.lastSelecterVideoRow != self.fetchResults.count - 1) { //不是最后一个视频
        NSUInteger i = self.context.lastSelecterVideoRow +1;
        NSIndexPath *nextCellIndexPath = [NSIndexPath indexPathForRow:i inSection:0];
        [self tableView:self.listTableView didSelectRowAtIndexPath:nextCellIndexPath];
        [self.listTableView reloadData];
    }
    else{
        [MBProgressHUD showHUDInView:[UIApplication sharedApplication].keyWindow HudType:MBProgressHUDModeCustomView withText:@"已经是最后一个视频了～"];
    }
}

#pragma mark -
#pragma mark - UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.fetchResults.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString* CellIdentifier = @"CellTwo";
    VideoPlayListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (!cell) {
        cell = [[[NSBundle mainBundle] loadNibNamed:@"VideoPlayListTableViewCell" owner:self options:nil] objectAtIndex:1];
        cell.backgroundColor = [UIColor clearColor];
        [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    if (indexPath.row < self.self.fetchResults.count){
        if (self.context.lastSelecterVideoListType == JDCloudPlayerVideoListTypeLocal) {
           [cell setupOffLineVideoModelArr:self.fetchResults withSelectedRow:self.context.lastSelecterVideoRow WithCurrRow:indexPath.row];
        }else{
           [cell setupOffLineVideoModelArr:self.fetchResults withSelectedRow:-1 WithCurrRow:indexPath.row];
        }
    }
    return cell;
}

- (CGFloat) tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 88.;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    self.context.lastSelecterVideoListType = JDCloudPlayerVideoListTypeLocal;
    self.context.lastSelecterVideoRow = indexPath.row;
    [self.listTableView reloadData];
    
    PHAsset *asset = self.fetchResults[indexPath.row];
    NSString* name = [(AVAsset*)asset valueForKey:@"filename"];
    [self setName:name];
    
    //播放器重设播放URL
    [[PHImageManager defaultManager] requestPlayerItemForVideo:self.fetchResults[indexPath.row] options:nil resultHandler:^(AVPlayerItem * _Nullable playerItem, NSDictionary * _Nullable info) {
        self.playerItem = playerItem;
        
        dispatch_async(dispatch_get_main_queue(), ^{
            
            if ([playerItem.asset isKindOfClass:[AVURLAsset class]])
            {
                VideoPlayListModel* model = [[VideoPlayListModel alloc] init];
                model.videoListType = JDCloudPlayerVideoListTypeLocal;
                model.title = name;
                model.imageUrl = nil;
                NSMutableDictionary* dic = [NSMutableDictionary dictionary];
                [dic setValue:@"标清" forKey:@"name"];
                [dic setValue:@"SD" forKey:@"code"];
                [dic setValue:@"mp4" forKey:@"format"];
                [dic setValue:[(AVURLAsset*)playerItem.asset URL].path forKey:@"playUrl"];
                model.videoPlayURLArr = [NSMutableArray arrayWithObject:dic];
                model.selectedPlayerDic = model.videoPlayURLArr[0];
                [self.context playWithPlayerInfoModel:model];
            }
            else if ([playerItem.asset isKindOfClass:[AVComposition class]]){
                [self playAVCompositionAsset:asset];
            }
        });
    }];
}

#pragma mark -
#pragma mark - 功能函数
//重设视频名字
- (void)setName:(NSString*)name{
    for (UIView* view in self.superview.superview.subviews) {
        if ([view isKindOfClass:[JDCloudMainVideoPlayView class]]) {
            for (UIView* viewTwo in view.subviews) {
                if ([viewTwo isKindOfClass:[JDCloudPlayerTopView class]]) {
                    for (UILabel* label in viewTwo.subviews) {
                        if ([label isKindOfClass:[UILabel class]]) {
                            label.text = name;
                        }
                    }
                }
            }
        }
    }
}

//适配时异常处理
- (void)playAVCompositionAsset:(PHAsset*)phasset
{
    if (@available(iOS 9.1, *)){
        NSArray *assetResources = [PHAssetResource assetResourcesForAsset:phasset];
        PHAssetResource *resource;
        
        for (PHAssetResource *assetRes in assetResources){
            if (assetRes.type == PHAssetResourceTypePairedVideo || assetRes.type == PHAssetResourceTypeVideo){
                resource = assetRes;
            }
        }
        NSString * fileName = @"";
        if (resource.originalFilename){
            fileName = resource.originalFilename;
        }
        
        if (fileName.length > 0 && (phasset.mediaType == PHAssetMediaTypeVideo || phasset.mediaSubtypes == PHAssetMediaSubtypePhotoLive)){
            PHVideoRequestOptions *options = [[PHVideoRequestOptions alloc] init];
            options.version = PHImageRequestOptionsVersionCurrent;
            options.deliveryMode = PHImageRequestOptionsDeliveryModeHighQualityFormat;
            NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
            NSString *documentsDirectory = [paths objectAtIndex:0];
            NSString *savePath =  [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"%@", fileName]];
            
            [[PHAssetResourceManager defaultManager] writeDataForAssetResource:resource toFile:[NSURL fileURLWithPath:savePath] options:nil completionHandler:^(NSError * _Nullable error)
             {
                 NSURL *url = [NSURL fileURLWithPath:savePath];
                 //                 [self.context resetPlayWithUrl:url];
                 
                 VideoPlayListModel* model = [[VideoPlayListModel alloc] init];
                 model.videoListType = JDCloudPlayerVideoListTypeLocal;
                 model.title = fileName;
                 model.imageUrl = nil;
                 NSMutableDictionary* dic = [NSMutableDictionary dictionary];
                 [dic setValue:@"标清" forKey:@"name"];
                 [dic setValue:@"SD" forKey:@"code"];
                 [dic setValue:@"mp4" forKey:@"format"];
                 [dic setValue:url.path forKey:@"playUrl"];
                 model.videoPlayURLArr = [NSMutableArray arrayWithObject:dic];
                 model.selectedPlayerDic = model.videoPlayURLArr[0];
                 [self.context playWithPlayerInfoModel:model];
             }];
        }
        
    }
    else
    {
#pragma mark TODO
        //需要处理更早的版本
        NSLog(@"该视频未能播放");
        
        // Fallback on earlier versions
    }
}



@end
