//
//  VideoListMainView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/10/30.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "VideoListMainView.h"

@interface VideoListMainView()<UIScrollViewDelegate>

@property (nonatomic, strong) UIScrollView *backgroundSV;               //直播、点播、本地视频、日志滑动SV
@property (nonatomic, strong) UIImageView *exchangeLineIV;              //蓝色切换条
@property (nonatomic, strong) UIImageView *downBorderIV;                //SV下面的黑边

@property (nonatomic, strong) NSArray<UIButton *> *buttons;             //直播、点播、本地视频、日志的button
@property (nonatomic, strong) UIButton *selectedButton;                 //选中的Button

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;     //播放器通过上下文传过来

@end

@implementation VideoListMainView

- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context
{
    self = [super initWithFrame:frame];
    if (self) {
        self.context = context;
        [self setupSubviews];
       
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)setupSubviews
{
    [self addSubview:self.backgroundSV];
    [self addSubview:self.exchangeLineIV];
    [self addSubview:self.downBorderIV];
    
    //滚动视图上的按钮
    self.buttons = [self creatButtonsWithCount:4];
}

//button
- (NSArray<UIButton *> *)creatButtonsWithCount:(NSInteger)count {
    
    CGFloat buttonWidth = ScreenWidth/4;
    NSArray* titleArr = @[@"直播",@"点播",@"本地视频",@"日志"];
    
    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:count];
    for (NSInteger index = 0; index < count; index++) {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(index * buttonWidth, 0, buttonWidth, self.frame.size.height);
        [button setTitle:titleArr[index] forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        button.titleLabel.font = [UIFont systemFontOfSize:15 * SCALE_FONT];
        [button addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
        button.tag = index + 100;
 
        [array addObject:button];
        [self addSubview:button];
    }
    return [array copy];
}

#pragma mark -
#pragma mark - 点击事件
//传过来的手势滑动结束
- (void)buttonClickActionWithButtonTag:(NSInteger)tag{
    UIButton* button = [[UIButton alloc]init];
    button.tag = tag;
    [self buttonClickAction:button];
}

//点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //直播
        {
            [UIView animateWithDuration:0.5 animations:^{
                self.exchangeLineIV.center = CGPointMake(ScreenWidth / 8, self.exchangeLineIV.center.y);
            }];
            
            if (self.delegate && [self.delegate respondsToSelector:@selector(videoListMainViewButtonClickAction:)]){
                [self.delegate videoListMainViewButtonClickAction:sender];
            }
        }
            break;
        case 101:   //点播
        {
            [UIView animateWithDuration:0.5 animations:^{
                self.exchangeLineIV.center = CGPointMake(ScreenWidth * 3 / 8, self.exchangeLineIV.center.y);
            }];
            
            if (self.delegate && [self.delegate respondsToSelector:@selector(videoListMainViewButtonClickAction:)]){
                [self.delegate videoListMainViewButtonClickAction:sender];
            }
        }
            break;
        case 102:   //本地视频
        {
            [UIView animateWithDuration:0.5 animations:^{
                self.exchangeLineIV.center = CGPointMake(ScreenWidth * 5 / 8, self.exchangeLineIV.center.y);
            }];
            
            if (self.delegate && [self.delegate respondsToSelector:@selector(videoListMainViewButtonClickAction:)]){
                [self.delegate videoListMainViewButtonClickAction:sender];
            }
        }
            break;
        case 103:   //播放日志
        {
            [UIView animateWithDuration:0.5 animations:^{
                self.exchangeLineIV.center = CGPointMake(ScreenWidth * 7 / 8, self.exchangeLineIV.center.y);
            }];
            
            if (self.delegate && [self.delegate respondsToSelector:@selector(videoListMainViewButtonClickAction:)]){
                [self.delegate videoListMainViewButtonClickAction:sender];
            }
        }
            break;
            
        default:
            break;
    }
}

#pragma mark -
#pragma mark - Lazy init
- (UIScrollView *)backgroundSV{
    if (!_backgroundSV) {
        _backgroundSV = [[UIScrollView alloc]initWithFrame:self.frame];
        _backgroundSV.backgroundColor = [UIColor clearColor];
        _backgroundSV.showsHorizontalScrollIndicator = NO;
        _backgroundSV.showsHorizontalScrollIndicator = NO;
        _backgroundSV.delegate = self;
        _backgroundSV.pagingEnabled = YES;
        _backgroundSV.contentSize = CGSizeMake(ScreenWidth, 0);
       
    }
    return _backgroundSV;
}

- (UIImageView *)exchangeLineIV{
    if (!_exchangeLineIV) {
        _exchangeLineIV = [[UIImageView alloc]initWithFrame:CGRectMake(0, self.frame.size.height - 3, 23, 3)];
        _exchangeLineIV.backgroundColor = RGBCOLOR(78, 185, 248);
    }
    return _exchangeLineIV;
}

- (UIImageView *)downBorderIV{
    if (!_downBorderIV) {
        _downBorderIV = [[UIImageView alloc]initWithFrame:CGRectMake(0, self.frame.size.height - 1, ScreenWidth, 1)];
        _downBorderIV.backgroundColor = [UIColor blackColor];
    }
    return _downBorderIV;
}


/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
