//
//  JDVideoPlayListTableViewCell.m
//  jdcloud_ios_live_demo
//
//  Created by pengdandan on 2018/9/17.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "VideoPlayListTableViewCell.h"
#import "SDWebImageDownloader.h"
#import "SDImageCache.h"
#import <Photos/PHImageManager.h>
#import <Photos/PHAsset.h>

@interface VideoPlayListTableViewCell()


@end

@implementation VideoPlayListTableViewCell

- (void)awakeFromNib {
    [super awakeFromNib];
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];
}

#pragma mark -
#pragma mark - 播放列表
- (void)setupVideoPlayListModel:(VideoPlayListModel *)model withSelectedRow:(NSInteger)selectedRow WithCurrRow:(NSInteger)currRow{
    
    //title
    self.videoTitleLabel.text = model.title;
    self.videoTitleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
    
    //time
    NSInteger playTime = [model.playTime integerValue];
    if(playTime != 0){
        self.videoTimeLabel.hidden = NO;
        self.videoTimeLabel.text = [JDStaticTools timeformatFromSeconds:playTime];
    }else{
        self.videoTimeLabel.hidden = YES;
    }
    
    //视频图片加载
    if (model.imageName.length > 0) {
        self.videoImageView.image = [UIImage imageNamed:model.imageName];
    }else{
        NSString *imgURL =  model.imageUrl;
        UIImage *cachedImage = [[SDImageCache sharedImageCache] imageFromDiskCacheForKey:imgURL];
        if ( !cachedImage ) {
            [self downloadImage:imgURL];
        } else {
            self.videoImageView.image = cachedImage;
        }
    }
    self.videoImageView.contentMode = UIViewContentModeScaleAspectFill;
    self.videoImageView.clipsToBounds = YES;
    
    //选中样式修改
    if(selectedRow == currRow){
        self.videoTitleLabel.textColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
    }else{
        self.videoTitleLabel.textColor = [UIColor whiteColor];
    }
}

// 利用 SDWebImage 框架提供的功能下载图片
- (void)downloadImage:(NSString *)imageURL {
    [[SDWebImageDownloader sharedDownloader] downloadImageWithURL:[NSURL URLWithString:imageURL] options:SDWebImageDownloaderUseNSURLCache progress:^(NSInteger receivedSize, NSInteger expectedSize, NSURL * _Nullable targetURL) {
        
    } completed:^(UIImage * _Nullable image, NSData * _Nullable data, NSError * _Nullable error, BOOL finished) {
        [[SDImageCache sharedImageCache] storeImage:image forKey:imageURL toDisk:YES completion:^{
            if ([NSThread isMainThread]) {
                self.videoImageView.image = image;
                
            }else{
                dispatch_async(dispatch_get_main_queue(), ^{
                    self.videoImageView.image = image;
                });
            }
        }];
    }];
}

#pragma mark -
#pragma mark - 离线视频
- (void)setupOffLineVideoModelArr:(PHFetchResult *)fetchResult withSelectedRow:(NSInteger)selectedRow WithCurrRow:(NSInteger)currRow{
    
    PHAsset *asset = fetchResult[currRow];
     //缩略图
    [[PHImageManager defaultManager] requestImageForAsset:asset targetSize:CGSizeMake(200, 200) contentMode:PHImageContentModeDefault options:nil resultHandler:^(UIImage * _Nullable result, NSDictionary * _Nullable info) {
        self.offLineImageView.image = result;
        self.offLineImageView.contentMode = UIViewContentModeScaleAspectFill;
        self.offLineImageView.clipsToBounds = YES;
    }];
    
    //时间
    NSTimeInterval duration = asset.duration;
    self.offLineUsedNumLabel.text = [JDStaticTools timeformatFromSeconds:duration];
    self.offLineUsedNumLabel.font = [UIFont systemFontOfSize:10 * SCALE_FONT];

    //title
    self.offLineTitleLabel.text = [(AVAsset*)asset valueForKey:@"filename"];
    self.offLineTitleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
    
    //选中样式修改
    if(selectedRow == currRow){
        self.offLineTitleLabel.textColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
    }else{
        self.offLineTitleLabel.textColor = [UIColor whiteColor];
    }
}

#pragma mark -
#pragma mark - 日志
- (void)setupLogModel:(JDCloudPlayLogModel *)model withSelectedRow:(NSInteger)selectedRow WithCurrRow:(NSInteger)currRow
{
    //日志time
    self.logTimeLabel.text = [model timeString];
    
    //日志title
    self.logTitleLabel.text = [model eventDescription];
}


@end
