//
//  JDCloudPlayerBottomView.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerBottomView.h"
//#import "JDVPlayer/JDCloudPlayerDefine.h"
#import "JDCloudPlayerProgressView.h"
#import "AppDelegate.h"
#import "JDCloudPlayerQualityListView.h"
#import "JDCloudPlayerSpeedListView.h"

//倍速变化通知
NSString *const JDVideoPlaySpeedChangNotification = @"JDVideoPlaySpeedChangNotification";
//清晰度变化通知
NSString *const JDVideoPlayQualityChangeNotification = @"JDVideoPlayQualityChangeNotification";

@interface JDCloudPlayerBottomView()<JDCloudPlayerVodProgressViewDelegate>

@property (nonatomic, strong) UIButton *playButton;                 //播放/暂停
@property (nonatomic, strong) UIButton *nextButton;                 //下一个视频
@property (nonatomic, strong) UILabel *leftTimeLabel;               //左侧时间
@property (nonatomic, strong) UILabel *rightTimeLabel;              //右侧时间
@property (nonatomic, strong) UILabel *fullScreenTimeLabel;         //全屏时时间
@property (nonatomic, strong) UIButton *speedButton;                //倍速
@property (nonatomic, strong) NSString *speedTitle;                 //倍速title
@property (nonatomic, strong) UIButton *qualityButton;              //清晰度
@property (nonatomic, strong) NSString *qualityTitle;               //清晰度title
@property (nonatomic, strong) UIButton *screenChangeButton;         //屏幕切换
@property (nonatomic, assign) BOOL isSeeking;
@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;

//开一个定时器，不断获取播放时长
@property (nonatomic ,strong) NSTimer* timer;

//播放状态
@property (nonatomic ,assign) BOOL playState;

@end

@implementation JDCloudPlayerBottomView


- (void)dealloc
{
    
}
    
- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context{
    if (self = [super initWithFrame:frame]) {
        self.context = context;
        [self addSubViews];
        [self setupData];
        [self addNotifications];
        self.progressCanUpdate = YES;
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupData
{
    UIButton* button = [[UIButton alloc]init];
    button.tag = 100;
    [self buttonClickAction:button];
    
    //时间
    [self starteTimer];
    
    //播放
    self.playState = YES;
    self.speedTitle = @"1倍速";
    self.qualityTitle = @"标清";
}

- (void)starteTimer
{
    if (_timer == nil) {
        self.timer = [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(getTimer:) userInfo:nil repeats:YES];
    }
    [[NSRunLoop mainRunLoop] addTimer:self.timer forMode:NSRunLoopCommonModes];
    [self.timer fire];
    self.playState = YES;

}

- (void)stopTimer
{
    if (_timer) {
        [self.timer invalidate];
        self.timer = nil;
        self.playState = NO;
    }
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)addSubViews
{
    [self addSubview:self.playButton];
    [self addSubview:self.nextButton];
    [self addSubview:self.leftTimeLabel];
    [self addSubview:self.rightTimeLabel];
    [self addSubview:self.fullScreenTimeLabel];
    [self addSubview:self.progressView];
    [self addSubview:self.speedButton];
    [self addSubview:self.qualityButton];
    [self addSubview:self.screenChangeButton];
}

- (void)layoutSubviews{
    [super layoutSubviews];
    
    float width = self.bounds.size.width;

    if ([JDStaticTools isInterfaceOrientationPortrait]) { //竖屏
        //隐藏下一个视频按钮、倍速、清晰度、横屏时间
        self.nextButton.hidden = YES;
        self.speedButton.hidden = YES;
        self.qualityButton.hidden = YES;
        self.fullScreenTimeLabel.hidden = YES;
        
        self.leftTimeLabel.hidden = NO;
        self.rightTimeLabel.hidden = NO;
        self.screenChangeButton.hidden = NO;
        self.playButton.hidden = NO;
        
        //设置播放、进度、时间、屏幕切换位置
        self.playButton.frame = CGRectMake(7, 7, 30, 30);
        self.screenChangeButton.frame = CGRectMake(width - 37, 7, 30, 30);
        self.leftTimeLabel.frame = CGRectMake(37, 0, 60, 44);
        self.rightTimeLabel.frame = CGRectMake(width - 37 - 60, 0, 60, 44);
        self.progressView.frame = CGRectMake(106, 17, width - 104 - 104, 44);
        [self.progressView.playSlider setThumbImage:[UIImage imageNamed:@"slider_thumb"] forState:UIControlStateNormal];
        
        if (self.context.infoModel.videoListType == JDCloudPlayerVideoListTypeLive) {
            self.rightTimeLabel.hidden = YES;
        }
    }
    else{ //横屏
        
        self.nextButton.hidden = NO;
        self.speedButton.hidden = NO;
        self.qualityButton.hidden = NO;
        self.fullScreenTimeLabel.hidden = NO;
        self.playButton.hidden = NO;
        
        self.leftTimeLabel.hidden = YES;
        self.rightTimeLabel.hidden = YES;
        self.screenChangeButton.hidden = YES;
        
        
        //设置播放、进度、时间、屏幕切换、下一个视频按钮、倍速、清晰度的位置
        self.progressView.frame = CGRectMake(10, 0, width - 20,20);
        [self.progressView.playSlider setThumbImage:[UIImage imageNamed:@"slider_thumb2"] forState:UIControlStateNormal];
        self.playButton.frame = CGRectMake(10, 15, 44, 44);
        self.nextButton.frame = CGRectMake(50, 17, 40, 40);
        self.fullScreenTimeLabel.frame = CGRectMake(98, 15, 140, 44);
        self.qualityButton.frame = CGRectMake(width - 70, 15, 60, 44);
        self.speedButton.frame = CGRectMake(width - 160, 15, 100, 44);
        
        if(self.context.lastSelecterVideoListType == JDCloudPlayerVideoListTypeLive){
            self.speedButton.hidden = YES;
        }
        if(self.context.lastSelecterVideoListType == JDCloudPlayerVideoListTypeScan){
            self.speedTitle = @"1倍速";
            [self.speedButton setTitle:self.speedTitle forState:UIControlStateNormal];
            self.rightTimeLabel.hidden = YES;
        }
    }
    [self.qualityButton setTitle:[self qualityButtonTitle] forState:UIControlStateNormal];
    
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //播放/暂停按钮
        {
            if (self.playState) {
                button.selected = YES;
                [self.context.jdCloudPlayer pause];
                self.playState = NO;
            }else{
                button.selected = NO;
                if (self.context.jdCloudPlayer.isPreparedToPlay) {
                    [self.context.jdCloudPlayer start];
                } else {
                    [self.context.jdCloudPlayer replay];
                }
                [self.context showLoadingView];
                self.playState = YES;
            }
        }
            break;
        case 101:   //下一个视频
        {
            [self.context videoPlayNextURL];
            self.speedTitle = @"1倍速";
            [self.speedButton setTitle:self.speedTitle forState:UIControlStateNormal];
            self.qualityTitle = @"标清";
            [self.qualityButton setTitle:[self qualityButtonTitle] forState:UIControlStateNormal];
        }
            break;
        case 102:   //倍速
        {
            for (UIView* view in self.superview.subviews) {
                if ([view isKindOfClass:[JDCloudPlayerSpeedListView class]]) {
                    [view removeFromSuperview];
                }
            }
            
            JDCloudPlayerSpeedListView* speedListView = [[JDCloudPlayerSpeedListView alloc]initWithFrame:CGRectZero withContext:self.context selectedTitle:self.speedTitle];
            [self.superview insertSubview:speedListView atIndex:[self.superview.subviews count]];
            
        }
            break;
        case 103:   //清晰度
        {
            for (UIView* view in self.superview.subviews) {
                if ([view isKindOfClass:[JDCloudPlayerQualityListView class]]) {
                    [view removeFromSuperview];
                }
            }
            JDCloudPlayerQualityListView* qualityListView = [[JDCloudPlayerQualityListView alloc]initWithFrame:CGRectZero withContext:self.context selectedTitle:[self qualityButtonTitle]];
            [self.superview insertSubview:qualityListView atIndex:[self.superview.subviews count]];

        }
            break;
        case 104:   //屏幕切换
        {
            //屏幕变大
            NSDictionary *dataDic = nil;
            if ([JDStaticTools isInterfaceOrientationPortrait]) {
                dataDic = [NSDictionary dictionaryWithObject:@"YES" forKey:@"isFullScreen"];
;
            } else {
                dataDic = [NSDictionary dictionaryWithObject:@"NO" forKey:@"isFullScreen"];
            }
            [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayScreenChangeNotification" object:nil userInfo:dataDic];
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - 通知
- (void)addNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(speedChang:) name:JDVideoPlaySpeedChangNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(qualityChange:) name:JDVideoPlayQualityChangeNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(gestureChange:) name:@"JDVideoPlayGestureNotification" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(stopTimer) name:@"JDVideoPlayRemoveTimeNotification" object:nil];
}

- (void)speedChang:(NSNotification *)infoNotification
{
    NSDictionary *dic = [infoNotification userInfo];
    self.speedTitle = [dic objectForKey:@"title"];
    [self.speedButton setTitle:self.speedTitle forState:UIControlStateNormal];
    if ([self.speedTitle isEqualToString:@"2倍速"]) {
        self.context.jdCloudPlayer.playRate = 2.0;
    } else if ([self.speedTitle isEqualToString:@"1.5倍速"]) {
        self.context.jdCloudPlayer.playRate = 1.5;
    } else if ([self.speedTitle isEqualToString:@"1.25倍速"]) {
        self.context.jdCloudPlayer.playRate = 1.25;
    } else if ([self.speedTitle isEqualToString:@"1倍速"]) {
        self.context.jdCloudPlayer.playRate = 1.0;
    } else if ([self.speedTitle isEqualToString:@"0.75倍速"]) {
        self.context.jdCloudPlayer.playRate = 0.75;
    } else if ([self.speedTitle isEqualToString:@"0.5倍速"]) {
        self.context.jdCloudPlayer.playRate = 0.5;
    } else {
        self.context.jdCloudPlayer.playRate = 1.0;
    }
}

- (void)qualityChange:(NSNotification *)infoNotification
{
    //刷新清晰度
    [self.qualityButton setTitle:[self qualityButtonTitle] forState:UIControlStateNormal];
}

- (void)gestureChange:(NSNotification *)infoNotification
{
    NSDictionary *dic = [infoNotification userInfo];
    self.isSeeking = [[dic objectForKey:@"isSeeking"] boolValue];
}

#pragma mark -
#pragma mark - 功能函数
//更新时间
- (void)getTimer:(NSTimer*)timer{
    if (self.progressCanUpdate) {
        
        //扫码播放 隐藏清晰度
        if(self.context.infoModel.videoListType == JDCloudPlayerVideoListTypeScan){
            self.qualityButton.hidden = YES;
        }else{
            self.qualityButton.hidden = NO;
        }
        
        self.leftTimeLabel.text = [JDStaticTools stringWithNSTimeInterval:self.context.jdCloudPlayer.currentPlayerTime];
        self.rightTimeLabel.text = [JDStaticTools stringWithNSTimeInterval:self.context.jdCloudPlayer.duration];
        if([self.rightTimeLabel.text isEqualToString:@"00:00"]){ //直播 隐藏倍速 进度条
            self.fullScreenTimeLabel.text = [NSString stringWithFormat:@"%@",self.leftTimeLabel.text];
            self.rightTimeLabel.hidden = YES;
            self.progressView.hidden = YES;
            self.speedButton.hidden = YES;
        }else{
            self.progressView.hidden = NO;
            self.speedButton.hidden = NO;
            self.fullScreenTimeLabel.text = [NSString stringWithFormat:@"%@/%@",self.leftTimeLabel.text,self.rightTimeLabel.text];
            self.progressView.progress = (float)self.context.jdCloudPlayer.currentPlayerTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);
            self.progressView.loadTimeProgress = (float)self.context.jdCloudPlayer.loadedTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);
            
            self.currentTime = self.context.jdCloudPlayer.currentPlayerTime;
            self.duration = self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT;
            self.context.infoModel.progress = self.progressView.progress;
            if (self.currentTime > self.context.jdCloudPlayer.duration ||
                self.currentTime == self.context.jdCloudPlayer.duration) {
                self.progressView.progress = 0.0;
            }
            if (self.progressView.progress == 1.0 || self.progressView.progress > 1.0) {
                self.progressView.progress = 0.0;
            }
            if (self.currentTime == self.context.jdCloudPlayer.duration && self.context.jdCloudPlayer.duration > 0) {
            }
        }
    }
}

/*
 * 功能 ：更新进度条
 * 参数 ：currentTime 当前播放时间
         TotalTime 播放总时长
 */
- (void)updateProgressWithCurrentTime:(float)currentTime TotalTime:(float)totalTime
{
    NSString *curTimeStr = [JDStaticTools stringWithNSTimeInterval:currentTime];
    NSString *totalTimeStr = [JDStaticTools stringWithNSTimeInterval:totalTime];
    self.leftTimeLabel.text = curTimeStr;
    self.rightTimeLabel.text = totalTimeStr;
    self.fullScreenTimeLabel.text = [NSString stringWithFormat:@"%@/%@", curTimeStr, totalTimeStr];
    self.progressView.progress = currentTime/totalTime;
}


/*
 * 功能 ：根据播放器状态，改变状态
 * 参数 ：state 播放器状态
 */
- (void)updateViewWithPlayerState:(JDCloudPlayerState)state {
    switch (state) {
        case JDCloudPlayerStatePlay:  //播放
        {
            self.playState = YES;
            [self.playButton setSelected:YES];
            [self.qualityButton  setUserInteractionEnabled:YES];
            [self.progressView setUserInteractionEnabled:YES];
        }
            break;
        case JDCloudPlayerStatePause:  //暂停
        {
            self.playState = NO;
            [self.playButton setSelected:NO];
            [self.progressView setUserInteractionEnabled:YES];
        }
            break;
        case JDCloudPlayerStateStop:  //停止
        {
            self.playState = NO;
            [self.playButton setSelected:NO];
            [self.qualityButton setUserInteractionEnabled:NO];
            [self.progressView setUserInteractionEnabled:NO];
            
        }
            break;
        default:
            break;
    }
}

/*
 * 功能 ：锁屏状态
 * 参数 ：isScreenLocked 是否是锁屏状态
         fixedPortrait  是否是绝对竖屏状态
 */
- (void)lockScreenWithIsScreenLocked:(BOOL)isScreenLocked fixedPortrait:(BOOL)fixedPortrait{
    if (!isScreenLocked)  //锁屏
    {
        self.playButton.hidden = NO;
        self.nextButton.hidden = NO;
        self.leftTimeLabel.hidden = NO;
        self.rightTimeLabel.hidden = NO;
        self.leftTimeLabel.hidden = NO;
        self.fullScreenTimeLabel.hidden = NO;
        if (self.context.infoModel.videoListType != JDCloudPlayerVideoListTypeLive) {
            self.progressView.hidden = NO;
        }
        self.speedButton.hidden = NO;
        self.qualityButton.hidden = NO;
        self.screenChangeButton.hidden = NO;
        
    }else{
        
        self.playButton.hidden = YES;
        self.nextButton.hidden = YES;
        self.rightTimeLabel.hidden = YES;
        self.leftTimeLabel.hidden = YES;
        self.fullScreenTimeLabel.hidden = YES;
        if (self.context.infoModel.videoListType != JDCloudPlayerVideoListTypeLive) {
            self.progressView.hidden = YES;
        }
        self.speedButton.hidden = YES;
        self.qualityButton.hidden = YES;
        self.screenChangeButton.hidden = YES;
    }
}

//获取清晰度的title
- (NSString*)qualityButtonTitle{
    
    NSString* title = @"标清";
    title = self.context.infoModel.selectedPlayerDic[@"name"];
    return title;
}


#pragma mark -
#pragma mark - 懒加载
//播放/暂停按钮
- (UIButton *)playButton{
    if (!_playButton) {
        _playButton = [[UIButton alloc] init];
        [_playButton setBackgroundImage:[UIImage imageNamed:@"竖屏-开始"] forState:UIControlStateNormal];
        [_playButton setBackgroundImage:[UIImage imageNamed:@"竖屏-暂停"] forState:UIControlStateSelected];
        _playButton.tag = 100;
        _playButton.selected = NO;
        [_screenChangeButton sizeToFit];
        [_playButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _playButton;
}

//下一个视频
- (UIButton *)nextButton{
    if (!_nextButton) {
        _nextButton = [[UIButton alloc] init];
        [_nextButton setBackgroundImage:[UIImage imageNamed:@"next_video"] forState:UIControlStateNormal];
        [_nextButton setBackgroundImage:[UIImage imageNamed:@"next_video"] forState:UIControlStateSelected];
        _nextButton.tag = 101;
        [_nextButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _nextButton;
}

//播放时间
- (UILabel *)leftTimeLabel{
    if (!_leftTimeLabel) {
        _leftTimeLabel = [[UILabel alloc] init];
        [_leftTimeLabel setFont:[UIFont systemFontOfSize:12 * SCALE_FONT]];
        _leftTimeLabel.text = @"00:00";
        _leftTimeLabel.textColor = [UIColor whiteColor];
        _leftTimeLabel.textAlignment = NSTextAlignmentCenter;
    }
    return _leftTimeLabel;
}

//总时间
- (UILabel *)rightTimeLabel{
    if (!_rightTimeLabel) {
        _rightTimeLabel = [[UILabel alloc] init];
        [_rightTimeLabel setFont:[UIFont systemFontOfSize:12 * SCALE_FONT]];
        _rightTimeLabel.text = @"00:00";
        _rightTimeLabel.textColor = [UIColor whiteColor];
        _rightTimeLabel.textAlignment = NSTextAlignmentCenter;
    }
    return _rightTimeLabel;
}

//播放时间/总时间
- (UILabel *)fullScreenTimeLabel{
    if (!_fullScreenTimeLabel) {
        _fullScreenTimeLabel = [[UILabel alloc] init];
        [_fullScreenTimeLabel setFont:[UIFont systemFontOfSize:12 * SCALE_FONT]];
        _fullScreenTimeLabel.text = @"00:00/00:00";
        _fullScreenTimeLabel.textColor = [UIColor whiteColor];
        _fullScreenTimeLabel.textAlignment = NSTextAlignmentLeft;
    }
    return _fullScreenTimeLabel;
}

//进度条
- (JDCloudPlayerProgressView *)progressView{
    if (!_progressView){
        _progressView = [[JDCloudPlayerProgressView alloc] init];
        _progressView.progress = 0.0;
        _progressView.delegate = self;
    }
    return _progressView;
}

//倍速
- (UIButton *)speedButton{
    if (!_speedButton) {
        _speedButton = [[UIButton alloc] init];
        [_speedButton setTag:102];
        _speedButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentRight;
        _speedButton.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_speedButton setTitle:@"1倍速" forState:UIControlStateNormal];
        [_speedButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _speedButton;
}

//清晰度
- (UIButton *)qualityButton{
    if (!_qualityButton) {
        _qualityButton = [[UIButton alloc] init];
        [_qualityButton setTag:103];
        _qualityButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentRight;
        _qualityButton.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_qualityButton setTitle:[self qualityButtonTitle] forState:UIControlStateNormal];
        [_qualityButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _qualityButton;
}

//屏幕切换
- (UIButton *)screenChangeButton{
    if (!_screenChangeButton) {
        _screenChangeButton = [[UIButton alloc] init];
        [_screenChangeButton setTag:104];
        [_screenChangeButton setBackgroundImage:[UIImage imageNamed:@"竖屏-全屏"] forState:UIControlStateNormal];
        [_screenChangeButton setBackgroundImage:[UIImage imageNamed:@"竖屏-全屏"] forState:UIControlStateSelected];
        [_screenChangeButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
        [_screenChangeButton sizeToFit];
    }
    return _screenChangeButton;
}

- (void)setProgress:(float)progress{
    _progress = progress;
    self.progressView.progress = progress;
}

- (void)setLoadTimeProgress:(float)loadTimeProgress{
    _loadTimeProgress = loadTimeProgress;
    self.progressView.loadTimeProgress = loadTimeProgress;
}

- (void)setIsPortrait:(BOOL)isPortrait{
    _isPortrait = isPortrait;
    [self setNeedsLayout];
    [self layoutIfNeeded];
}


#pragma mark - JDCloudPlayerVodProgressViewDelegate
- (void)jdCloudVodProgressView:(JDCloudPlayerProgressView *)progressView dragProgressSliderValue:(float)value event:(UIControlEvents)event
{
    switch (event) {
        case UIControlEventTouchDown:
        {
            //slider 手势按下时，不做隐藏操作
            self.progressCanUpdate = NO;
        }
            break;
        case UIControlEventValueChanged:
        {
            self.progressCanUpdate = NO;
            if (value*self.context.jdCloudPlayer.duration < self.currentTime) {
                self.loadTimeProgress = value;
            }
            [self updateProgressWithCurrentTime:value*self.context.jdCloudPlayer.duration TotalTime:self.context.jdCloudPlayer.duration];
            self.context.infoModel.progress = self.currentTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);
        }
            break;
        case UIControlEventTouchUpInside:
        {
            //slider滑动结束后，
            self.progressCanUpdate = NO;
            if (value*self.context.jdCloudPlayer.duration < self.currentTime) {
                self.loadTimeProgress = value;
            }
            [self updateProgressWithCurrentTime:value*self.context.jdCloudPlayer.duration TotalTime:self.context.jdCloudPlayer.duration];
            self.context.infoModel.progress = self.currentTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);
//            [self.context.jdCloudPlayer seekToTime:value*self.context.jdCloudPlayer.duration];
        }
            break;
        case UIControlEventTouchUpOutside:{
            self.progressCanUpdate = NO;
            if (value*self.context.jdCloudPlayer.duration < self.currentTime) {
                self.loadTimeProgress = value;
            }
            [self updateProgressWithCurrentTime:value*self.context.jdCloudPlayer.duration TotalTime:self.context.jdCloudPlayer.duration];
            self.context.infoModel.progress = self.currentTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);

//            [self.context.jdCloudPlayer seekToTime:value*self.context.jdCloudPlayer.duration];
        }
            break;
            //点击事件
        case UIControlEventTouchDownRepeat:{
            self.progressCanUpdate = NO;
            if (value*self.context.jdCloudPlayer.duration < self.currentTime) {
                self.loadTimeProgress = value;
            }
            [self updateProgressWithCurrentTime:value*self.context.jdCloudPlayer.duration TotalTime:self.context.jdCloudPlayer.duration];
            self.context.infoModel.progress = self.currentTime/(self.context.jdCloudPlayer.duration > 0 ? self.context.jdCloudPlayer.duration : MAXFLOAT);
        }
            break;
        case UIControlEventEditingDidEnd:
        {
            [self.context.jdCloudPlayer seekToTime:value*self.context.jdCloudPlayer.duration];

        }
            break;
        default:
            self.progressCanUpdate = NO;
            break;
    }
}


@end
