//
//  JDCloudPlayerQualityListView.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerQualityListView.h"

@interface JDCloudPlayerQualityListView ()

@property (nonatomic, strong) NSArray<UIButton *> *buttons;       //总共的清晰度个数
@property (nonatomic, strong) NSMutableArray* btnTitleArr;        //总共的清晰度的titleArr
@property (nonatomic, strong) NSString *selectedTitle;            //传过来的选中的清晰度title
@property (nonatomic, strong) UIButton *selectedButton;           //上次选中的按钮
@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;

@end

@implementation JDCloudPlayerQualityListView

- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context selectedTitle:(NSString*)title{
    if (self = [super initWithFrame:frame]) {
        self.frame = CGRectMake(ScreenWidth - 250, 0, 250, ScreenHeight);
        self.context = context;
        self.selectedTitle = title;
        [self addSubViews];
        
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)addSubViews
{
    self.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);

    //高清、标清、流畅
    self.buttons = [self creatButtonsWithCount:self.btnTitleArr.count];
    
    //选中的button
    for (int i = 0; i < self.btnTitleArr.count; i++) {
        if ([self.btnTitleArr[i][@"name"] isEqualToString:self.selectedTitle]) {
            self.selectedButton = self.buttons[i];
            self.selectedButton.selected = YES;
            return;
        }
    }
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    if([self.selectedButton.titleLabel.text isEqualToString:button.titleLabel.text]){ //之前选中的和现在选中的一样
        [self removeFromSuperview];
    }
    else{
        self.selectedButton.selected = NO;
        button.selected = YES;
        self.selectedButton = button;
        self.context.infoModel.selectedPlayerDic = self.btnTitleArr[self.selectedButton.tag - 100];
        [self.context playWithPlayerInfoModel:self.context.infoModel];
        
        NSDictionary *dataDic = [NSDictionary dictionaryWithObject:button.titleLabel.text forKey:@"title"];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayQualityChangeNotification" object:nil userInfo:dataDic];
        
        [self removeFromSuperview];
    }
}

#pragma mark -
#pragma mark - 功能函数
//button
- (NSArray<UIButton *> *)creatButtonsWithCount:(NSInteger)count {
    
    CGFloat height = (ScreenHeight - 30)/8;

    NSMutableArray<UIButton *> *array = [NSMutableArray arrayWithCapacity:count];

    for (NSInteger index = 0; index < count; index++)
    {
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.frame = CGRectMake(0, 30 + index * height, self.frame.size.width, height);
        button.tag = 100 + index;
        [button setTitle:self.btnTitleArr[index][@"name"] forState:UIControlStateNormal];
        [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [button setTitleColor:[JDStaticTools  colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        button.titleLabel.font = [UIFont systemFontOfSize:16.0 * SCALE_FONT];
        [button addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];

        [array addObject:button];
        [self addSubview:button];
    }
    return [array copy];
}

//点击其他区域 view移除
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
   // UITouch *touch  = touches.anyObject;
   // if ([touch.view isKindOfClass:[self class]]) {
        [self removeFromSuperview];
   // }
}

//视频格式转
- (NSInteger)playerVideoFormatWithString:(NSString*)format{
    if ([format isEqualToString:@"m3u8"] || [format isEqualToString:@"M3U8"] || [format isEqualToString:@"M3u8"]) {
        return JDCloudPlayerVideoFormatM3u8;
    }else if ([format isEqualToString:@"mp4"] || [format isEqualToString:@"MP4"] || [format isEqualToString:@"Mp4"]) {
        return JDCloudPlayerVideoFormatMP4;
    }else if ([format isEqualToString:@"rtmp"] || [format isEqualToString:@"Rtmp"] || [format isEqualToString:@"RTMP"] ) {
        return JDCloudPlayerVideoFormatRtmp;
    }else if  ([format isEqualToString:@"flv"] || [format isEqualToString:@"Flv"] || [format isEqualToString:@"FLV"] ) {
        return JDCloudPlayerVideoFormatFlv;
    }else{
        return JDCloudPlayerVideoFormatOther;
    }
}

#pragma mark -
#pragma mark - 懒加载
- (NSMutableArray *)btnTitleArr{
    if (!_btnTitleArr) {
        _btnTitleArr = [NSMutableArray array];
        _btnTitleArr = [self arrayVideoPlayURLArr:self.context.infoModel.videoPlayURLArr];
    }
    return _btnTitleArr;
}

//videoPlayURLArr中排序 流畅-超清
- (NSMutableArray*)arrayVideoPlayURLArr:(NSMutableArray*)videoPlayURLArr{
    NSMutableArray* arr = [NSMutableArray array];
    for (NSMutableDictionary* dic in videoPlayURLArr) {
        if ([[dic objectForKey:@"name"] isEqualToString:@"流畅"]) {
            [arr addObject:dic];
        }
    }
    for (NSMutableDictionary* dic in videoPlayURLArr) {
        if ([[dic objectForKey:@"name"] isEqualToString:@"标清"]) {
            [arr addObject:dic];
        }
    }
    for (NSMutableDictionary* dic in videoPlayURLArr) {
        if ([[dic objectForKey:@"name"] isEqualToString:@"高清"]) {
            [arr addObject:dic];
        }
    }
    for (NSMutableDictionary* dic in videoPlayURLArr) {
        if ([[dic objectForKey:@"name"] isEqualToString:@"超清"]) {
            [arr addObject:dic];
        }
    }
    return arr;
}



@end
