//
//  JDCloudPlayerProgressView.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerProgressView.h"

static const CGFloat JDCloudPlayerViewProgressViewLoadtimeViewLeft      = 2 ;  //loadtimeView 左侧距离父视图距离
static const CGFloat JDCloudPlayerViewProgressViewLoadtimeViewTop       = 4 ; //loadtimeView 顶部距离父视图距离
static const CGFloat JDCloudPlayerViewProgressViewLoadtimeViewHeight    = 2 ;  //loadtimeView 高度
static const CGFloat JDCloudPlayerViewProgressViewPlaySliderTop         = 0 ; //playSlider 顶部距离父视图距离
static const CGFloat JDCloudPlayerViewProgressViewPlaySliderHeight      = 10 ; //playSlider 高度


@interface JDCloudPlayerProgressView()<JDCloudPlayerViewSliderDelegate>

@property (nonatomic, strong) UIProgressView *loadtimeView; //缓冲条，loadTime
@end

@implementation JDCloudPlayerProgressView

- (void)dealloc
{
    
}

- (UIProgressView *)loadtimeView{
    if (!_loadtimeView) {
        _loadtimeView = [[UIProgressView alloc] initWithProgressViewStyle:UIProgressViewStyleDefault];
        _loadtimeView.progress = 0.0;
        //设置它的风格，为默认的
        _loadtimeView.trackTintColor= [UIColor clearColor];
        //设置轨道的颜色
        _loadtimeView.progressTintColor= [JDStaticTools  colorWithHexString:kJDVCThemeColor];
    }
    return _loadtimeView;
}

- (JDCloudPlayerViewSlider *)playSlider{
    if (!_playSlider) {
        _playSlider = [[JDCloudPlayerViewSlider alloc] init];
        _playSlider.value = 0.0;
        //thumb左侧条的颜色
        _playSlider.minimumTrackTintColor = [JDCloudPlayerProgressView colorWithHexString:@"00c1de"];
        _playSlider.maximumTrackTintColor = [UIColor colorWithWhite:0.5 alpha:0.2];
        //thumb图片
        [_playSlider setThumbImage:[UIImage imageNamed:@"slider_thumb"] forState:UIControlStateNormal];
        //手指落下
        [_playSlider addTarget:self action:@selector(progressSliderDownAction:) forControlEvents:UIControlEventTouchDown];
        //手指抬起
        [_playSlider addTarget:self action:@selector(progressSliderUpAction:) forControlEvents:UIControlEventTouchUpInside];
        //value发生变化
        [_playSlider addTarget:self action:@selector(updateProgressSliderAction:) forControlEvents:UIControlEventValueChanged];
        
        [_playSlider addTarget:self action:@selector(cancelProgressSliderAction:) forControlEvents:UIControlEventTouchCancel];
        //手指在外面抬起
        [_playSlider addTarget:self action:@selector(updateProgressUpOUtsideSliderAction:) forControlEvents:UIControlEventTouchUpOutside];
        
        [_playSlider addTarget:self action:@selector(updateProgressEditingDidEndAction:) forControlEvents:UIControlEventEditingDidEnd];
    }
    return _playSlider;
}

#pragma mark - init
- (instancetype)init{
    return  [self initWithFrame:CGRectZero];
}

- (instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame]) {
        [self addSubview:self.loadtimeView];
        
        self.playSlider.sliderDelegate = self;
        [self addSubview:self.playSlider];
    }
    return self;
}

- (void)layoutSubviews{
//    self.loadtimeView.frame = CGRectMake(JDCloudPlayerViewProgressViewLoadtimeViewLeft,JDCloudPlayerViewProgressViewLoadtimeViewTop, self.bounds.size.width-2*JDCloudPlayerViewProgressViewLoadtimeViewLeft, JDCloudPlayerViewProgressViewLoadtimeViewHeight);
    self.loadtimeView.frame = CGRectMake(0,JDCloudPlayerViewProgressViewLoadtimeViewTop, self.bounds.size.width, JDCloudPlayerViewProgressViewLoadtimeViewHeight);
    self.playSlider.frame = CGRectMake(0,JDCloudPlayerViewProgressViewPlaySliderTop, self.bounds.size.width, JDCloudPlayerViewProgressViewPlaySliderHeight);
}


#pragma mark - 重写setter方法
//- (void)setSkin:(JDCloudPlayerViewSkin)skin{
   // _skin = skin;
    //    self.playSlider.minimumTrackTintColor = [AliyunUtil textColor:skin];
    //    [self.playSlider setThumbImage:[AliyunUtil imageWithNameInBundle:@"al_play_settings_radiobtn_normal" skin:skin] forState:UIControlStateNormal];

 //   [self.playSlider setThumbImage:[UIImage imageNamed:@"alivcPlayThumb"] forState:UIControlStateNormal];

 //   self.playSlider.minimumTrackTintColor = [UIColor colorWithHexString:@"00c1de"];

//}

- (void)setProgress:(float)progress{
    _progress = progress;
    [self.playSlider setValue:progress animated:NO];
}

- (void)setLoadTimeProgress:(float)loadTimeProgress{
    _loadTimeProgress = loadTimeProgress;
    [self.loadtimeView setProgress:loadTimeProgress];
}


#pragma mark - public method
/*
 * 功能 ：更新进度条
 * 参数 ：currentTime 当前播放时间
 durationTime 播放总时长
 */
- (void)updateProgressWithCurrentTime:(float)currentTime durationTime:(float)durationTime{
    [self.playSlider setValue:currentTime/durationTime animated:NO];
}

#pragma mark - slider action
- (void)progressSliderDownAction:(UISlider *)sender{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventTouchDown];
    }
}

- (void)updateProgressSliderAction:(UISlider *)sender{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventValueChanged];
    }
}

- (void)progressSliderUpAction:(UISlider *)sender{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventTouchUpInside];
    }
}

- (void)updateProgressUpOUtsideSliderAction:(UISlider *)sender{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventTouchUpOutside];
    }
}

- (void)updateProgressEditingDidEndAction:(UISlider *)sender
{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventEditingDidEnd];
    }
}

- (void)cancelProgressSliderAction:(UISlider *)sender{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sender.value event:UIControlEventTouchCancel];
    }
}


- (void)jdCloudPlayerViewSlider:(JDCloudPlayerViewSlider *)slider clickedSlider:(float)sliderValue{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sliderValue event:UIControlEventTouchDownRepeat]; //实际是点击事件
    }
}

- (void)jdCloudPlayerViewSlider:(JDCloudPlayerViewSlider *)slider endTouchSlider:(float)sliderValue
{
    if (self.delegate && [self.delegate respondsToSelector:@selector(jdCloudVodProgressView:dragProgressSliderValue:event:)]) {
        [self.delegate jdCloudVodProgressView:self dragProgressSliderValue:sliderValue event:UIControlEventEditingDidEnd]; //实际是点击事件
    }
}

+ (UIColor *)colorWithHexString:(NSString *)hexStr {
    CGFloat r, g, b, a;
    if (hexStrToRGBA(hexStr, &r, &g, &b, &a)) {
        return [UIColor colorWithRed:r green:g blue:b alpha:a];
    }
    return nil;
}


static BOOL hexStrToRGBA(NSString *str,
                         CGFloat *r, CGFloat *g, CGFloat *b, CGFloat *a) {
    NSCharacterSet *set = [NSCharacterSet whitespaceAndNewlineCharacterSet];
    str = [[str stringByTrimmingCharactersInSet:set] uppercaseString];
    if ([str hasPrefix:@"#"]) {
        str = [str substringFromIndex:1];
    } else if ([str hasPrefix:@"0X"]) {
        str = [str substringFromIndex:2];
    }
    
    NSUInteger length = [str length];
    //         RGB            RGBA          RRGGBB        RRGGBBAA
    if (length != 3 && length != 4 && length != 6 && length != 8) {
        return NO;
    }
    
    //RGB,RGBA,RRGGBB,RRGGBBAA
    if (length < 5) {
        *r = hexStrToInt([str substringWithRange:NSMakeRange(0, 1)]) / 255.0f;
        *g = hexStrToInt([str substringWithRange:NSMakeRange(1, 1)]) / 255.0f;
        *b = hexStrToInt([str substringWithRange:NSMakeRange(2, 1)]) / 255.0f;
        if (length == 4)  *a = hexStrToInt([str substringWithRange:NSMakeRange(3, 1)]) / 255.0f;
        else *a = 1;
    } else {
        *r = hexStrToInt([str substringWithRange:NSMakeRange(0, 2)]) / 255.0f;
        *g = hexStrToInt([str substringWithRange:NSMakeRange(2, 2)]) / 255.0f;
        *b = hexStrToInt([str substringWithRange:NSMakeRange(4, 2)]) / 255.0f;
        if (length == 8) *a = hexStrToInt([str substringWithRange:NSMakeRange(6, 2)]) / 255.0f;
        else *a = 1;
    }
    return YES;
}

static inline NSUInteger hexStrToInt(NSString *str) {
    uint32_t result = 0;
    sscanf([str UTF8String], "%X", &result);
    return result;
}

@end
