//
//  JDCloudPlayerViewGestureView.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerGestureView.h"
#import "JDCloudPrivateDefine.h"
#import "JDCloudPlayerSeekPopupView.h"
#import <MediaPlayer/MediaPlayer.h>
#import "JDCloudPlayerGestureModel.h"
#import "JDCloudBrightnessView.h"

static const CGFloat JDCloudGestureViewBrightImageViewWidth        = 125;    //亮度view宽度
static const CGFloat JDCloudGestureViewBrightProgressHeight        = 20;     //亮度进度条高度
static const CGFloat JDCloudGestureViewBrightProgressLeft          = 15;     //亮度进度条 相对父试图左侧距离
static const CGFloat JDCloudGestureViewBrightProgressBottom        = 10;     //亮度进度条 距离父视图底边距离
//自定义滑动方向：未知、水平、垂直。 默认：未知
static JDCloudOrientation moveOrientation = JDCloudOrientationUnknow;

//在屏幕的左边 or 右边
static UISwipeGestureRecognizerDirection direction = UISwipeGestureRecognizerDirectionUp;

NSString *const JDVideoPlayGestureNotification = @"JDVideoPlayGestureNotification";


@interface JDCloudPlayerGestureView ()<UIGestureRecognizerDelegate,JDCloudPlayerGestureModelDelegate>

@property (nonatomic, strong) UITapGestureRecognizer *singleClick;       //单击手势
@property (nonatomic, strong) UITapGestureRecognizer *doubleClick;       //双击手势
@property (nonatomic, strong) UIPanGestureRecognizer *panGesture;        //滑动手势
@property (nonatomic, assign) CGPoint startPoint;                        //手势开始的点

@property (nonatomic, strong) MPVolumeView *volumeView;                  //音量控件
@property (nonatomic, strong) UISlider *volumeSlider;                    //音量控制器
@property (nonatomic, assign) float systemVolume;                        //声音值
@property (nonatomic, assign) float systemBrightness;                    //亮度值

@property (nonatomic, strong) JDCloudPlayerSeekPopupView *seekView;       //前进/后退
/*
 * 功能 ：手势
 */
@property (nonatomic, strong) JDCloudPlayerGestureModel *gestureModel;

@end

@implementation JDCloudPlayerGestureView

#pragma mark - 懒加载
- (JDCloudPlayerSeekPopupView *)seekView{
    if (!_seekView) {
        _seekView = [[JDCloudPlayerSeekPopupView alloc] init];
    }
    return _seekView;
}

- (JDCloudPlayerGestureModel *)gestureModel{
    if (!_gestureModel) {
        _gestureModel = [[JDCloudPlayerGestureModel alloc] init];
    }
    return _gestureModel;
}

- (instancetype)init {
    return [self initWithFrame:CGRectZero];
}

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self addSubviews];
        
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)addSubviews
{
    [JDCloudBrightnessView sharedBrightnessView];
    
    [self.gestureModel setView:self];
    self.gestureModel.delegate = self;
    
    self.volumeView = [[MPVolumeView alloc] init];
    for (UIView *view in [self.volumeView subviews]) {
        if ([view.class.description isEqualToString:@"MPVolumeSlider"]) {
            self.volumeSlider = (UISlider *)view;
            break;
        }
    }
    [self.volumeView setFrame:CGRectZero];
    [self.volumeView setShowsVolumeSlider:NO];
    [self.volumeView sizeToFit];
}


- (void)setEnableGesture:(BOOL)enableGesture {
    [self.gestureModel setEnableGesture:enableGesture];
}

- (void)setSeekTime:(NSTimeInterval)time direction : (UISwipeGestureRecognizerDirection)direction{
    [self.seekView setSeekTime:time direction:direction];
}

#pragma mark - private method
- (void)setBrightnessUp{
    if ([UIScreen mainScreen].brightness >=1) {
        return;
    }
    [UIScreen mainScreen].brightness += 0.01;
}

- (void)setBrightnessDown{
    if ([UIScreen mainScreen].brightness <=0) {
        return;
    }
    [UIScreen mainScreen].brightness -= 0.01;
}

- (void)setVolumeUp
{
    [_volumeView setHidden:YES];
    self.systemVolume = self.volumeSlider.value;
    if (self.systemVolume < 1) {
        self.systemVolume += 0.2/17.0;
    }
    [self.volumeSlider setValue:self.systemVolume animated:NO];
    [self.volumeSlider sendActionsForControlEvents:UIControlEventAllEvents];
}

- (void)setVolumeDown{
    [_volumeView setHidden:YES];
    self.systemVolume = self.volumeSlider.value;
    if (self.systemVolume > 0) {
        self.systemVolume -= 0.2/17.0;
    }
    [self.volumeSlider setValue:self.systemVolume animated:NO];
    [self.volumeSlider sendActionsForControlEvents:UIControlEventAllEvents];
}

#pragma mark - delegate
- (void)jdCloudPlayerGestureModel:(JDCloudPlayerGestureModel *)JDCloudGestureModel
                           state:(UIGestureRecognizerState)state
                 moveOrientation:(JDCloudOrientation)moveOrientation
{
    switch (state) {
        case UIGestureRecognizerStateBegan: {
            break;
        }
        case UIGestureRecognizerStateChanged: {
            if (moveOrientation == JDCloudOrientationHorizontal) {
                if (self.seekView.superview == nil) {
                    [self.seekView showWithParentView:self];
                }
            }
            break;
        }
        case UIGestureRecognizerStateEnded:
        case UIGestureRecognizerStateFailed:
        case UIGestureRecognizerStateCancelled:{
            [self.seekView dismissView];
            break;
        }
        default:
            break;
    }
}

- (void)jdCloudPlayerGestureModel:(JDCloudPlayerGestureModel *)JDCloudGestureModel volumeDirection:(UISwipeGestureRecognizerDirection)direction{
    if (direction == UISwipeGestureRecognizerDirectionUp) {
        [self setVolumeUp];
    }else if (direction == UISwipeGestureRecognizerDirectionDown){
        [self setVolumeDown];
    }
}

- (void)jdCloudPlayerGestureModel:(JDCloudPlayerGestureModel *)JDCloudGestureModel brightnessDirection:(UISwipeGestureRecognizerDirection)direction{
    if (direction == UISwipeGestureRecognizerDirectionUp) {
        [self setBrightnessUp];
    }else if (direction == UISwipeGestureRecognizerDirectionDown){
        [self setBrightnessDown];
    }
}

- (void)horizontalOrientationMoveOffset:(float)moveOffset gestureState:(UIGestureRecognizerState)gestureState{
    if (self.delegate && [self.delegate respondsToSelector:@selector(horizontalOrientationMoveOffset: gestureState:)]) {
        [self.delegate horizontalOrientationMoveOffset:moveOffset gestureState:gestureState];
    }
}

- (void)onDoubleClicked {
    if (self.delegate && [self.delegate respondsToSelector:@selector(onDoubleClickedWithJDCloudGestureView:)]) {
        [self.delegate onDoubleClickedWithJDCloudGestureView:self];
    }
}

- (void)onSingleClicked {
    if (self.delegate && [self.delegate respondsToSelector:@selector(onSingleClickedWithJDCloudGestureView:)]) {
        [self.delegate onSingleClickedWithJDCloudGestureView:self];
    }
}

@end
