//
//  JDCloudMainVideoPlayView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/10/9.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDCloudMainVideoPlayView.h"
#import "JDCloudPlayerTopView.h"
#import "JDCloudPlayerBottomView.h"
#import "JDCloudPlayerGestureView.h"
#import "JDCloudPlayerLoadingView.h"
#import "JDCloudPlayerPopLayer.h"
#import "JDCloudPlayerErrorView.h"
#import "JDCloudPlaySetsModel.h"
#import "JDCloudFirstGestureGuideView.h"
#import "JDCloudOnlyAudioPlayView.h"
#import "MoreBottomAlertView.h"
#import "JDCloudPlayerSmallWindow.h"
#import "AppDelegate.h"
#import "JDCloudInterceptAndGifView.h"
#import "MBProgressHUD+Help.h"
#import "UIImageView+WebCache.h"

@interface JDCloudMainVideoPlayView () <JDCloudPlayerGestureViewDelegate,
                                        JDCloudPlayerDelegate,
                                        JDCloudPopLayerDelegate>

@property (nonatomic, strong) JDCloudPlayerTopView* topView;                 //导航View
@property (nonatomic, strong) JDCloudPlayerBottomView* bottomView;           //最下面的View
@property (nonatomic, strong) JDCloudPlayerLoadingView *loadingView;         //loading
@property (nonatomic, strong) JDCloudPlayerPopLayer *popLayer;               //弹出的错误提示界面
@property (nonatomic, strong) JDCloudReachability *reachability;             //网络监听
@property (nonatomic, strong) JDCloudPlayerGestureView* gestureView;         //手势View
@property (nonatomic, strong) JDCloudOnlyAudioPlayView* onlyAudioPlayView;    //纯音频推流View
@property (nonatomic, strong) JDCloudFirstGestureGuideView* gestureGuideView; //第一次手势指引
@property (nonatomic, strong) NSURL *playUrl;                                 //当前播放的url
@property (nonatomic, strong) UIButton *lockButton;                           //锁屏按钮
@property (nonatomic, strong) UIButton *screenShopButton;                     //截图按钮
@property (nonatomic, strong) UIButton *gifButton;                            //生成gif按钮
@property (nonatomic, strong) UIImageView *coverImageView;                    //封面
@property (nonatomic, assign) BOOL isLocked;
@property (nonatomic, assign) BOOL hideControlView;
@property (nonatomic, assign) BOOL mProgressCanUpdate;                        //进度条是否更新，默认是NO
@property (nonatomic, assign) NSTimeInterval currentDuration;                 //记录播放时长
@property (nonatomic, assign) NSTimeInterval beforeClaritTime;                //切换清晰度之前播放时间
@property (nonatomic ,assign) JDCloudPlayMethod playMethod; //播放方式
@property (nonatomic, strong) JDCloudPlaySetsModel *playSetsModel;

@property (nonatomic, strong) VideoPlayListModel *playerInfoModel;             //传给播放器的数据
@property (nonatomic, weak  ) id<JDVideoPlayContextProtocol> context;          //上下文
//@property (nonatomic, strong)JDCloudPlayerCacheSingleton *cacheSingleton;
@property (nonatomic, assign) BOOL hadFirstFrameComed;                         //第一帧视频是否已经返回，用于判断是否隐藏视频封面
@end

@implementation JDCloudMainVideoPlayView

- (void)dealloc
{
    
}


- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context{
    if (self = [super initWithFrame:frame]){
        self.context = context;
        [self addSubviews];
        [self setupData];
        [self addNotifications];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupData
{

}

#pragma mark -
#pragma mark - 初始化子试图
- (void)addSubviews
{
    self.backgroundColor = [UIColor grayColor];
    [self addSubview:self.coverImageView];
    [self addSubview:self.gestureView];
    [self addSubview:self.topView];
    [self addSubview:self.bottomView];
    [self addSubview:self.lockButton];
    [self addSubview:self.screenShopButton];
    [self addSubview:self.gifButton];
    [self addSubview:self.popLayer];
    [self addSubview:self.loadingView];
    
    //屏幕旋转通知
    [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleDeviceOrientationDidChange:)
                                                 name:UIDeviceOrientationDidChangeNotification
                                               object:nil];
    //网络状态判定
    _reachability = [JDCloudReachability reachabilityForInternetConnection];
    [_reachability startNotifier];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(reachabilityChanged)
                                                 name:JDCloudReachabilityChangedNotification
                                               object:nil];
}

- (void)layoutSubviews{
    [super layoutSubviews];
    
    //有悬浮窗时，不处理大小
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    if(app.playerSmallWindow){
        return;
    }
    self.screenShopButton.hidden = NO;
    self.gifButton.hidden = NO;
    if ([JDStaticTools isInterfaceOrientationPortrait]) { //竖屏  - top : 44.0 - left : 0.0 - bottom : 34.0 - rigself    JDCloudMainVideoPlayView *    0x108706e40    0x0000000108706e40ht : 0.0
        float width = self.bounds.size.width;
        float height = self.bounds.size.height;
        self.topView.frame = CGRectMake(0, SafeAreaTopTwoHeight, width, 44);
        self.bottomView.frame = CGRectMake(0, height - 44, width, 44);
        self.topView.hidden = NO;
        self.bottomView.hidden = NO;
        //隐藏锁屏、截屏、gif、手势view
        self.lockButton.hidden = YES;
        if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeVOD) {
            self.screenShopButton.hidden = YES;
            self.gifButton.hidden = YES;
        }
        self.gestureView.hidden = YES;
    }
    else{ //横屏
        //1、位置设置
        self.frame = CGRectMake(0, 0, ScreenWidth, ScreenHeight);
        float width = self.frame.size.width;
        float height = self.frame.size.height;
        self.gestureView.frame = self.bounds;
        if(IPHONEX){  //iphoneX安全区域： - top : 0.0  - left : 44.0  - bottom : 21.0  - right : 44.0
            self.topView.frame = CGRectMake(44, SafeAreaTopTwoHeight, width - 88, 44);
            self.bottomView.frame = CGRectMake(44, height - 54 - 21, width - 88, 54);
            self.lockButton.frame = CGRectMake(20 + 44,(ScreenHeight-44)/2, 44, 44);
            self.screenShopButton.frame = CGRectMake(ScreenWidth - 88, (ScreenHeight - 44 * 2 - 21)/2, 44, 44);
            self.gifButton.frame = CGRectMake(ScreenWidth - 88, ScreenHeight- (ScreenHeight - 44 * 2 - 21)/2 - 44, 44, 44);
        }else{
            self.topView.frame = CGRectMake(10, SafeAreaTopTwoHeight, width - 20, 44);
            self.bottomView.frame = CGRectMake(10, height - 54, width - 20, 54);
            self.lockButton.frame = CGRectMake(20, (ScreenHeight-44)/2, 44, 44);
            self.screenShopButton.frame = CGRectMake(ScreenWidth - 64, (ScreenHeight - 44 * 2 - 21)/2, 44, 44);
            self.gifButton.frame = CGRectMake(ScreenWidth - 64, ScreenHeight- (ScreenHeight - 44 * 2 - 21)/2 - 44, 44, 44);
        }
        
        self.lockButton.hidden = NO;
        //2、显示横屏时隐藏设置
        for (UIView *view in self.subviews) {
//            if ([view isMemberOfClass:[JDCloudInterceptAndGifView class]]) {
//                self.lockButton.hidden = YES;
//                break;
//            }
        }
        if (self.isLocked) {
            self.screenShopButton.hidden = YES;
            self.gifButton.hidden = YES;
            self.topView.hidden = YES;
            self.bottomView.hidden = YES;
            self.gestureView.hidden = YES;
        } else {
            if (self.playerInfoModel.videoListType != JDCloudPlayerVideoListTypeVOD) {
                self.screenShopButton.hidden = YES;
                self.gifButton.hidden = YES;
            }
            self.topView.hidden = NO;
            self.bottomView.hidden = NO;
            self.gestureView.hidden = NO;
        }
    }
    
    self.coverImageView.frame= self.bounds;              //封面
    self.jdCloudPlayer.playerView.frame = self.bounds;   //播放
    self.loadingView.frame = CGRectMake((self.bounds.size.width -  130)/2, (self.bounds.size.height - 120)/2, 130, 120); //loading
    self.popLayer.frame = self.bounds;
    
    //直播样式修改
    if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeLive) {
        self.gestureView.hidden = YES;
        [self.bottomView setNeedsLayout];
    }
    //判断是否是纯音频
    [self showOnlyAudioPlayView];
}

- (void)showLoadingView
{
    if (self.loadingView) {
        self.loadingView.hidden = NO;
    }
}

- (void)replay{
    [self playWithPlayerInfoModel:self.playerInfoModel];
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //锁屏
            [self lockScreenAction];
            break;
        case 101:   //截图
            [self screenShopAction];
            break;
        case 102:   //生成gif
            [self creatGifAction];
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - 功能函数
//重设播放器参数
- (void)playWithPlayerInfoModel:(VideoPlayListModel *)infoModel
{
    self.hadFirstFrameComed = NO;
    JDCloudPlaySetsModel *globalPlaySetsModel = [JDCloudPlaySetsModel sharedSetVideoPlayManager];
    
    infoModel.shouldPlayNOWiFi = globalPlaySetsModel.canPlayNoWifi;
    if (infoModel.videoListType == JDCloudPlayerVideoListTypeLocal) {
        [self playWithModel:infoModel];
        self.jdCloudPlayer.disableMemoryCache = NO;
        self.popLayer.hidden = YES;
    } else {
        self.networkStatus = [JDStaticTools checkNetWork];
        if (self.networkStatus == JDCloudNetworkStatusNotReachable){  //无网络
            self.playerInfoModel = infoModel;
            [self playWithModel:infoModel];
            [self.jdCloudPlayer pause];
            self.popLayer.hidden = NO;
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeUnreachableNetwork popMsg:@"当前无网络连接"];
            self.jdCloudPlayer.disableMemoryCache = NO;

        } else if (self.networkStatus == JDCloudNetworkStatusReachableViaWWAN && infoModel.shouldPlayNOWiFi == NO) {  //4G、3G、2G
            self.playerInfoModel = infoModel;
            [self playWithModel:infoModel];
            [self.jdCloudPlayer pause];
            self.popLayer.hidden = NO;
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeUseMobileNetwork popMsg:@"当前为4G网络，是否继续播放"];
            self.jdCloudPlayer.disableMemoryCache = YES;
            [self.bottomView updateViewWithPlayerState:self.context.jdCloudPlayer.playerState];
        } else {
            [self playWithModel:infoModel];
            self.jdCloudPlayer.disableMemoryCache = NO;
        }
    }
}

//重设播放器的model
- (void)playWithModel:(VideoPlayListModel *)infoModel
{
    if (_loadingView) {
        [_loadingView dismiss];
    }
//    if (self.playUrl && [self.playUrl.absoluteString rangeOfString:@"127.0.0.1"].location != NSNotFound) {
//        [JDCloudPlayerUtil shutdownClientByUrl:self.playUrl.absoluteString];
//    }
    if (self.jdCloudPlayer.playerView) {
        [self.jdCloudPlayer.playerView removeFromSuperview];
    }
    //数据处理
    if (self.playerInfoModel == infoModel) {
        self.playerInfoModel = infoModel;
        self.beforeClaritTime = self.playerInfoModel.progress * self.bottomView.duration;
    } else {
        self.beforeClaritTime = 0;
        self.playerInfoModel = infoModel;
        if (self.bottomView) {
            self.bottomView.progress = 0.0;
            self.bottomView.loadTimeProgress = 0.0;
            self.bottomView.progressCanUpdate = NO;
            [self.bottomView updateProgressWithCurrentTime:0.0 TotalTime:0.0];
        }
    }
    
    [self setPlayerUrl];
//    [self setCacheAndPreDownload];
    [self setCoverUrl:[NSURL URLWithString:infoModel.imageUrl]];
    if (self.jdCloudPlayer && self.jdCloudPlayer.playerView) {
        [self resetPlayWithUrl:self.playUrl];
    } else {
        [self prepareToPlayWithUrl:self.playUrl];
    }
    [self.jdCloudPlayer prepareToPlay];
    self.jdCloudPlayer.delegate = self;
    self.jdCloudPlayer.adaptivePlayback = YES;
    [self insertSubview:self.jdCloudPlayer.playerView atIndex:0];
    [self.loadingView show];
    self.jdCloudPlayer.playerView.backgroundColor = [UIColor blackColor];
    self.jdCloudPlayer.shouldAutoplay = YES;
    self.jdCloudPlayer.isOnlyAudio = SharedSetVideoPlayManager.isOnlyAudio;
//    self.jdCloudPlayer.isMirrorPlay = SharedSetVideoPlayManager.isMirrorPlay;
    self.jdCloudPlayer.isMirrorPlay = infoModel.isMirrorPlay;
    self.jdCloudPlayer.isPlayOnBackGroud = SharedSetVideoPlayManager.isBackgroundPlay;
    self.jdCloudPlayer.scalingMode = SharedSetVideoPlayManager.scalingMode;
    if (self.beforeClaritTime > 0) {
        self.bottomView.progressCanUpdate = NO;
        [self.bottomView updateProgressWithCurrentTime:self.beforeClaritTime TotalTime:self.bottomView.duration];
    }
    
    self.gifButton.hidden = YES;
    self.screenShopButton.hidden = YES;
    self.bottomView.progressView.hidden = NO;
    if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeLive) {
        self.bottomView.progressView.hidden = YES;
    } else if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeVOD) {
        self.gifButton.hidden = NO;
        self.screenShopButton.hidden = NO;
    }
    [self.bottomView setNeedsLayout];
}

//第一次准备播放设着URL
- (void)prepareToPlayWithUrl:(NSURL *)url
{
    //播放器
    JDCloudOptionModel *model = [[JDCloudOptionModel alloc] init];
    model.videoFormat = self.playerInfoModel.videoFormat;
    [self.jdCloudPlayer setVideoWithURL:url withPlayerOption:model];
}

//重设URL
- (void)resetPlayWithUrl:(NSURL *)url
{
    if ([self.jdCloudPlayer isPlaying]) {
        [self.jdCloudPlayer stop];
    }
    JDCloudOptionModel *model = [[JDCloudOptionModel alloc] init];
    model.videoFormat = self.playerInfoModel.videoFormat;
    [self.jdCloudPlayer resetWithUrl:url withPlayerOption:model];
}

//设置清晰图
- (void)setPlayerUrl
{
    NSString *currentPlayerUrlStr = [self getPlayerUrlStrWithModel:self.playerInfoModel];
    if (currentPlayerUrlStr) {
        currentPlayerUrlStr = [currentPlayerUrlStr stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        self.playUrl = [NSURL URLWithString:currentPlayerUrlStr];
    }
}

//设置边播边下载与预缓存下一个视频
//- (void)setCacheAndPreDownload
//{
//    if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeVOD) {
//        NSString *playUrl = [JDCloudPlayerUtil getProxyUrl:self.playUrl.absoluteString];
//        self.playUrl = [NSURL URLWithString:playUrl];
//        VideoPlayListModel *nextModel = [self.context nextPlayInfoModel];
//        NSString *playerUrlStr = nil;
//        if (nextModel) {
//            playerUrlStr = [self getPlayerUrlStrWithModel:nextModel];
//        }
//        if (playerUrlStr) {
//            [JDCloudPlayerUtil preDownload:playerUrlStr];
//        }
//    }
//}

- (NSString *)getPlayerUrlStrWithModel:(VideoPlayListModel *)model
{
    NSString *playerUrlStr = model.selectedPlayerDic[@"playUrl"];
    return playerUrlStr;
}

//锁屏
- (void)lockScreenAction{
    self.lockButton.selected = !self.lockButton.selected;
    if (self.lockButton.selected) {
        self.topView.hidden = YES;
        self.bottomView.hidden = YES;
        if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeVOD)
        {
            self.gifButton.hidden = YES;
            self.screenShopButton.hidden = YES;
        }
        self.gestureView.hidden = YES;
        self.isLocked = YES;
    } else {
        self.topView.hidden = NO;
        self.bottomView.hidden = NO;
        self.gestureView.hidden = NO;
        if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeVOD)
        {
            self.gifButton.hidden = NO;
            self.screenShopButton.hidden = NO;
        }
        self.isLocked = NO;
    }
    if (self.playerInfoModel.videoListType != JDCloudPlayerVideoListTypeVOD) {
        self.gestureView.hidden = YES;
        self.gifButton.hidden = YES;
        self.screenShopButton.hidden = YES;
    }
}

//截图
- (void)screenShopAction{
    UIImage *image = [self.jdCloudPlayer thumbnailImageAtCurrentTime];
    UIImageWriteToSavedPhotosAlbum(image, self, @selector(image:didFinishSavingWithError:contextInfo:), nil);
}

//保存截图到本地相册
- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo{
    
    if (error == nil) {
        NSString *message = @"截屏保存成功";
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"提示信息" message:message preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *ok = [UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:nil];
        [alert addAction:ok];
        if ([self.context isKindOfClass:[UIViewController class]]) {
            UIViewController *viewController = (UIViewController *)self.context;
            [viewController presentViewController:alert animated:YES completion:nil];
        }
    }else{
        NSString *message = @"截屏保存失败";
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"提示信息" message:message preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *ok = [UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDefault handler:nil];
        [alert addAction:ok];
        if ([self.context isKindOfClass:[UIViewController class]]) {
            UIViewController *viewController = (UIViewController *)self.context;
            [viewController presentViewController:alert animated:YES completion:nil];
        }
    }
}

//生成gif
- (void)creatGifAction{
//    JDCloudInterceptAndGifView *interceptAndGif = [[JDCloudInterceptAndGifView alloc] initWithContext:self.context];
//    interceptAndGif.videoPath = self.playUrl.absoluteString;
//    interceptAndGif.downloadVideoDurationTime = 15;
//    if (interceptAndGif.downloadVideoDurationTime > self.jdCloudPlayer.duration) {
//        [MBProgressHUD showHUDInView:self HudType:MBProgressHUDModeCustomView withText:@"视频总时长过短，无法生成gif～"];
//        return;
//    }
//    [self addSubview:interceptAndGif];
//    [interceptAndGif show];
//    interceptAndGif.showedBlock = ^(id info) {
//        self.isLocked = YES;
//        self.lockButton.hidden = YES;
//        [self setNeedsLayout];
//        [self layoutIfNeeded];
//        [self.context.jdCloudPlayer pause];
//    };
//    interceptAndGif.hidedBlock = ^(id info) {
//        self.isLocked = NO;
//        [self setNeedsLayout];
//        [self layoutIfNeeded];
//        [self.context.jdCloudPlayer start];
//    };
}

//判断是否增加纯音频推流View
- (void)showOnlyAudioPlayView{
    [self.onlyAudioPlayView setNeedsLayout];
    //判断是否是纯音频推流
    if (SharedSetVideoPlayManager.isOnlyAudio) {
        for (UIView* view in self.superview.subviews) {  // more加载superView上 横屏
            if ([view isKindOfClass:[MoreBottomAlertView class]]) {
                [self insertSubview:self.onlyAudioPlayView belowSubview:view];
                return;
            }
        }
        for (UIView* view in self.subviews) { //more加载self上 竖屏
            if ([view isKindOfClass:[MoreBottomAlertView class]]) {
                [self insertSubview:self.onlyAudioPlayView belowSubview:view];
                return;
            }
        }
        [self insertSubview:self.onlyAudioPlayView atIndex:[self.subviews count]];
    }else{
        [self.onlyAudioPlayView removeFromSuperview];
    }
}

//设着封面
- (void)setCoverUrl:(NSURL *)coverUrl{
    if (coverUrl) {
        if (self.coverImageView) {
            [self.coverImageView sd_setImageWithURL:coverUrl completed:^(UIImage * _Nullable image, NSError * _Nullable error, SDImageCacheType cacheType, NSURL * _Nullable imageURL) {
                if (self.hadFirstFrameComed == NO && image) {
                    self.coverImageView.image = image;
                    self.coverImageView.hidden = NO;
                }
            }];
//            dispatch_async(dispatch_get_global_queue(0, 0), ^{
//                NSData *data = [NSData dataWithContentsOfURL:coverUrl];
//                dispatch_async(dispatch_get_main_queue(), ^{
//
//                });
//            });
        }
    }
}

#pragma mark -
#pragma mark - 播放器状态回调
- (void)updatePlayViewDataWithEvent:(JDCloudPlayerEvent)event player:(JDCloudPlayer *)player{
    switch (event) {
        case JDCloudPlayerEventPrepareDone:
        {
//            [_loadingView dismiss];
            self.bottomView.progressCanUpdate = YES;
            self.bottomView.duration = player.duration;
        }
            break;
        case  JDCloudPlayerEventFirstFrame:
        {
            if (self.popLayer) {
                self.popLayer.hidden = YES;
            }
            self.bottomView.progressCanUpdate = YES;
//            [_loadingView dismiss];
            self.hadFirstFrameComed = YES;
            //开启常亮状态
            [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
            self.bottomView.progress = 0.0;
            self.bottomView.loadTimeProgress = 0.0;
            //隐藏封面
            if (self.coverImageView) {
                self.coverImageView.hidden = YES;
                NSLog(@"播放器:首帧加载完成封面隐藏");
            }
        }
            break;
        case JDCloudSeekVideoStart:
        {
            self.bottomView.progressCanUpdate = YES;
            [_loadingView dismiss];
        }
            break;
        case JDCloudSeekAudioStart:
        {
            self.bottomView.progressCanUpdate = YES;
            [_loadingView dismiss];
        }
            break;
        case JDCloudPlayerEventPlay:
        {
            //隐藏封面
            if (self.coverImageView) {
                self.coverImageView.hidden = YES;
            }
            self.bottomView.progressCanUpdate = YES;

            [_loadingView dismiss];
            
//            self.jdCloudPlayer.isOnlyAudio = SharedSetVideoPlayManager.isOnlyAudio;
//            self.jdCloudPlayer.isMirrorPlay = SharedSetVideoPlayManager.isMirrorPlay;
//            self.jdCloudPlayer.isPlayOnBackGroud = SharedSetVideoPlayManager.isBackgroundPlay;
//            self.jdCloudPlayer.scalingMode = SharedSetVideoPlayManager.scalingMode;
            
            if (self.beforeClaritTime > 0) {
                [self.jdCloudPlayer seekToTime:self.beforeClaritTime];
                self.bottomView.progress = self.beforeClaritTime/(self.jdCloudPlayer.duration > 0 ? self.jdCloudPlayer.duration : MAXFLOAT);
                self.beforeClaritTime = 0;
            }
        }
            break;
        case JDCloudPlayerEventPause:
        {
            //播放器暂停回调
            [_loadingView dismiss];
            self.bottomView.progressCanUpdate = YES;
        }
            break;
        case JDCloudPlayerEventInterrupted:
        {
            [self.jdCloudPlayer pause];
            //播放器暂停回调
            [_loadingView dismiss];
            self.bottomView.progressCanUpdate = NO;
        }
            break;
        case JDCloudPlayerEventFinish:{
            //播放完成回调
            [self.loadingView dismiss];
            if (self.jdCloudPlayer.duration > 0) {
                [self.jdCloudPlayer seekToTime:0];
                self.bottomView.progress = 0.0;
                self.bottomView.progressCanUpdate = YES;
            } else {
                if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeLive) {
                    [MBProgressHUD showHUDInView:self HudType:MBProgressHUDModeCustomView withText:@"直播结束～"];
                }
            }
        }
            break;
            
        case JDCloudPlayerEventNaturalSizeChanged:{
//            [MBProgressHUD showHUDInView:self HudType:MBProgressHUDModeCustomView withText:@"直播结束～"];
            break;
        }
        case JDCloudPlayerEventStop: {
            //stop 回调
            //取消常亮状态
            [self.loadingView dismiss];
            [[UIApplication sharedApplication] setIdleTimerDisabled:NO];
            //隐藏封面
            if (self.coverImageView) {
                self.coverImageView.hidden = YES;
                NSLog(@"播放器:播放停止封面隐藏");
            }
        }
            break;
        case JDCloudPlayerEventSeekDone :{
            self.bottomView.progressCanUpdate = YES;
        }
            break;
        case JDCloudPlayerEventBeginLoading: {
            self.bottomView.progressCanUpdate = NO;
            //展示loading动画
            [_loadingView show];
        }
            break;
        case JDCloudPlayerEventEndLoading: {
            //关闭loading动画
            [_loadingView dismiss];
            self.bottomView.progressCanUpdate = YES;
        }
            break;
        case JDCloudPlayerEventSeekingForward:
        case JDCloudPlayerEventSeekingBackward: {
                [self.loadingView show];
                self.bottomView.progressCanUpdate = NO;
            }
            break;
        default:
            break;
    }
}

//网络状态判定
- (BOOL)networkChangedToShowPopView{
    BOOL ret = NO;
    [_loadingView dismiss];
    self.popLayer.hidden = YES;
    if (self.playerInfoModel.videoListType == JDCloudPlayerVideoListTypeLocal) {
        return NO;
    }
    self.networkStatus = [self.reachability currentReachabilityStatus];
    switch (self.networkStatus) {
        case JDCloudNetworkStatusNotReachable://由播放器底层判断是否有网络
            self.bottomView.currentTime = self.jdCloudPlayer.currentPlayerTime;
            [self.jdCloudPlayer pause];
            self.popLayer.hidden = NO;
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeUnreachableNetwork popMsg:@"当前无网络连接"];
            break;
        case JDCloudNetworkStatusReachableViaWiFi:
            if (self.jdCloudPlayer.shouldAutoplay == NO) {
                self.jdCloudPlayer.shouldAutoplay = YES;
            }
            [self.jdCloudPlayer resume];
            break;
        case JDCloudNetworkStatusReachableViaWWAN:
        {
            self.bottomView.currentTime = self.context.jdCloudPlayer.currentPlayerTime;
            if (self.jdCloudPlayer.shouldAutoplay) {
                self.jdCloudPlayer.shouldAutoplay = NO;
            }
            [self.jdCloudPlayer pause];
            self.popLayer.hidden = NO;
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeUseMobileNetwork popMsg:@"当前为4G网络，是否继续播放"];
            ret = YES;
        }
            break;
        default:
            break;
    }
    return ret;
}

//根据错误信息，展示popLayer界面
- (void)showPopLayerWithErrorModel:(JDCloudPlayerVideoErrorModel *)errorModel{
    switch (errorModel.errorCode) {
        case -40011:
        case 100:
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeNetworkTimeOutError popMsg:errorModel.errorMsg];
            break;
        case -110:
        case -1004:
        case -1007:
        {
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeLoadDataError popMsg:errorModel.errorMsg];
        }
            break;
        default:
        {
            [self.popLayer showPopViewWithCode:JDCloudPlayerPopCodeServerError popMsg:errorModel.errorMsg];
        }
            break;
    }
}

- (void)updatePlayDataReplay{
    [self playWithModel:self.playerInfoModel];
}

- (NSMutableDictionary *)getPlayerInfo{
    NSMutableDictionary *infoDict = [NSMutableDictionary dictionary];
    NSNumber *inOnlyAudio = [NSNumber numberWithBool:SharedSetVideoPlayManager.isOnlyAudio];
    [infoDict setObject:inOnlyAudio forKey:@"是否纯音播放"];
//    NSNumber *isMirrorPlay = [NSNumber numberWithBool:SharedSetVideoPlayManager.isMirrorPlay];
    NSNumber *isMirrorPlay = [NSNumber numberWithBool:self.playerInfoModel.isMirrorPlay];
    [infoDict setObject:isMirrorPlay forKey:@"是否镜像播放"];
    NSNumber *isBackgroundPlay = [NSNumber numberWithBool:SharedSetVideoPlayManager.isBackgroundPlay];
    [infoDict setObject:isBackgroundPlay forKey:@"是否后台播放"];
    NSNumber *scalingMode = [NSNumber numberWithInteger:SharedSetVideoPlayManager.scalingMode];
    [infoDict setObject:scalingMode forKey:@"视频裁剪模式"];
    if (self.playUrl) {
        [infoDict setObject:self.playUrl forKey:@"playUrl"];
    }
    NSString *netType = nil;
    switch ([self.reachability currentReachabilityStatus]) {
        case JDCloudNetworkStatusNotReachable:
            netType = @"无网";
            break;
        case JDCloudNetworkStatusReachableViaWiFi:
            netType = @"WiFi";
            break;
        case JDCloudNetworkStatusReachableViaWWAN:
            netType = @"4G";
            break;
        default:
            netType = @"4G";
            break;
    }
    [infoDict setObject:netType forKey:@"网络类型"];
    return infoDict;
}

- (NSMutableDictionary *)getPlayerErrorInfoWithModel:(JDCloudPlayerVideoErrorModel *)errorModel
{
    NSMutableDictionary *errorDict = [NSMutableDictionary dictionary];
    if (errorModel.errorMsg) {
        [errorDict setObject:errorModel.errorMsg forKey:@"errorMsg"];
    }
    [errorDict setObject:[NSNumber numberWithInt:errorModel.errorCode] forKey:@"errorCode"];
    if (errorModel.errorVid) {
        [errorDict setObject:errorModel.errorVid forKey:@"errorVid"];
    }
    if (errorModel.errorRequestId) {
        [errorDict setObject:errorModel.errorRequestId forKey:@"errorRequestId"];
    }
    return errorDict;
}

#pragma mark - JDCloudPlayerDelegate
- (void)player:(JDCloudPlayer *)player onEventCallback:(JDCloudPlayerEvent)event
{
    if (player == self.context.jdCloudPlayer) {
        [self updatePlayViewDataWithEvent:event player:player];
        [self.context playHappenEvent:event withPlayerInfo:[self getPlayerInfo]];
        [self.bottomView updateViewWithPlayerState:self.context.jdCloudPlayer.playerState];
    }
}

- (void)player:(JDCloudPlayer *)player playBackErrorModel:(JDCloudPlayerVideoErrorModel *)errorModel
{
    if (player == self.context.jdCloudPlayer) {
        [_loadingView dismiss];
        //根据错误信息，展示popLayer界面
        [self showPopLayerWithErrorModel:errorModel];
        NSMutableDictionary *playerInfo = [self getPlayerInfo];
        [playerInfo setObject:[self getPlayerErrorInfoWithModel:errorModel] forKey:@"playerError"];
        [self.context playHappenEvent:JDCloudPlayerEventFinish withPlayerInfo:playerInfo];
        [self.bottomView updateViewWithPlayerState:self.context.jdCloudPlayer.playerState];
    }
}

#pragma mark - JDCloudPopLayerDelegate
- (void)showPopViewWithType:(JDCloudErrorType)type{
    JDCloudPlaySetsModel *globalPlaySetsModel = [JDCloudPlaySetsModel sharedSetVideoPlayManager];
    self.popLayer.hidden = YES;
    switch (type) {
        case JDCloudErrorTypeReplay:
        {
            //重播
            [self.jdCloudPlayer replay];
            [self.jdCloudPlayer seekToTime:self.bottomView.currentTime];
            self.jdCloudPlayer.disableMemoryCache = NO;

        }
            break;
        case JDCloudErrorTypeRetry:
        {
            [self.context.jdCloudPlayer pause];
            self.bottomView.currentTime = self.jdCloudPlayer.currentPlayerTime;
            if (self.context.jdCloudPlayer.shouldAutoplay == NO) {
                self.context.jdCloudPlayer.shouldAutoplay = YES;
            }
            //重试播放
            if ([self networkChangedToShowPopView]) {
                return;
            }
            [self updatePlayDataReplay];
        }
            break;
        case JDCloudErrorTypePause:
        {
            self.jdCloudPlayer.disableMemoryCache = NO;
            globalPlaySetsModel.canPlayNoWifi = YES;
            self.playerInfoModel.shouldPlayNOWiFi = YES;
            if (self.context.jdCloudPlayer.playerState == JDCloudPlayerStatePause){
                [self.context.jdCloudPlayer resume];
            } else {
                if (self.context.jdCloudPlayer.shouldAutoplay == NO) {
                    self.context.jdCloudPlayer.shouldAutoplay = YES;
                }
                
                [self updatePlayDataReplay];
            }
        }
            break;
        default:
            break;
    }
}

- (void)onBackClickedWithPopLayer:(JDCloudPlayerPopLayer *)popLayer{
    self.bottomView.currentTime = self.context.jdCloudPlayer.currentPlayerTime;
    [self.context.jdCloudPlayer pause];
    self.popLayer.hidden = YES;
}



#pragma mark -
#pragma mark - JDCloudPlayerGestureViewDelegate
//单击屏幕
- (void)onSingleClickedWithJDCloudGestureView:(JDCloudPlayerGestureView *)gestureView
{
    if(self.topView.hidden){
        self.topView.hidden = NO;
        self.bottomView.hidden = NO;
    }else{
        self.topView.hidden = YES;
        self.bottomView.hidden = YES;
    }
}

//双击屏幕
- (void)onDoubleClickedWithJDCloudGestureView:(JDCloudPlayerGestureView *)gestureView
{
    JDCloudPlayerState playeState = self.jdCloudPlayer.playerState;
    if (playeState == JDCloudPlayerStatePlay){
        [self.jdCloudPlayer pause];
    }else if(playeState == JDCloudPlayerStatePause){
        [self.jdCloudPlayer resume];
    }
}

//手势水平移动偏移量
- (void)horizontalOrientationMoveOffset:(float)moveOffset gestureState:(UIGestureRecognizerState)gestureState
{
    UISwipeGestureRecognizerDirection direction = UISwipeGestureRecognizerDirectionLeft;
    if (moveOffset >= 0) {
        direction = UISwipeGestureRecognizerDirectionRight;
    }
    float x =  moveOffset;
    double duration = self.jdCloudPlayer.duration;
    float width = self.bottomView.frame.size.width;
    double seekTime = self.playerInfoModel.progress * duration;
    if (duration > 3600) {
        seekTime += x / width * duration * 0.1;
    } else if (1800 < duration && duration <= 3600) {
        seekTime += x / width * duration * 0.2;
    } else if (600 < duration && duration <= 1800) {
        seekTime += x / width * duration * 0.34;
    } else if (240 < duration && duration <= 600) {
        seekTime += x / width * duration * 0.5;
    } else {
        seekTime += x / width * duration;
    }
    if (seekTime < 0) {
        seekTime = 0;
    } else if (seekTime > duration) {
        seekTime = duration;
    }
    self.bottomView.progressCanUpdate = NO;
    [self.gestureView setSeekTime:seekTime direction:direction];
    [self.bottomView updateProgressWithCurrentTime:seekTime TotalTime:direction];
    self.bottomView.progress = seekTime/(duration > 0 ? duration : MAXFLOAT);
    if (gestureState == UIGestureRecognizerStateEnded) {
        [self.jdCloudPlayer seekToTime:seekTime];
        self.playerInfoModel.progress = seekTime/(duration > 0 ? duration : MAXFLOAT);
    }
    if (x < 0) {
        self.bottomView.loadTimeProgress = self.bottomView.progress;
    }
}

#pragma mark -
#pragma mark - 通知
- (void)addNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeSreenState:) name:@"JDVideoPlayScreenChangeNotification" object:nil];
}

//屏幕变化的通知
- (void)changeSreenState:(NSNotification *)infoNotification
{
    NSDictionary *dic = [infoNotification userInfo];
    NSString *isFullScreen = [dic objectForKey:@"isFullScreen"];
    if ([isFullScreen isEqualToString:@"YES"]) {
        [self interfaceOrientation:(UIInterfaceOrientation)UIDeviceOrientationLandscapeLeft];
    }else{
        [self interfaceOrientation:(UIInterfaceOrientation)UIDeviceOrientationPortrait];
    }
}

//强制旋转
- (void)interfaceOrientation:(UIInterfaceOrientation)orientation {
    if ([[UIDevice currentDevice] respondsToSelector:@selector(setOrientation:)]) {
        SEL selector  = NSSelectorFromString(@"setOrientation:");
        NSInvocation *invocation = [NSInvocation invocationWithMethodSignature:[UIDevice instanceMethodSignatureForSelector:selector]];
        [invocation setSelector:selector];
        [invocation setTarget:[UIDevice currentDevice]];
        int val = orientation;
        [invocation setArgument:&val atIndex:2];
        [invocation invoke];
    }
}

//监听屏幕变化
- (void)handleDeviceOrientationDidChange:(UIInterfaceOrientation)interfaceOrientation{
    UIDevice *device = [UIDevice currentDevice] ;
    switch (device.orientation) {
        case UIDeviceOrientationFaceUp:
        case UIDeviceOrientationFaceDown:
        case UIDeviceOrientationUnknown:
        case UIDeviceOrientationPortraitUpsideDown:
            break;
        case UIDeviceOrientationLandscapeLeft:
        case UIDeviceOrientationLandscapeRight:
        {
            //假如第一次出现全屏 增加手势引导
            if ([[[NSUserDefaults standardUserDefaults] valueForKey:JDIsFirstFullscreenPlay] length] <1)
            {
                [[NSUserDefaults standardUserDefaults] setValue:@"NO" forKey:JDIsFirstFullscreenPlay];
                //增加新手指引
                self.gestureGuideView = [[JDCloudFirstGestureGuideView alloc]init];
                //加在window上了
                UIWindow *window = [UIApplication sharedApplication].keyWindow;
                [window addSubview:self.gestureGuideView];
            }
            //全屏
            [self.onlyAudioPlayView removeFromSuperview];
            [self setNeedsLayout];
            
        }
            break;
        case UIDeviceOrientationPortrait:
        {
            [self.gestureGuideView removeFromSuperview];
            [self.onlyAudioPlayView removeFromSuperview];
            //退出全屏
            [self setNeedsLayout];
        }
            break;
        default:
            break;
    }
}

//网络通知
- (void)reachabilityChanged{
    [self networkChangedToShowPopView];
}


#pragma mark -
#pragma mark - 懒加载
//底层播放器
- (JDCloudPlayer *)jdCloudPlayer
{
    if (!_jdCloudPlayer) {
    _jdCloudPlayer = [[JDCloudPlayer alloc]init];
    }
    return _jdCloudPlayer;
}

//锁屏按钮
- (UIButton *)lockButton{
    if (!_lockButton) {
        _lockButton = [[UIButton alloc] init];
        [_lockButton setImage:[UIImage imageNamed:@"横屏-未锁屏"] forState:UIControlStateNormal];
        [_lockButton setImage:[UIImage imageNamed:@"横屏-锁住"] forState:UIControlStateSelected];
        _lockButton.backgroundColor = [UIColor blackColor];
        _lockButton.alpha = 0.4;
        _lockButton.layer.masksToBounds = YES;
        _lockButton.layer.cornerRadius = 22;
        _lockButton.tag = 100;
        [_lockButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _lockButton;
}

//截屏按钮
- (UIButton *)screenShopButton{
    if (!_screenShopButton) {
        _screenShopButton = [[UIButton alloc] init];
        [_screenShopButton setImage:[UIImage imageNamed:@"横屏-截图"] forState:UIControlStateNormal];
        [_screenShopButton setImage:[UIImage imageNamed:@"横屏-截图"] forState:UIControlStateSelected];
        _screenShopButton.backgroundColor = [UIColor blackColor];
        _screenShopButton.alpha = 0.4;
        _screenShopButton.layer.masksToBounds = YES;
        _screenShopButton.layer.cornerRadius = 22;
        _screenShopButton.tag = 101;
        [_screenShopButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _screenShopButton;
}

//生成gif按钮
- (UIButton *)gifButton{
    if (!_gifButton) {
        _gifButton = [[UIButton alloc] init];
        [_gifButton setImage:[UIImage imageNamed:@"横屏-GIF"] forState:UIControlStateNormal];
        [_gifButton setImage:[UIImage imageNamed:@"横屏-GIF"] forState:UIControlStateSelected];
        _gifButton.backgroundColor = [UIColor blackColor];
        _gifButton.alpha = 0.4;
        _gifButton.layer.masksToBounds = YES;
        _gifButton.layer.cornerRadius = 22;
        _gifButton.tag = 102;
        [_gifButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _gifButton;
}

//topView
- (JDCloudPlayerTopView *)topView{
    if (!_topView) {
        _topView = [[JDCloudPlayerTopView alloc]initWithFrame:CGRectZero withContext:self.context];
        
    }
    return _topView;
}

//最下面View
- (JDCloudPlayerBottomView *)bottomView{
    if (!_bottomView) {
        _bottomView = [[JDCloudPlayerBottomView alloc] initWithFrame:CGRectZero withContext:self.context];
    }
    return _bottomView;
}

//手势View
- (JDCloudPlayerGestureView *)gestureView{
    if (!_gestureView) {
        _gestureView = [[JDCloudPlayerGestureView alloc]init];
        _gestureView.delegate = self;
    }
    return _gestureView;
    
}

//loading
- (JDCloudPlayerLoadingView *)loadingView{
    if (!_loadingView) {
        _loadingView = [[JDCloudPlayerLoadingView alloc] init];
    }
    return _loadingView;
}

//返回
- (JDCloudPlayerPopLayer *)popLayer{
    if (!_popLayer) {
        _popLayer = [[JDCloudPlayerPopLayer alloc] init];
        _popLayer.frame = self.bounds;
        _popLayer.hidden = YES;
        _popLayer.delegate = self;
    }
    return _popLayer;
}

//封面
- (UIImageView *)coverImageView{
    if (!_coverImageView) {
        _coverImageView = [[UIImageView alloc] init];
        _coverImageView.clipsToBounds = YES;
        _coverImageView.backgroundColor = [UIColor clearColor];
        _coverImageView.contentMode = UIViewContentModeScaleToFill;
    }
    return _coverImageView;
}

//纯音频页面
- (JDCloudOnlyAudioPlayView *)onlyAudioPlayView{
    if (!_onlyAudioPlayView) {
        _onlyAudioPlayView = [[JDCloudOnlyAudioPlayView alloc]initWithFrame:CGRectZero WithContext:_context];
    }
    return _onlyAudioPlayView;
}

@end


