//
//  JDCloudPlayerSmallWindow.m
//  jdcloud_ios_live_demo
//
//  Created by 王亚伟 on 2018/10/23.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDCloudPlayerSmallWindow.h"
#import "AppDelegate.h"
#import "JDCloudPlaySetsModel.h"

@interface JDCloudPlayerSmallWindow()
{
    CGPoint _loc_in;
}

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;

@end


@implementation JDCloudPlayerSmallWindow

- (instancetype)initWithContext:(id<JDVideoPlayContextProtocol>)context
{
     self = [super initWithFrame:CGRectMake(ScreenWidth/2.0, ScreenHeight - ScreenWidth/2.0 * 9.0/16.0, ScreenWidth/2.0, ScreenWidth/2.0 * 9.0/16.0)];
    if (self) {
        self.context = context;
        [self setupSubviews];
    }
    return self;
}

- (void)showInWindow
{
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    app.playerSmallWindow = self;
    if (![app.window.subviews containsObject:self]) {
        [app.window addSubview:self];
    }
}

- (void)resumVC
{
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    [self removeFromSuperview];
    [self.context showViewController];
    app.playerSmallWindow = nil;
}

- (void)setupSubviews
{
    [self addSubview:self.context.jdCloudPlayer.playerView];
    self.context.jdCloudPlayer.playerView.frame = CGRectMake(0, 0, ScreenWidth/2.0, ScreenWidth/2.0 * 9.0/16.0);
   
    //返回
    UIButton* _leftBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [_leftBtn setFrame:CGRectMake(0, 0, 16, 16)];
    [_leftBtn setBackgroundImage:[UIImage imageNamed:@"img_delete"] forState:UIControlStateNormal];
    [_leftBtn setBackgroundImage:[UIImage imageNamed:@"img_delete"] forState:UIControlStateHighlighted];
    [_leftBtn addTarget:self action:@selector(onCloseBtn:) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:_leftBtn];
    [_leftBtn sizeToFit];
    
    UIPanGestureRecognizer *panGes = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panAction:)];
    [self addGestureRecognizer:panGes];
    
    UITapGestureRecognizer* tapGes = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapAction:)];
    [self addGestureRecognizer:tapGes];
}


- (void)panAction:(UIPanGestureRecognizer *)ges
{
    int width = [UIScreen mainScreen].bounds.size.width;
    int height = [UIScreen mainScreen].bounds.size.height;
    CGPoint loc = [ges locationInView:self.superview];
    if (ges.state == UIGestureRecognizerStateBegan) {
        _loc_in = [ges locationInView:self];
    }
    
    // 坐标矫正，避免画面超出屏幕
    CGFloat x;
    CGFloat y;
    if (self.frame.size.width + loc.x - _loc_in.x >= width){
        x = width - self.frame.size.width * 0.5;
    }else if (loc.x - _loc_in.x <= 0) {
        x = self.frame.size.width * 0.5;
    }else {
        x = self.frame.size.width * 0.5 - _loc_in.x + loc.x;
    }
    
    if (self.frame.size.height - _loc_in.y + loc.y >= height) {
        y = height - self.frame.size.height * 0.5;
    }else if (loc.y - _loc_in.y <= 0){
        y = self.frame.size.height * 0.5;
    }else {
        y = self.frame.size.height * 0.5 - _loc_in.y + loc.y;
    }
    
    [UIView animateWithDuration:0 animations:^{
        self.center = CGPointMake(x, y);
    }];
}

- (void)tapAction:(UITapGestureRecognizer *)ges
{
    [self resumVC];
}

- (void)onCloseBtn:(id)sender
{
    [self destroyPlayer];
}

- (void)destroyPlayer
{
//    SharedSetVideoPlayManager.isMirrorPlay = NO;
    SharedSetVideoPlayManager.isOnlyAudio = NO;
    SharedSetVideoPlayManager.isBackgroundPlay = NO;
    SharedSetVideoPlayManager.scalingMode = 1;
    [self.context.jdCloudPlayer pause];
    [self.context releaseViewController];
    [self removeFromSuperview];
    AppDelegate* app = (AppDelegate*)[UIApplication sharedApplication].delegate;
    app.playerSmallWindow = nil;
}

- (void)hidenPlayer
{
    [self.context.jdCloudPlayer pause];
    self.hidden = YES;
}

- (void)reShowPlayer{
    [self.context.jdCloudPlayer resume];
    self.hidden = NO;
}

@end
