//
//  JDCloudPlayerTopView.m
//  IJKMediaDemo
//
//  Created by 王亚伟 on 2018/9/18.
//  Copyright © 2018年 wangyawei All rights reserved.
//

#import "JDCloudPlayerTopView.h"
#import "MoreBottomAlertView.h"
#import "JDCloudPlayerSmallWindow.h"
#import "JDCloudPlaySetsModel.h"

@interface JDCloudPlayerTopView()

@property (nonatomic, strong) UILabel *titleLabel;            //标题
@property (nonatomic, strong) UIButton *backButton;           //返回按钮
@property (nonatomic, strong) UIButton *moreButton;           //更多

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;

@end

@implementation JDCloudPlayerTopView


- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context{
    if (self = [super initWithFrame:frame]) {
        self.context = context;
        [self addSubviews];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化子试图
- (void)addSubviews
{
    [self addSubview:self.titleLabel];
    [self addSubview:self.backButton];
    [self addSubview:self.moreButton];
}

- (void)layoutSubviews{
    [super layoutSubviews];
    
    float width = self.bounds.size.width;
    float height = self.bounds.size.height;
    
    self.backButton.frame = CGRectMake(0, 0, 44, height);
    self.titleLabel.frame = CGRectMake(44, 0, width - 88 , height);
    self.moreButton.frame = CGRectMake(width - 44, 0, 44,height);
    
    //屏幕旋转通知
    [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(handleDeviceOrientationDidChange:)
                                                 name:UIDeviceOrientationDidChangeNotification
                                               object:nil
     ];
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //返回
        {
            if ([JDStaticTools isInterfaceOrientationPortrait]) { //竖屏
                //先变小
                NSDictionary *dataDic = [NSDictionary dictionaryWithObject:@"NO" forKey:@"isFullScreen"];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayScreenChangeNotification" object:nil userInfo:dataDic];
                
                //当视频播放状态下，返回时出现悬浮窗口
                if(self.context.jdCloudPlayer.isPlaying && !SharedSetVideoPlayManager.isOnlyAudio){
                    JDCloudPlayerSmallWindow* smallWindow = [[JDCloudPlayerSmallWindow alloc] initWithContext:self.context];
                    [smallWindow showInWindow];
                    [self.context hideViewController];
                }else{
                    
                    //清除之前设置
//                    SharedSetVideoPlayManager.isMirrorPlay = NO;
                    self.context.infoModel.isMirrorPlay = NO;
                    SharedSetVideoPlayManager.isOnlyAudio = NO;
                    SharedSetVideoPlayManager.isBackgroundPlay = NO;
                    SharedSetVideoPlayManager.scalingMode = 1;
                    [self.context popViewController];
                }
            }
            else{ //横屏
                //变成小屏 发通知
                NSDictionary *dataDic = [NSDictionary dictionaryWithObject:@"NO" forKey:@"isFullScreen"];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayScreenChangeNotification" object:nil userInfo:dataDic];
            }
        }
            break;
        case 101:   //更多
        {
            for (UIView* view in self.superview.subviews) {
                if ([view isKindOfClass:[MoreBottomAlertView class]]) {
                    [view removeFromSuperview];
                }
            }
            MoreBottomAlertView* moreView = [[MoreBottomAlertView alloc]initWithFrame:CGRectMake(0, 0, ScreenWidth, ScreenHeight) withContext:self.context];
            
            if ([JDStaticTools isInterfaceOrientationPortrait]) { //竖屏
                [_context showBottomPopupView:moreView];
                
            }else{ //横屏
                [self.superview insertSubview:moreView atIndex:[self.superview.subviews count]];
            }
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - 全屏设置
//监听屏幕变化
- (void)handleDeviceOrientationDidChange:(UIInterfaceOrientation)interfaceOrientation{
    UIDevice *device = [UIDevice currentDevice] ;
    switch (device.orientation) {
        case UIDeviceOrientationFaceUp:
        case UIDeviceOrientationFaceDown:
        case UIDeviceOrientationUnknown:
        case UIDeviceOrientationPortraitUpsideDown:
        case UIDeviceOrientationLandscapeLeft:
        case UIDeviceOrientationLandscapeRight:
        case UIDeviceOrientationPortrait:
        {
            for (UIView* view in self.superview.subviews) {
                if ([view isKindOfClass:[MoreBottomAlertView class]]) {
                    [view removeFromSuperview];
                }
            }
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark - 懒加载
//标题
- (UILabel *)titleLabel{
    if(!_titleLabel){
        _titleLabel = [[UILabel alloc] init];
        [_titleLabel setTextColor:[UIColor colorWithRed:(231 / 255.0) green:(231 / 255.0) blue:(231 / 255.0) alpha:1]];
        [_titleLabel setFont:[UIFont systemFontOfSize:15.0f * SCALE_FONT]];
        _titleLabel.text = @"京东云发展战略部署";
    }
    return _titleLabel;
}

//返回按钮
- (UIButton *)backButton{
    if (!_backButton){
        _backButton = [[UIButton alloc] init];
        [_backButton setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateNormal];
        [_backButton setBackgroundImage:[UIImage imageNamed:@"返回"] forState:UIControlStateSelected];
        _backButton.tag = 100;
        [_backButton sizeToFit];
        [_backButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _backButton;
}

//更多按钮
- (UIButton *)moreButton{
    if (!_moreButton) {
        _moreButton = [[UIButton alloc] init];
        [_moreButton setBackgroundImage:[UIImage imageNamed:@"竖屏-更多操作"] forState:UIControlStateNormal];
        [_moreButton setBackgroundImage:[UIImage imageNamed:@"竖屏-更多操作"] forState:UIControlStateSelected];
        _moreButton.tag = 101;
        [_backButton sizeToFit];
        [_moreButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _moreButton;
}

//moreView
//- (MoreBottomAlertView *)moreView{
//    if (!_moreView) {
//        _moreView = [[MoreBottomAlertView alloc]initWithFrame:CGRectMake(0, 0, ScreenWidth, ScreenHeight) withContext:self.context];
//    }
//    return _moreView;
//}



@end

