//
//  MoreBottomAlertView.m
//  jdcloud_ios_live_demo
//
//  Created by 彭丹丹 on 2018/10/8.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "MoreBottomAlertView.h"
#import "JDShowDocumentViewController.h"
#import "JDCloudPlaySetsModel.h"
#import "JDCloudOnlyAudioPlayView.h"
#import "JDCloudMainVideoPlayView.h"

@interface MoreBottomAlertView()

@property (nonatomic, strong) UIView* containerView;                     //bgView
@property (nonatomic, strong) UIButton* lookDocumentBtn;                 //查看文档
@property (nonatomic, strong) UIButton* onlyAudioPlayBtn;                //纯音频播放
@property (nonatomic, strong) UIButton* exSubtitlesBtn;                  //外挂字幕
@property (nonatomic, strong) UIImageView* lineImageView;                //线
@property (nonatomic, strong) UILabel* mirrorPlayLabel;                  //镜像播放title
@property (nonatomic, strong) UISwitch* mirrorPlaySwitch;                //镜像播放
@property (nonatomic, strong) UILabel* backgroundPlayLabel;              //后台播放title
@property (nonatomic, strong) UISwitch* backgroundPlaySwitch;            //后台播放
@property (nonatomic, strong) UILabel* zoomLabel;                        //缩放模式title
@property (nonatomic, strong) NSArray<UIButton *> *zoomButtons;          //缩放模式btn
@property (nonatomic, strong) UIButton *cancelButton;                    //取消btn
@property (nonatomic, strong) UIButton *fillScreen;                      //全屏
@property (nonatomic, strong) UIButton *aspectFill;                      //剪裁
@property (nonatomic, strong) UIButton *aspectFit;                       //填充

@property (nonatomic, assign) BOOL isExSubtitlesOpen;                    //是否开启外挂字幕。 YES：是
@property (nonatomic, strong) UIButton *zoomTypeButton;                  //选中缩放模式{填充、裁剪、全屏}
@property (nonatomic, strong) UIButton *selectedButton;                  //选中缩放模式{纯音频播放、查看文档、外挂字幕}

@property (nonatomic, weak) id<JDVideoPlayContextProtocol> context;

@end

@implementation MoreBottomAlertView

- (instancetype)initWithFrame:(CGRect)frame withContext:(id<JDVideoPlayContextProtocol>)context{
    self = [super initWithFrame:frame];
    if (self) {
        self.context = context;
        [self setupData];
        [self setupSubviews];
        [self resetZoomButtons];
        [self addNotifications];
    }
    return self;
}

#pragma mark -
#pragma mark - 初始化数据
- (void)setupData
{
//    [self.mirrorPlaySwitch setOn:SharedSetVideoPlayManager.isMirrorPlay];
    [self.mirrorPlaySwitch setOn:self.context.infoModel.isMirrorPlay];
    [self.backgroundPlaySwitch setOn:SharedSetVideoPlayManager.isBackgroundPlay];
}

#pragma mark -
#pragma mark - 初始化子视图
- (void)setupSubviews
{
    //背景色
    self.backgroundColor = RGBACOLOR(255, 255, 255, 0);
    
    [self addSubview:self.containerView];
    
    [self.containerView addSubview:self.lookDocumentBtn];
    [self.containerView addSubview:self.onlyAudioPlayBtn];
    [self.containerView addSubview:self.exSubtitlesBtn];
    [self.containerView addSubview:self.lineImageView];
    [self.containerView addSubview:self.mirrorPlaySwitch];
    [self.containerView addSubview:self.backgroundPlaySwitch];
    [self.containerView addSubview:self.zoomLabel];
    [self.containerView addSubview:self.fillScreen];
    [self.containerView addSubview:self.aspectFit];
    [self.containerView addSubview:self.aspectFill];
    [self.containerView addSubview:self.cancelButton];
}

- (void)resetZoomButtons
{
    switch (SharedSetVideoPlayManager.scalingMode) {
        case JDCloudScalingModeAspectFit: //填充
        {
            self.aspectFit.selected = YES;
            self.aspectFill.selected = NO;
            self.fillScreen.selected = NO;
            self.zoomTypeButton = self.aspectFit;
       }
            break;
        case JDCloudScalingModeAspectFill: //剪裁
        {
            self.aspectFit.selected = NO;
            self.aspectFill.selected = YES;
            self.fillScreen.selected = NO;
            self.zoomTypeButton = self.aspectFill;
        }
            break;
        case JDCloudScalingModeFill: //全屏
        {
            self.aspectFit.selected = NO;
            self.aspectFill.selected = NO;
            self.fillScreen.selected = YES;
            self.zoomTypeButton = self.fillScreen;
        }
            break;
        default:
        {
            self.aspectFit.selected = NO;
            self.aspectFill.selected = NO;
            self.fillScreen.selected = YES;
            self.zoomTypeButton = self.fillScreen;
        }
            break;
    }
}

- (void)layoutSubviews{
    [super layoutSubviews];

    if ([JDStaticTools isInterfaceOrientationPortrait]) { //竖屏

        self.frame = CGRectMake(0, 0, self.frame.size.width, self.frame.size.height);
        self.containerView.frame = CGRectMake(0, self.frame.size.height - 234 - SafeAreaBottomHeight, self.frame.size.width, 234 + SafeAreaBottomHeight);
        [self setViewLocation];
        
        self.cancelButton.hidden = NO;
        self.zoomLabel.hidden = YES;
        self.fillScreen.hidden = YES;
        self.aspectFit.hidden = YES;
        self.aspectFill.hidden = YES;
        
    }else{  //横屏

        self.frame = CGRectMake(0, 0, self.frame.size.width, self.frame.size.height);
        self.containerView.frame = CGRectMake(ScreenWidth - 300, 0, 300, ScreenHeight);
        [self setViewLocation];
        
        self.cancelButton.hidden = YES;
        self.zoomLabel.hidden = NO;
        self.fillScreen.hidden = NO;
        self.aspectFit.hidden = NO;
        self.aspectFill.hidden = NO;
    }
}

//设置位置
- (void)setViewLocation{
    
    float width = self.containerView.frame.size.width;
    float height = self.containerView.frame.size.height;
    float btnHeight = 104;
    
    //查看文档 top, left, bottom, right
    self.lookDocumentBtn.frame = CGRectMake(0, 0, width/3, btnHeight);
   // self.lookDocumentBtn.backgroundColor = [UIColor greenColor];
    [self.lookDocumentBtn setImageEdgeInsets:UIEdgeInsetsMake(-(btnHeight-44)/2, (width/6 - 50/2), 0, 0)];
    [self.lookDocumentBtn setTitleEdgeInsets:UIEdgeInsetsMake(0, -(width/3  - 44)/2, - btnHeight/2, 0)];
    
    //正常播放
    self.onlyAudioPlayBtn.frame = CGRectMake(width/3, 0, width/3, btnHeight);
    [self.onlyAudioPlayBtn setImageEdgeInsets:UIEdgeInsetsMake(-(btnHeight-44)/2, (width/6 - 50/2), 0, 0)];
    [self.onlyAudioPlayBtn setTitleEdgeInsets:UIEdgeInsetsMake(0, -(width/3  - 44)/2, - btnHeight/2, 0)];
    
    //外挂字幕
    self.exSubtitlesBtn.frame = CGRectMake(width/3*2, 0, width/3, btnHeight);
    [self.exSubtitlesBtn setImageEdgeInsets:UIEdgeInsetsMake(-(btnHeight-44)/2, (width/6 - 50/2), 0, 0)];
    [self.exSubtitlesBtn setTitleEdgeInsets:UIEdgeInsetsMake(0, -(width/3  - 44)/2, - btnHeight/2, 0)];
    
    //镜像、后台、取消
    self.lineImageView.frame = CGRectMake(0, btnHeight, width, 1);
    self.mirrorPlayLabel.frame = CGRectMake(15, btnHeight + 20, 80, 34);
    self.mirrorPlaySwitch.frame = CGRectMake(90, btnHeight + 20, 46, 34);
    self.backgroundPlayLabel.frame = CGRectMake(width/2 + 15, btnHeight + 20, 80, 34);
    self.backgroundPlaySwitch.frame = CGRectMake(width/2+80+15, btnHeight + 20, 46, 34);
    self.cancelButton.frame = CGRectMake(0, height - 60, width, 60);
    
    //剪裁、全屏
    self.zoomLabel.frame = CGRectMake(15, btnHeight + 74, 80, 30);
    self.fillScreen.frame = CGRectMake(85+65*2, btnHeight + 74, 55, 30);
    self.aspectFit.frame = CGRectMake(85+65, btnHeight + 74, 55, 30);
    self.aspectFill.frame = CGRectMake(85, btnHeight + 74, 55, 30);
}

#pragma mark -
#pragma mark - 点击事件
- (void)buttonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    switch (button.tag) {
        case 100:   //查看文档
        {
          //  button.selected = !button.selected;
          //  self.selectedButton.selected = NO;
          //  self.selectedButton = button;

            //横屏先变小
            if (![JDStaticTools isInterfaceOrientationPortrait]) {
                [self removeFromSuperview];
                NSDictionary *dataDic = [NSDictionary dictionaryWithObject:@"NO" forKey:@"isFullScreen"];
                [[NSNotificationCenter defaultCenter] postNotificationName:@"JDVideoPlayScreenChangeNotification" object:nil userInfo:dataDic];
            }
            
            //去查看文档页面
            JDShowDocumentViewController* VC = [[JDShowDocumentViewController alloc]init];
            [_context pushViewController:VC];
        }
            break;
        case 101:  //纯音频播放
        {
            
            if (!SharedSetVideoPlayManager.isOnlyAudio) {
                SharedSetVideoPlayManager.isOnlyAudio = YES;
                self.context.jdCloudPlayer.isOnlyAudio = YES;
                button.selected = YES;
                
            }else{
                SharedSetVideoPlayManager.isOnlyAudio = NO;
                self.context.jdCloudPlayer.isOnlyAudio = NO;
                button.selected = NO;
                
            }
            for (UIView* view in self.superview.subviews) {
                if ([view isKindOfClass:[JDCloudMainVideoPlayView class]]) {
                    [view setNeedsLayout];
                    return;
                }
            }
            for (UIView* view in self.superview.superview.subviews) {
                if ([view isKindOfClass:[JDCloudMainVideoPlayView class]]) {
                    [view setNeedsLayout];
                    return;
                }
            }
        }
            break;
        case 102:  //外挂字幕
        {
            button.selected = !button.selected;
           // self.selectedButton.selected = NO;
           // self.selectedButton = button;
            
            //设置外挂字幕
          //  if (button.selected) {
                
//                [_context.jdCloudPlayer  setExternalSubtitle:@""];
          //  } else {
//                [_context.jdCloudPlayer  setExternalSubtitle:nil];
          //  }
           // self.isExSubtitlesOpen = button.selected;
        }
            break;
        case 105:  //取消
        {
            [self removeFromSuperview];
        }
            break;
        default:
            break;
    }
}

- (void)switchClickAction:(UISwitch *)sender
{
    UISwitch* selectedSwitch = (UISwitch*)sender;
    switch (selectedSwitch.tag) {
        case 200:  //镜像播放
        {
//            SharedSetVideoPlayManager.isMirrorPlay = !SharedSetVideoPlayManager.isMirrorPlay;
            self.context.infoModel.isMirrorPlay = !self.context.infoModel.isMirrorPlay;
            [selectedSwitch setOn:self.context.infoModel.isMirrorPlay];
            self.context.jdCloudPlayer.isMirrorPlay = selectedSwitch.isOn;
        }
            break;
        case 201: //后台播放
        {
            SharedSetVideoPlayManager.isBackgroundPlay = !SharedSetVideoPlayManager.isBackgroundPlay;
            [selectedSwitch setOn:SharedSetVideoPlayManager.isBackgroundPlay];
            self.context.jdCloudPlayer.isPlayOnBackGroud = selectedSwitch.isOn;
        }
            break;
        default:
            break;
    }
}

- (void)modelButtonClickAction:(UIButton *)sender
{
    UIButton* button = (UIButton *)sender;
    button.selected = YES;
    if (button != self.zoomTypeButton) {
        self.zoomTypeButton.selected = NO;
        self.zoomTypeButton = button;
    }
    switch (button.tag) {
        case 300:   //填充
        {
            self.context.jdCloudPlayer.scalingMode = JDCloudScalingModeAspectFit;
            SharedSetVideoPlayManager.scalingMode = JDCloudScalingModeAspectFit;
        }
            break;
        case 301:  //裁剪
        {
            self.context.jdCloudPlayer.scalingMode = JDCloudScalingModeAspectFill;
            SharedSetVideoPlayManager.scalingMode = JDCloudScalingModeAspectFill;
        }
            break;
        case 302:   //全屏
        {
            self.context.jdCloudPlayer.scalingMode = JDCloudScalingModeFill;
            SharedSetVideoPlayManager.scalingMode = JDCloudScalingModeFill;
        }
            break;
        default:
            break;
    }
}

//点击其他区域 view移除
- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
     [self removeFromSuperview];
    
   // UITouch *touch  = touches.anyObject;
    //if ([touch.view isKindOfClass:[self.containerView class]]) {
     //   [self removeFromSuperview];
   // }
}

#pragma mark -
#pragma mark - cazy init
- (UIView *)containerView{
    if (!_containerView) {
        _containerView = [[UIView alloc]init];
        _containerView.backgroundColor = RGBACOLOR(0, 0, 0, 0.7);
    }
    return _containerView;
}

//查看文档
- (UIButton*)lookDocumentBtn{
    if (!_lookDocumentBtn){
        _lookDocumentBtn = [[UIButton alloc]init];
        [_lookDocumentBtn setTitle:@"查看文档" forState:UIControlStateNormal];
        _lookDocumentBtn.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_lookDocumentBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_lookDocumentBtn setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_lookDocumentBtn setImage:[UIImage imageNamed:@"竖屏-查看文档" ]forState:UIControlStateNormal];
        _lookDocumentBtn.tag = 100;
        [_lookDocumentBtn addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _lookDocumentBtn;
}

//纯音频播放
- (UIButton*)onlyAudioPlayBtn{
    if (!_onlyAudioPlayBtn){
        _onlyAudioPlayBtn = [[UIButton alloc]init];
        [_onlyAudioPlayBtn setTitle:@"纯音频播放" forState:UIControlStateNormal];
        _onlyAudioPlayBtn.selected = SharedSetVideoPlayManager.isOnlyAudio;
        _onlyAudioPlayBtn.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_onlyAudioPlayBtn setTitleEdgeInsets:UIEdgeInsetsMake(0, 0, - 57, 0)];
        [_onlyAudioPlayBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_onlyAudioPlayBtn setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_onlyAudioPlayBtn setImage:[UIImage imageNamed:@"竖屏-纯音频播放" ]forState:UIControlStateNormal];
        _onlyAudioPlayBtn.tag = 101;
        [_onlyAudioPlayBtn addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _onlyAudioPlayBtn;
}

//外挂字幕
- (UIButton*)exSubtitlesBtn{
    if (!_exSubtitlesBtn){
        _exSubtitlesBtn = [[UIButton alloc]init];
        [_exSubtitlesBtn setTitle:@"外挂字幕" forState:UIControlStateNormal];
        _exSubtitlesBtn.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_exSubtitlesBtn setTitleEdgeInsets:UIEdgeInsetsMake(0, 0, - 57, 0)];
        [_exSubtitlesBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_exSubtitlesBtn setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_exSubtitlesBtn setImage:[UIImage imageNamed:@"竖屏-外挂字幕" ]forState:UIControlStateNormal];
        _exSubtitlesBtn.tag = 102;
        [_exSubtitlesBtn addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _exSubtitlesBtn;
}

//线
- (UIImageView*)lineImageView{
    if (!_lineImageView) {
        _lineImageView = [[UIImageView alloc] init];
        _lineImageView.backgroundColor = [UIColor whiteColor];
    }
    return _lineImageView;
}

//镜像播放title
- (UILabel *)mirrorPlayLabel{
    if (!_mirrorPlayLabel) {
        _mirrorPlayLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 80, 60)];
        _mirrorPlayLabel.textAlignment = NSTextAlignmentLeft;
        _mirrorPlayLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        _mirrorPlayLabel.textColor = [UIColor whiteColor];
        _mirrorPlayLabel.text = @"镜像播放";
        [self.containerView addSubview:_mirrorPlayLabel];
    }
    return _mirrorPlayLabel;
}

//镜像播放
- (UISwitch *)mirrorPlaySwitch{
    if (!_mirrorPlaySwitch){
        _mirrorPlaySwitch = [[UISwitch alloc]init];
//        [_mirrorPlaySwitch setOn:SharedSetVideoPlayManager.isMirrorPlay];
        [_mirrorPlaySwitch setOn:self.context.infoModel.isMirrorPlay];
        _mirrorPlaySwitch.tag = 200;
        _mirrorPlaySwitch.onTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
        _mirrorPlaySwitch.tintColor = [UIColor grayColor];
        [_mirrorPlaySwitch addTarget:self action:@selector(switchClickAction:) forControlEvents:UIControlEventValueChanged];
    }
    return _mirrorPlaySwitch;
}

//后台播放title
- (UILabel *)backgroundPlayLabel{
    if (!_backgroundPlayLabel) {
        _backgroundPlayLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 0, 80, 60)];
        _backgroundPlayLabel.textAlignment = NSTextAlignmentLeft;
        _backgroundPlayLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        _backgroundPlayLabel.textColor = [UIColor whiteColor];
        _backgroundPlayLabel.text = @"后台播放";
        [self.containerView addSubview:_backgroundPlayLabel];
    }
    return _backgroundPlayLabel;
}

//后台播放
- (UISwitch *)backgroundPlaySwitch{
    if (!_backgroundPlaySwitch)
    {
        _backgroundPlaySwitch = [[UISwitch alloc]init];
        [_backgroundPlaySwitch setOn:SharedSetVideoPlayManager.isBackgroundPlay];
        _backgroundPlaySwitch.tag = 201;
        _backgroundPlaySwitch.onTintColor = [JDStaticTools colorWithHexString:kJDVCThemeColor];
        _backgroundPlaySwitch.tintColor = [UIColor grayColor];
        [_backgroundPlaySwitch addTarget:self action:@selector(switchClickAction:) forControlEvents:UIControlEventValueChanged];
    }
   return _backgroundPlaySwitch;
}

//缩放模式title
- (UILabel *)zoomLabel{
    if (!_zoomLabel) {
        _zoomLabel = [[UILabel alloc] init];
        _zoomLabel.textAlignment = NSTextAlignmentLeft;
        _zoomLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        _zoomLabel.textColor = [UIColor whiteColor];
        _zoomLabel.text = @"缩放模式";
    }
    return _zoomLabel;
}

//取消按钮
- (UIButton*)cancelButton{
    if(!_cancelButton){
        _cancelButton = [JDStaticTools setupButtonframe:CGRectZero title:@"取消" fontSize:17.*SCALE_FONT textColor:[UIColor whiteColor] normalImageName:nil hightLightedImageName:nil tag:105];
        _cancelButton.backgroundColor = RGBCOLOR(39, 47, 80);
        [_cancelButton addTarget:self action:@selector(buttonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _cancelButton;
}

//填充
- (UIButton *)aspectFit{
    if (!_aspectFit) {
        _aspectFit = [[UIButton alloc]init];
        _aspectFit.tag = 300;
        _aspectFit.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_aspectFit setTitle:@"填充" forState:UIControlStateNormal];
        [_aspectFit setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_aspectFit setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_aspectFit setBackgroundImage:[UIImage imageNamed:@"按钮_白框"] forState:UIControlStateNormal];
        [_aspectFit setBackgroundImage:[UIImage imageNamed:@"按钮_蓝框"] forState:UIControlStateSelected];
        [_aspectFit addTarget:self action:@selector(modelButtonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _aspectFit;
}

//裁剪
- (UIButton *)aspectFill{
    if (!_aspectFill) {
        _aspectFill = [[UIButton alloc]init];
        _aspectFill.tag = 301;
        _aspectFill.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_aspectFill setTitle:@"裁剪" forState:UIControlStateNormal];
        [_aspectFill setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_aspectFill setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_aspectFill setBackgroundImage:[UIImage imageNamed:@"按钮_白框"] forState:UIControlStateNormal];
        [_aspectFill setBackgroundImage:[UIImage imageNamed:@"按钮_蓝框"] forState:UIControlStateSelected];
        [_aspectFill addTarget:self action:@selector(modelButtonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _aspectFill;
}

//全屏
- (UIButton *)fillScreen{
    if (!_fillScreen) {
        _fillScreen = [[UIButton alloc]init];
        _fillScreen.tag = 302;
        _fillScreen.titleLabel.font = [UIFont systemFontOfSize:14 * SCALE_FONT];
        [_fillScreen setTitle:@"全屏" forState:UIControlStateNormal];
        [_fillScreen setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
        [_fillScreen setTitleColor:[JDStaticTools colorWithHexString:kJDVCThemeColor] forState:UIControlStateSelected];
        [_fillScreen setBackgroundImage:[UIImage imageNamed:@"按钮_白框"] forState:UIControlStateNormal];
        [_fillScreen setBackgroundImage:[UIImage imageNamed:@"按钮_蓝框"] forState:UIControlStateSelected];
        [_fillScreen addTarget:self action:@selector(modelButtonClickAction:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _fillScreen;
}

#pragma mark -
#pragma mark - 通知
- (void)addNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeOnlyAudioState:) name:@"JDVideoPlayOnlyAudioChangeNotification" object:nil];
}

//纯音频改变，按钮选中改变
- (void)changeOnlyAudioState:(NSNotification *)infoNotification
{
    self.selectedButton.selected = NO;
}

@end
