//
//  JDCloudVideoTrimmerView.m
//  jdcloud_ios_live_demo
//
//  Created by 王亚伟 on 2018/11/7.
//  Copyright © 2018年 pengdandan. All rights reserved.
//

#import "JDCloudVideoTrimmerView.h"
#import "JDCloudVideoThumbView.h"
#import "JDCloudVideoGifAllImagesView.h"

@interface JDCloudVideoTrimmerView()

@property (strong, nonatomic)JDCloudVideoGifAllImagesView *allImagesView;
@property (strong, nonatomic) UIView *leftOverlayView;
@property (strong, nonatomic) UIView *rightOverlayView;
@property (strong, nonatomic) JDCloudVideoThumbView *leftThumbView;
@property (strong, nonatomic) JDCloudVideoThumbView *rightThumbView;

@property (strong, nonatomic) UIView *middleOverLayerView;
@property (strong, nonatomic) UIView *trackerView;
@property (strong, nonatomic) UIView *topBorder;
@property (strong, nonatomic) UIView *bottomBorder;

@property (nonatomic) CGFloat totalTime;
@property (nonatomic) CGFloat startTime;
@property (nonatomic) CGFloat endTime;

@property (nonatomic) CGPoint leftStartPoint;
@property (nonatomic) CGPoint middleStartPoint;
@property (nonatomic) CGPoint rightStartPoint;
@property (nonatomic) CGFloat overlayWidth;

@end

@implementation JDCloudVideoTrimmerView

#pragma mark - Initiation
- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.clipsToBounds = YES;
        [self setBackgroundColor:[UIColor clearColor]];
        [self addSubview:self.allImagesView];
        [self addSubview:self.leftOverlayView];
        [self addSubview:self.rightOverlayView];
        [self addSubview:self.topBorder];
        [self addSubview:self.bottomBorder];
        [self addSubview:self.middleOverLayerView];
        [self.middleOverLayerView addSubview:self.trackerView];
    }
    return self;
}


#pragma mark - Private methods
- (void)resetSubviewsWithTotalTime:(CGFloat)totalTime
                         StartTime:(CGFloat)startTime
                          duration:(CGFloat)duration;
{
    [self.topBorder setBackgroundColor:self.themeColor];
    [self.bottomBorder setBackgroundColor:self.themeColor];
    CGRect leftThumbViewRect = CGRectMake(startTime/totalTime * self.width, 0, 10, self.height);
    if (self.leftThumbImage) {
        self.leftThumbView = [[JDCloudVideoThumbView alloc] initWithFrame:leftThumbViewRect thumbImage:self.leftThumbImage];
    } else {
        self.leftThumbView = [[JDCloudVideoThumbView alloc] initWithFrame:leftThumbViewRect color:self.themeColor right:NO];
    }
    [self.leftThumbView.layer setMasksToBounds:YES];
    UIPanGestureRecognizer *leftPanGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(moveLeftThumbView:)];
    [self.leftThumbView addGestureRecognizer:leftPanGestureRecognizer];
    [self addSubview:self.leftThumbView];
    
    [self.leftOverlayView setUserInteractionEnabled:YES];
    [self.leftOverlayView setBackgroundColor:[UIColor colorWithWhite:1 alpha:0.2]];
    
    
    self.trackerView.backgroundColor = self.trackerColor;
    self.trackerView.frame = CGRectMake(self.leftThumbView.x + self.leftThumbView.width, 4, 4, self.height - 8);
    self.trackerView.layer.masksToBounds = true;
    
    CGRect rightThumbViewRect = CGRectMake((startTime + duration)/totalTime * self.width - 10, 0, 10, self.height);
    if (self.rightThumbImage) {
        self.rightThumbView = [[JDCloudVideoThumbView alloc] initWithFrame:rightThumbViewRect thumbImage:self.rightThumbImage];
    } else {
        self.rightThumbView = [[JDCloudVideoThumbView alloc] initWithFrame:rightThumbViewRect color:self.themeColor right:YES];
    }
    [self.rightThumbView.layer setMasksToBounds:YES];
    UIPanGestureRecognizer *rightPanGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(moveRightThumbView:)];
    [self.rightThumbView addGestureRecognizer:rightPanGestureRecognizer];
    [self addSubview:self.rightThumbView];

    [self.rightOverlayView setUserInteractionEnabled:YES];
    [self.rightOverlayView setBackgroundColor:[UIColor colorWithWhite:1 alpha:0.2]];
    
    UIPanGestureRecognizer *middlePanGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(moveMiddleThumbView:)];
    [self.middleOverLayerView addGestureRecognizer:middlePanGestureRecognizer];
    
    [self updateSubViews];
    
    self.totalTime = totalTime;
    self.startTime = startTime;
    self.endTime = startTime + duration;
//    [self notifyDelegate];
    
}

- (void)setThumbnailCount:(NSInteger)thumbnailCount
{
    self.allImagesView.imagesTotalCount = thumbnailCount;
}

- (void)addThumbnailPaths:(NSArray *)thumbnailPaths
{
    [self.allImagesView addImagePaths:thumbnailPaths];
}

- (void)moveLeftThumbView:(UIPanGestureRecognizer *)gesture
{
    switch (gesture.state) {
        case UIGestureRecognizerStateBegan:
            self.leftStartPoint = [gesture locationInView:self];
            break;
        case UIGestureRecognizerStateChanged:
        {
            CGPoint point = [gesture locationInView:self];
            if (point.x + self.width/6 >= self.rightThumbView.left) {
                return;
            } else {
                //            int deltaX = point.x - self.leftStartPoint.x;
                self.leftThumbView.left = point.x;
                self.leftStartPoint = point;
                [self updateSubViews];
            }
            break;
        }
        case UIGestureRecognizerStateEnded:
        {
             [self notifyDelegate];
        }
            
        default:
            break;
    }
    
    
}

- (void)moveMiddleThumbView:(UIPanGestureRecognizer *)gesture
{
    CGFloat leftToRightWidth = self.rightThumbView.left - self.leftThumbView.right;
    switch (gesture.state) {
        case UIGestureRecognizerStateBegan:
            self.middleStartPoint = [gesture locationInView:self];
            break;
        case UIGestureRecognizerStateChanged:
        {
            CGPoint point = [gesture locationInView:self];
            int deltaX = point.x - self.middleStartPoint.x;
            if (deltaX < 0) {
                if (self.leftThumbView.left + deltaX < 0) {
                    self.leftThumbView.left = 0;
                    self.rightThumbView.left = self.leftThumbView.right + leftToRightWidth;
                    [self updateSubViews];
//                    [self notifyDelegate];
                }
            } else if (deltaX > 0) {
                if (self.rightThumbView.right + deltaX > self.width) {
                    self.rightThumbView.left = self.width -  self.rightThumbView.width;
                    self.leftThumbView.right = self.rightThumbView.left - leftToRightWidth;
                    [self updateSubViews];
//                    [self notifyDelegate];
                }
            }
            //            CGPoint center = self.rightThumbView.center;
            
            
            self.leftThumbView.left = self.leftThumbView.left + deltaX;
            self.rightThumbView.left = self.rightThumbView.left + deltaX;
            self.middleStartPoint = point;
            [self updateSubViews];
            break;
        }
        case UIGestureRecognizerStateEnded:
        {
            [self notifyDelegate];
        }
            
        default:
            break;
    }
}

- (void)moveRightThumbView:(UIPanGestureRecognizer *)gesture
{
    switch (gesture.state) {
        case UIGestureRecognizerStateBegan:
            self.rightStartPoint = [gesture locationInView:self];
            break;
        case UIGestureRecognizerStateChanged:
        {
            CGPoint point = [gesture locationInView:self];
            
            if (point.x - self.width/6 <= self.leftThumbView.right) {
                return;
            } else {
                //            int deltaX = point.x - self.rightStartPoint.x;
                
                //            CGPoint center = self.rightThumbView.center;
                
                
                self.rightThumbView.left = point.x;
                self.rightStartPoint = point;
                [self updateSubViews];
                [self notifyDelegate];
            }
            break;
        }
            
        default:
            break;
    }
}

- (void)updateSubViews
{
    CGFloat height = self.borderWidth;
    [self.topBorder setFrame:CGRectMake(self.leftThumbView.right, 0, self.rightThumbView.left-self.leftThumbView.right, height)];
    [self.bottomBorder setFrame:CGRectMake(self.leftThumbView.right, self.height - height, self.rightThumbView.left-self.leftThumbView.right, height)];
    self.leftOverlayView.width = self.leftThumbView.left;
    self.rightOverlayView.left = self.rightThumbView.right;
    self.rightOverlayView.width = self.width - self.rightThumbView.right;
    self.middleOverLayerView.left = self.leftThumbView.right;
    self.middleOverLayerView.width = self.rightThumbView.left - self.leftThumbView.right;
    self.trackerView.frame = CGRectMake(0, height, 3, self.height - height * 2);
}

- (void)notifyDelegate
{
    self.startTime = self.leftThumbView.left * self.totalTime/ScreenWidth;
    self.endTime = self.rightThumbView.left * self.totalTime/ScreenWidth;
    if (self.delegate && [self.delegate respondsToSelector:@selector(trimmerDidChangeLeftPosition:rightPosition:)]) {
        [self.delegate trimmerDidChangeLeftPosition:self.startTime rightPosition:self.endTime];
    }
}

#pragma mark - 懒加载
-(JDCloudVideoGifAllImagesView *)allImagesView
{
    if (!_allImagesView) {
        _allImagesView = [[JDCloudVideoGifAllImagesView alloc] initWithFrame:CGRectMake(0, 0, self.width, self.height)];
    }
    return _allImagesView;
}

-(UIView *)topBorder
{
    if (!_topBorder) {
        _topBorder = [[UIView alloc] init];
    }
    return _topBorder;
}

-(UIView *)bottomBorder
{
    if(!_bottomBorder){
        _bottomBorder = [[UIView alloc] init];
    }
    return _bottomBorder;
}

-(UIView *)leftOverlayView
{
    if(!_leftOverlayView){
        _leftOverlayView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, self.height)];
    }
    return _leftOverlayView;
}

-(UIView *)middleOverLayerView
{
    if (!_middleOverLayerView) {
        _middleOverLayerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, self.height)];
        _middleOverLayerView.backgroundColor = [UIColor clearColor];
    }
    return _middleOverLayerView;
}


-(UIView *)trackerView
{
    if(!_trackerView){
        _trackerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, 0)];
    }
    return _trackerView;
}

-(UIView *)rightOverlayView
{
    if(!_rightOverlayView){
        _rightOverlayView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, self.height)];
    }
    return _rightOverlayView;
}

@end
