//
//  JDCloudPlayerDelegate.h
//


#ifndef JDCloudPlayerDelegate_h
#define JDCloudPlayerDelegate_h

#import "JDCloudPlayerVideoErrorModel.h"

@class JDCloudPlayer;

/*!
   @protocol JDCloudPlayerDelegate
   @abstract 京东云播放器SDK-播放代理方法类
   @discussion 本类包含了所有播放相关的代理回调接口方法，包括播放状态变化，错误回调等功能
 */
@protocol JDCloudPlayerDelegate <NSObject>

@required

/*!
   @abstract  播放改变的各种状态
   @param player  播放器
   @param event      改变状态事件
   @discussion see JDCloudPlayerEvent
 */
- (void)player:(JDCloudPlayer *)player onEventCallback:(JDCloudPlayerEvent)event;

/*!
   @abstract  播放器播放时发生错误时的回调信息
   @param player   播放器
   @param errorModel  播放器报错时提供的错误信息对象
   @discussion see JDCloudPlayerVideoErrorModel
 */
- (void)player:(JDCloudPlayer *)player playBackErrorModel:(JDCloudPlayerVideoErrorModel *)errorModel;

/*!
  @abstract  接收用户自定义数据（通过SEI传输）
  @param     player   播放器
  @param     message  自定义消息
  @discussion 需要调用方去重
*/
- (void)player:(JDCloudPlayer *)player didReceiveUserCustomMessage:(NSString *)message;

@end

#endif /* JDCloudPlayerDelegate_h */
