//
//  JDCloudPlayerVideoErrorModel.h
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/*!
 @class JDCloudPlayerVideoErrorModel
 @abstract 京东云播放器SDK-错误码模型
 @discussion 本类包含了播放过程中错误码的一些模型参数
 
 errorCode ｜ errorMsg
 1｜未知错误
 100｜无法链接到服务器
 200｜不支持快进
 -1004｜读取错误
 -1007｜数据错误
 -1010｜不支持播放该视频类型
 -110｜播放超时
 -10000｜播放器发生错误
 -10501｜播放器鉴权失败
 -40001｜播放的文件不存在
 -40002｜视频描述信息有错误，导致不能播放
 -20003｜视频不允许访问
 -40004｜解码视频失败
 -40005｜解码音频失败
 -40006｜不支持的视频播放格式
 -40007｜视频解码发生错误
 -40008｜音频解码发生错误
 -40009｜视频播放地址无效
 -40010｜内存不足导致不能播放
 -40011｜网络请求超时
 -40012｜请求参数有问题
 -40013｜视频未授权，不能播放
 -40014｜视频禁止播放
 -40015｜未找到该视频
 -40016｜客户端请求错误
 -40017｜服务的端发生错误
 */
@interface JDCloudPlayerVideoErrorModel : NSObject

/*!
 @abstract   出错的错误信息
 */
@property (nonatomic, copy) NSString *errorMsg;

/*!
 @abstract   出错的错误码
 */
@property (nonatomic, assign) NSInteger errorCode;

/*!
 @abstract   出错的vid
 */
@property (nonatomic, copy) NSString *errorVid;

/*!
 @abstract   出错的url
 */
@property (nonatomic, copy) NSString *errorUrl;

/*!
 @abstract   出错的requestId
 */
@property (nonatomic, copy) NSString *errorRequestId;

@end

NS_ASSUME_NONNULL_END

