//
//  JDSTimeLineItem.h
//  JDSTimeLine
//
//  Created by jdmt on 2019/3/29.
//  Copyright © 2019 com.jd.avplayer. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "UIKit/UIKit.h"
#import <AVFoundation/AVFoundation.h>

NS_ASSUME_NONNULL_BEGIN

/*!
 @enum JDSTimeLineItemType
 @abstract  时间线Item类型
 */
typedef NS_ENUM(NSUInteger, JDSTimeLineItemType){
    /*! 未知*/
    JDSTimeLineItemType_Unknown     = 0,
    /*! Movie*/
    JDSTimeLineItemType_Movie       = 1,
    /*! BGM*/
    JDSTimeLineItemType_BGM         = 2,
    /*! 录音*/
    JDSTimeLineItemType_Voice       = 4,
    /*! 字幕贴纸*/
    JDSTimeLineItemType_Layer       = 8,
    /*! 视频Filter*/
    JDSTimeLineItemType_VideoFilter = 16,
};

/*!
 @@protocol JDSTimeLineItemProtocol
 @abstract 时间线Item接口协议
 */
@protocol JDSTimeLineItemProtocol <NSObject>

/*!
 @abstract 时间线Item类型
 @return 返回时间线Item类型
 */
- (JDSTimeLineItemType)type;

/*!
 @abstract 时间线Item时间范围
 @return 返回时间线Item时间范围
 */
- (CMTimeRange)timeRangeInWhole;

@end

/*!
 @class JDSTimeLineItem
 @abstract 基类，表示时间线上的一个片段，可以是Movie、BGM、录音、字幕、贴纸等
 @discussion 本类用于继承，表示时间线上的一个片段
 */
@interface JDSTimeLineItem : NSObject<JDSTimeLineItemProtocol>


@end

NS_ASSUME_NONNULL_END
