//
//JDCloudAVSDK.h
//

#import <Foundation/Foundation.h>

/*!
  @class JDCloudAVSDK
  @abstract 京东云直播SDK-全局类
  @discussion 本类包含了鉴权、日志等功能
*/
@interface JDCloudAVSDK : NSObject

/*!
 @abstract   License配置
 @param      license  license
 */
+(void)startWithLicense:(NSString*)license;

/*!
@abstract   配置sdk日志回调
@param      outputLogCallback  回调函数
 logItem 通用样例
 {
     p = OUT;
     time = 1600931800;
     track = "jdlive_closeStream";
     v = 1;
     xdata =     {
         "app_name" = "com.jdweiwei.appall";
         "dev_type" = "iPhone 6S Plus";
         "sys_name" = iOS;
         "sys_ver" = "13.6";
         "state" = 0;
         "unique_id" = "ADA7D73D-4FF1-4B7E-91AD-346EF2856758";
 
     };
 }
 日志公共字段
 track | NSString | 日志打点名称
 time  | NSNumber | 日志时间戳
 xdata ｜ NSDictionaryNS ｜ 日志数据字典
 app_name | NSString | 应用唯一ID
 dev_type | NSString | 设备类型
 sys_name | NSString | 系统名
 sys_ver  | NSString | 系统版本
 unique_id | NSString | 设备唯一ID
 
 日志业务字段
 track:jdlive_config
 videoCodec | NSNumber | 视频编码器 2是264硬编
 audioCodec | NSNumber | 音频编码器 0是aac
 streamVideoSize | NSString | 推流视频分辨率 w,h
 videoBitrate | NSNumber | 视频码率
 audioBitrate | NSNumber | 音频码率
 fps | NSNumber | 视频帧率
 audioSampleRate | NSNumber | 音频采样率
 
 track:jdlive_streamState
 state | NSNumber | 推流状态 参考JDLPushStreamState
 
 track:jdlive_streamError
 errorCode | NSNumber | 推流错误码 参考JDLStreamErrorCode
 
 track:jdlive_startStream
 url | NSString | 推流地址
 time | NSNumber | 当前时间戳
 
 track:jdlive_connectSuccess
 connect_time | NSNumber | 连接时长（包含dns解析时间）
 
 track:jdlive_dns
 dns_type | NSNumber | dns类型
 host_url | NSString | 推流地址
 stream_proto | NSString | 推流协议 rtmp
 dns_parse_time | NSNumber | dns解析时间
 host_ip | NSString | 推流节点ip
 
 track:jdlive_closeStream
 time | NSNumber | 当前时间戳
 
 track:jdlive_streamStatistics
 outTotalBytes | NSNumber | 推流上行网速
 currentVideoBitrate | NSNumber | 当前视频编码码率
 
 track:jdplayer_error
 errorCode | NSNumber | 播放错误码 参考JDCloudPlayerVideoErrorModel
 
 track:jdplayer_renderingStart
 dns_type | NSNumber | dns类型
 host_url | NSString | 播放地址
 host_ip | NSString | 播放节点ip
 dns_parse_time | NSNumber | dns解析时长
 prepare_time | NSNumber | 播放起播时长
 first_frame_time | NSNumber | 首帧显示时长
 video_size | NSString | 视频宽高
 video_bitrate | NSNumber | 视频码率
 video_fps | NSNumber | 视频帧率
 video_codec | NSString | 视频编码格式
 video_decoder | NSString | 视频解码器
 audio_sample_rate | NSNumber | 音频采样率
 audio_codec | NSNumber | 音频编码格式
 audio_bitrate | NSNumber | 音频码率
 
 track:jdplayer_streamStatistics
 tcpSpeed | NSNumber | 下载网速
 
*/
+(void)setOutputLogCallback:(void (^)(NSDictionary* logItem))outputLogCallback;


/*!
@abstract   配置sdk日志是否上报
@param      enable YES上报 NO不上报 默认NO
*/
+(void)setSDKLogSendEnable:(BOOL)enable;

@end
