//
//  JDCloudPlayerDefine.h
//

#import <CoreMedia/CoreMedia.h>

#ifndef JDCloudPlayerDefine_h
#define JDCloudPlayerDefine_h

/*!
   @abstract   视频原始数据的回调
   @param      sampleBuffer  采样数据
 */
typedef void (^JDCloudPlayerVideoRawDataBlock)(CMSampleBufferRef sampleBuffer);

/*!
   @abstract   音频原始数据的回调
   @param      sampleBuffer  采样数据
 */
typedef void (^JDCloudPlayerAudioRawDataBlock)(CMSampleBufferRef sampleBuffer);

/*!
   @abstract   音频原始数据的回调
   @param pData 回调输出的数据指针
   @param len sample数
   @param fmt 数据的格式
   @param pts 目前没有时间戳输出
 */
typedef void (^JDCloudPlayerAudioPcmDataBlock)(uint8_t** pData, int len, const AudioStreamBasicDescription* fmt, CMTime pts);

/*!
   @abstract   截取进度回调
   @param      frameTime  截取帧的总时长
   @param      frameIndex 截取第一帧的位置
 */
typedef void (^JDCloudProgressBlock)(float frameTime, int frameIndex);

/*!
   @abstract   截取小视频完成的回调
 */
typedef void (^JDCloudDownloadVideoFinishedBlock)(int ret);

/*!
   @abstract   获取小视频完成的回调
 */
typedef void (^JDCloudRecordVideoFinishedBlock)(int ret);

/*!
   @abstract   视频转GIF完成的回调
 */
typedef void (^JDCloudRecordGifFinishedBlock)(int ret);


/*!
   @enum JDCloudPlayerClaritType
   @abstract  视频清晰度
 */
typedef NS_ENUM(NSInteger, JDCloudPlayerClaritType) {
    /*! 流畅*/
    JDCloudMPMovieClaritTypeSD,
    /*! 标清*/
    JDCloudMPMovieClaritTypeCIF,
    /*! 高清*/
    JDCloudMPMovieClaritTypeHD,
};

/*!
   @enum JDCloudPlayerVideoType
   @abstract  视频列表类型
 */
typedef NS_ENUM(NSInteger, JDCloudPlayerVideoListType) {
    /*! 点播*/
    JDCloudPlayerVideoListTypeVOD,
    /*! 直播*/
    JDCloudPlayerVideoListTypeLive,
    /*! 本地*/
    JDCloudPlayerVideoListTypeLocal,
    /*! 扫描二维码*/
    JDCloudPlayerVideoListTypeScan,
};

/*!
 @enum JDCloudPlayerVideoFormat
 @abstract  视频格式
 */
typedef NS_ENUM(NSInteger, JDCloudPlayerVideoFormat) {
    /*! MP4*/
    JDCloudPlayerVideoFormatMP4,
    /*! Rtmp*/
    JDCloudPlayerVideoFormatRtmp,
    /*! M3u8*/
    JDCloudPlayerVideoFormatM3u8,
    /*! Flv*/
    JDCloudPlayerVideoFormatFlv,
    /*! 其他*/
    JDCloudPlayerVideoFormatOther,
};

/*!
   @enum JDCloudPlayerEvent
   @abstract  视频事件
 */
typedef NS_ENUM(NSUInteger, JDCloudPlayerEvent) {
    /*! 准备成功*/
    JDCloudPlayerEventPrepareDone = 0,
    /*! 开始播放*/
    JDCloudPlayerEventPlay,
    /*! 跳转后视频开始渲染*/
    JDCloudSeekVideoStart,
    /*! 跳转后音频开始播放*/
    JDCloudSeekAudioStart,
    /*! 视频宽高改变*/
    JDCloudPlayerEventNaturalSizeChanged,
    /*! 第一帧加载完成*/
    JDCloudPlayerEventFirstFrame,
    /*! 暂停播放*/
    JDCloudPlayerEventPause,
    /*! 停止播放*/
    JDCloudPlayerEventStop,
    /*! 播放已结束*/
    JDCloudPlayerEventFinish,
    /*! 开始加载视频数据*/
    JDCloudPlayerEventBeginLoading,
    /*! 视频资源加载完成*/
    JDCloudPlayerEventEndLoading,
    /*! 跳转成功*/
    JDCloudPlayerEventSeekDone,
    /*! 播放被中断*/
    JDCloudPlayerEventInterrupted,
    /*! 向前跳转*/
    JDCloudPlayerEventSeekingForward,
    /*! 向后跳转*/
    JDCloudPlayerEventSeekingBackward
};

/*!
   @enum JDCloudPlayerDisplayMode
   @abstract  浏览方式
 */
typedef NS_ENUM(NSUInteger, JDCloudPlayerDisplayMode) {
    /*! 保持原始比例*/
    JDCloudPlayerDisplayModeFit = 0,
    /*! 全屏占满屏幕*/
    JDCloudPlayerDisplayModeFitWithCropping
};

/*!
 @enum JDCloudPlayerState
 @abstract  视频状态
 */
typedef NS_ENUM(NSUInteger, JDCloudPlayerState) {
    /*! 播放*/
    JDCloudPlayerStatePlay,
    /*! 暂停*/
    JDCloudPlayerStatePause,
    /*! 停止*/
    JDCloudPlayerStateStop,
};

/*!
   @enum JDCloudScalingMode
   @abstract  缩放模式
 */
typedef NS_ENUM(NSInteger, JDCloudScalingMode) {
    /*! 不改变比例*/
    JDCloudScalingModeNone,
    /*! 填充 统一尺度直到宽高某一条件匹配*/
    JDCloudScalingModeAspectFit,
    /*! 剪裁 统一尺度直到，画面填充满可见的边界。宽或高的内容可能被剪切*/
    JDCloudScalingModeAspectFill,
    /*! 全屏 不统一尺度，宽高完全匹配设置的区域，画面可能被拉伸*/
    JDCloudScalingModeFill
};

/*!
   @enum JDCloudLoadState
   @abstract  播放下载状态
 */
typedef NS_OPTIONS(NSUInteger, JDCloudLoadState) {
    /*! 未知*/
    JDCloudLoadStateUnknown        = 0,
    /*! 可以播放*/
    JDCloudLoadStatePlayable       = 1 << 0,
    /*! 缓存完成 全部可以播放*/
    JDCloudLoadStatePlaythroughOK  = 1 << 1,
    /*! 不可以播放*/
    JDCloudLoadStateStalled        = 1 << 2,
};

/*!
   @enum JDCloudTimeOption
   @abstract 时间选择 根据时间选择，制定该时间的缩略图，用于预览
 */
typedef NS_ENUM(NSInteger, JDCloudTimeOption) {
    /*! 最近的关键帧*/
    JDCloudTimeOptionNearestKeyFrame,
    /*! 准确的关键帧*/
    JDCloudTimeOptionExact
};

/*!
   @enum JDCloudGifSize
   @abstract 视频旋转方向
 */
typedef NS_ENUM(NSInteger, JDCloudRenderRotate) {
    /*! 不旋转*/
    JDCloudRenderRotate0 = 0,
    /*! 旋转90度*/
    JDCloudRenderRotate90 = 90,
    /*! 旋转180度*/
    JDCloudRenderRotate180 = 180,
    /*! 旋转270度*/
    JDCloudRenderRotate270 = 270
};



#endif /* JDCloudPlayerDefine_h */
