//
//  JDLConfig.h
//  JDLive
//


#import <Foundation/Foundation.h>
#import "JDLConstants.h"

/*!
   @class JDLConfig
   @abstract 京东云直播SDK-推流配置接口类
   @discussion 本类包含了所有推流相关的初始化配置参数
 */
@interface JDLConfig : NSObject

/*!
   @abstract 初始化默认配置方法
 */
+ (instancetype) defaultConfig;

/*!
   @abstract  采集音视频的来源
   @discussion see JDLCaptureSourceType
 */
@property (assign, nonatomic) JDLCaptureSourceType captureSourceType;

/*!
   @abstract  采集分辨率
   @discussion see JDLPushResolution
 */
@property (assign, nonatomic) JDLPushResolution captureResolution;

/*!
   @abstract  推流分辨率
   @discussion see JDLPushResolution
*/
@property (assign, nonatomic) JDLPushResolution pushResolution;

/*!
   @abstract  视频直播场景
   @discussion see JDLVideoLiveScene
 */
@property (assign, nonatomic) JDLVideoLiveScene videoLiveScene;

/*!
   @abstract  性能模式
   @discussion see JDLVideoPerformanceMode
 */
@property (assign, nonatomic) JDLVideoPerformanceMode videoPerformanceMode;

/*!
   @abstract  帧率
 */
@property (assign, nonatomic) int fps;

/*!
   @abstract  最大帧率
 */
@property (assign, nonatomic) int fpsMax;

/*!
   @abstract  最小帧率
 */
@property (assign, nonatomic) int fpsMin;

/*!
   @abstract  码率, 单位kbps
 */
@property (assign, nonatomic) int videoBitrate;

/*!
   @abstract 最大码率, 单位kbps
 */
@property (assign, nonatomic) int videoBitrateMax;

/*!
   @abstract  最小码率, 单位kbps
 */
@property (assign, nonatomic) int videoBitrateMin;

/*!
   @abstract  视频旋转方向
 */
@property (assign, nonatomic) JDLDeviceOrientationMode deviceOrientationMode;

/*!
   @abstract  最大重连次数
 */
@property (assign, nonatomic) unsigned int maxReconnectCount;

/*!
   @abstract  重连时间间隔, 单位秒
 */
@property (assign, nonatomic) double reconnectInterval;

/*!
   @abstract  摄像头类型
   @discussion see JDLPushCameraType
 */
@property (assign, nonatomic) JDLPushCameraType cameraType;

/*!
   @abstract  视频编码模式
   @discussion see JDLPushVideoEncoderMode
 */
@property (assign, nonatomic) JDLPushVideoEncoderMode videoEncoderMode;

/*!
   @abstract  音频编码模式
   @discussion see JDLPushAudioEncoderMode
 */
@property (assign, nonatomic) JDLPushAudioEncoderMode audioEncoderMode;

/*!
   @abstract  音频码率
 */
@property (assign, nonatomic) int audioBitrate;

/*!
   @abstract  音频采样率
 */
@property (assign, nonatomic) double audioSampleRate;

/*!
   @abstract  单通道, 双通道
   @discussion see JDLPushAudioChannel
 */
@property (assign, nonatomic) JDLPushAudioChannel audioChannel;

/*!
   @abstract  关键帧间隔
 */
@property (assign, nonatomic) int keyIntervalTime;

/*!
   @abstract  预览是否镜像, 默认NO
 */
@property (assign, nonatomic) BOOL previewMirror;

/*!
   @abstract  推流是否镜像, 默认NO
 */
@property (assign, nonatomic) BOOL pushMirror;

/*!
   @abstract  只有音频, 默认NO
 */
@property (assign, nonatomic) BOOL audioOnly;

/*!
   @abstract  后台推流开关，默认YES
 */
@property (assign, nonatomic) BOOL isBackgroundPush;//未实现

/*!
  @abstract  外部获取SDK中的视频帧数据模式
  @discussion see JDLExternalVideoGetType
*/
@property (assign, nonatomic)JDLExternalVideoGetType externalVideoGetType;

/*!
  @abstract  外部获取SDK中的音频帧数据模式
  @discussion see JDLExternalAudioGetType
*/
@property (assign, nonatomic)JDLExternalAudioGetType externalAudioGetType;

@end



