//
//  JDLConstants.h
//  JDLive
//


#import <Foundation/Foundation.h>

/*!
 @abstract   每帧回调声明
 */
typedef void (^UpdateBlock)(id waterMark, NSTimeInterval timestamp, int frameCount);

/*!
  @enum JDLCaptureSourceType
  @abstract  采集音视频的来源类型
 */
typedef NS_ENUM(NSInteger, JDLCaptureSourceType) {
    /*! 摄像头*/
    JDLCaptureSourceType_Camera = 0,
    /*! 录屏*/
    JDLCaptureSourceType_ReplayKit,
};

/*!
   @enum JDLPushResolution
   @abstract  分辨率
 */
typedef NS_ENUM(NSInteger, JDLPushResolution){
    /*! 360*640*/
    JDLPushResolution360P = 0,
    /*! 540*960*/
    JDLPushResolution540P,
    /*! 720*1280*/
    JDLPushResolution720P,
    /*! 1080*1920*/
    JDLPushResolution1080P
};

/*!
   @enum JDLPushQualityMode
   @abstract  推流模式
 */
typedef NS_ENUM(NSInteger, JDLPushQualityMode){
    /*! 清晰度优先模式*/
    JDLPushQualityModeResolutionFirst = 0,
    /*! 流畅度优先模式*/
    JDLPushQualityModeFluencyFirst,
    /*! 自定义模式(固定码率、帧率)*/
    JDLPushQualityModeCustom,
};

/*!
   @enum JDLVideoPerformanceMode
   @abstract  性能模式
 */
typedef NS_ENUM(NSInteger, JDLVideoPerformanceMode){
    /*! 低*/
    JDLVideoPerformanceMode_Low= 0,
    /*! 平均*/
    JDLVideoPerformanceMode_Balance,
    /*! 高*/
    JDLVideoPerformanceMode_High,
};

/*!
   @enum JDLPushCameraType
   @abstract  摄像头方向
 */
typedef NS_ENUM(NSInteger, JDLPushCameraType){
    /*! 前置摄像头*/
    JDLPushCameraTypeFront = 0,
    /*! 后置摄像头*/
    JDLPushCameraTypeBack,
};

/*!
   @enum JDLPushScallingMode
   @abstract  缩放模式
 */
typedef NS_ENUM(NSInteger, JDLPushScallingMode){
    /*! 填充*/
    JDLPushScallingModeFit = 0,
    /*! 裁剪*/
    JDLPushScallingModeCrop,
};

/*!
   @enum JDLPushVideoEncoderMode
   @abstract  视频编码模式
 */
typedef NS_ENUM(NSInteger, JDLPushVideoEncoderMode){
    /*! 自动选择*/
    JDLPushVideoEncoderMode_AUTO = 0,
    /*! 软编X264*/
    JDLPushVideoEncoderMode_X264,
    /*! 软编265 暂时不支持*/
    JDLPushVideoEncoderMode_X265,
    /*! 硬编videotoolbox 264*/
    JDLPushVideoEncoderMode_VT264,
    /*! 硬编videotoolbox 265*/
    JDLPushVideoEncoderMode_VT265,
} ;

/*!
   @enum JDLPushAudioEncoderMode
   @abstract  音频编码模式
 */
typedef NS_ENUM(NSInteger, JDLPushAudioEncoderMode){
    /*! AAC_LC*/
    JDLPushAudioEncoderMode_AAC_LC = 0,
    /*! HE_AAC*/
    JDLPushAudioEncoderMode_HE_AAC,
    /*! HE_AAC_V2*/
    JDLPushAudioEncoderMode_HE_AAC_V2,
    /*! AAC_LD*/
    JDLPushAudioEncoderMode_AAC_LD,
};

/*!
   @enum JDLPushAudioChannel
   @abstract  声道数
 */
typedef NS_ENUM(NSInteger, JDLPushAudioChannel){
    /*! 单声道*/
    JDLPushAudioChannel_1 = 1,
    /*! 双声道*/
    JDLPushAudioChannel_2 = 2,
};

/*!
   @enum JDLAudioDeviceType
   @abstract  音频输入设备类型
 */
typedef NS_ENUM(NSInteger, JDLAudioDeviceType){
    /*! Built-in microphone on an iOS device*/
    JDLAudioDeviceType_Builtin = 0,
    /*! Microphone on a wired headset*/
    JDLAudioDeviceType_Beadset,
    /*! 蓝牙设备*/
    JDLAudioDeviceType_Bluetooth,
    /*! 未知设备*/
    JDLAudioDeviceType_Unknow = 100,
};

/*!
   @enum JDLRotationMode
   @abstract  旋转模式
 */
typedef NS_ENUM(NSInteger, JDLRotationMode){
    /*! 无旋转*/
    kJDLNoRotation,
    /*! 向左*/
    kJDLRotateLeft,
    /*! 向右*/
    kJDLRotateRight,
    /*! 竖直*/
    kJDLFlipVertical,
    /*! 水平*/
    kJDLFlipHorizonal,
    /*! 右垂直*/
    kJDLRotateRightFlipVertical,
    /*! 右水平*/
    kJDLRotateRightFlipHorizontal,
    /*! 180度*/
    kJDLRotate180
};

/*!
   @enum JDLBGMusicPlayerState
   @abstract  音频播放状态
 */
typedef NS_ENUM(NSInteger, JDLBGMusicPlayerState){
    /*! 初始状态*/
    JDLBGMusicPlayerState_Init,
    /*! 背景音正在播放*/
    JDLBGMusicPlayerState_Starting,
    /*! 背景音停止*/
    JDLBGMusicPlayerState_Stopped,
    /*! 背景音正在播放*/
    JDLBGMusicPlayerState_Playing,
    /*! 背景音暂停*/
    JDLBGMusicPlayerState_Paused,
    /*! 背景音播放出错*/
    JDLBGMusicPlayerState_Error,
    /*! 背景音被打断*/
    JDLBGMusicPlayerState_Interrupted
};

/*!
   @enum JDLPushStreamMode
   @abstract  推流模式
 */
typedef NS_ENUM(NSInteger, JDLPushStreamMode){
    /*! 正常推流*/
    JDLPushStreamMode_Normal = 0,
    /*! 背景图推流*/
    JDLPushStreamMode_Image_Audio,
    /*! 音频推流*/
    JDLPushStreamMode_Audio,
    /*! 视图推流*/
    JDLPushStreamMode_UIView
};

/*!
   @enum JDLPushStreamState
   @abstract  推流状态
 */
typedef NS_ENUM(NSInteger, JDLPushStreamState){
    /*! 初始化时状态为空闲*/
    JDLPushStreamState_Idle = 0,
    /*! 连接中*/
    JDLPushStreamState_Connecting,
    /*! 已连接*/
    JDLPushStreamState_Connected,
    /*! 断开连接中*/
    JDLPushStreamState_Disconnecting,
    /*! 推流出错*/
    JDLPushStreamState_Error,
    /*! 推流停止*/
    JDLPushStreamState_Stop,
};

/*!
   @enum JDLNetworkStateCode
   @abstract  网络状况事件码
 */
typedef NS_ENUM(NSInteger, JDLNetworkStateCode){
    /*! 正常无错误*/
    JDLNetworkStateCode_NONE = 0,
    /*! 发送包时间过长，( 单次发送超过 500毫秒 ）*/
    JDLNetworkStateCode_SEND_SLOW,
    /*! 估计带宽调整，上调*/
    JDLNetworkStateCode_BANDWIDTH_UP,
    /*! 估计带宽调整，下调*/
    JDLNetworkStateCode_BANDWIDTH_DOWN,
    /*! 帧率调整，上调*/
    JDLNetworkStateCode_FPS_UP,
    /*! 帧率调整，下调*/
    JDLNetworkStateCode_FPS_DOWN,
    /*! 鉴权失败*/
    JDLNetworkStateCode_AUTH_FAILED,
    /*! 输入音频不连续*/
    JDLNetworkStateCode_AUDIO_DISCONTINUOUS,
    /*! 网络变为不可用*/
    JDLNetworkStateCode_NET_UNREACHABLE,
    /*! 网络变为可用*/
    JDLNetworkStateCode_NET_REACHABLE,
};

/*!
   @enum JDLVideoLiveScene
   @abstract  视频直播场景
 */
typedef NS_ENUM(NSInteger, JDLVideoLiveScene){
    /*! 自动*/
    JDLVideoLiveScene_Scene0 = 0,
    /*! 美女直播*/
    JDLVideoLiveScene_Scene1,
    /*! 风景直播*/
    JDLVideoLiveScene_Scene2,
    /*! 运动直播*/
    JDLVideoLiveScene_Scene3,
} ;

/*!
 @enum JDLDeviceOrientationMode
 @abstract  屏幕模式
 */
typedef NS_ENUM(NSInteger, JDLDeviceOrientationMode){
    /*! 横屏*/
    JDLDeviceOrientationMode_H = 0,
    /*! 竖屏*/
    JDLDeviceOrientationMode_V,
    /*! 自适应*/
    JDLDeviceOrientationMode_AUTO,
} ;



/*!
   @enum JDLMediaRecordState
   @abstract  旁路录制状态
 */
typedef NS_ENUM(NSInteger, JDLMediaRecordState){
    /*! 初始状态*/
    JDLMediaRecordState_Idle,
    /*! 录像中*/
    JDLMediaRecordState_Recording,
    /*! 录像停止*/
    JDLMediaRecordState_Record_Stopped,
    /*! 录像失败*/
    JDLMediaRecordState_Record_Error,
};

/*!
   @enum JDLBeautyStyle
   @abstract  美颜风格
 */
typedef NS_ENUM(NSInteger, JDLBeautyStyle){
    /*! 不美颜*/
    JDLBeautyStyleNone = 0,
    /*! 简单美颜*/
    JDLBeautyStyleSimple,
};

/*!
  @enum JDLExternalVideoGetType
  @abstract  外部视频数据类型
*/
typedef NS_ENUM(NSInteger, JDLExternalVideoGetType){
    /*! 外部不获取SDK中的视频帧数据*/
    JDLExternalVideoGetType_OrigialNO = 0,
//    /*! 外部获取SDK中的SampleBuffer视频帧*/
//    JDLExternalVideoGetType_OrigialSampleBuffer = 1,
//    /*! 外部获取SDK中的Texture视频帧*/
//    JDLExternalVideoGetType_OrigialTexture = 2
};

/*!
  @enum JDLExternalAudioGetType
  @abstract  外部音频数据类型
*/
typedef NS_ENUM(NSInteger, JDLExternalAudioGetType){
    /*! 外部不获取SDK中的音频帧数据*/
    JDLExternalAudioGetType_OrigialNO = 0,
//    /*! 外部获取SDK中的SampleBuffer音频帧*/
//    JDLExternalAudioGetType_OrigialSampleBuffer = 1,
};

/*!
 @enum JDLStreamErrorCode
 @abstract  推流错误码
 */
typedef NS_ENUM(NSUInteger, JDLStreamErrorCode) {
    /*! 正常*/
    JDLStreamErrorCode_NONE = 0,
    /*! 推流失败*/
    JDLStreamErrorCode_ERROR,
    /*! SDK鉴权没有初始化*/
    JDLStreamErrorCode_AUTH_NO_INIT,
    /*! SDK鉴权参数不合法*/
    JDLStreamErrorCode_AUTH_PARAM_INVALID,
    /*! SDK被禁用*/
    JDLStreamErrorCode_AUTH_FORBIDDEN,
    /*! SDK鉴权权限受限*/
    JDLStreamErrorCode_AUTH_LIMITED,
    /*! SDK鉴权解析异常*/
    JDLStreamErrorCode_AUTH_FORMAT_INVALID,
    /*! 此流已存在*/
    JDLStreamErrorCode_ALREADY_EXIST_STREAM_NAME,
    /*! 签名不匹配*/
    JDLStreamErrorCode_SIGNATURE_NOT_MATCH,
    /*! refer鉴权失败*/
    JDLStreamErrorCode_REFER_NOT_MATCH,
    /*! 流被拒绝*/
    JDLStreamErrorCode_FORBIDDEN,
    /*! 无效的AccessKey*/
    JDLStreamErrorCode_INVALID_ACCESS_KEY,
    /*! 缺少access key*/
    JDLStreamErrorCode_MISS_ACCESS_KEY,
    /*! 推流失败 未知原因*/
    JDLStreamErrorCode_INTERNAL_ERROR,
    /*! DNS解析错误*/
    JDLStreamErrorCode_DNS_PARSE_FAILED
};

