//
//  JDLinkPK.h
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>
#import "JDLive.h"


@class JDLinkPK;

/*!
   @protocol JDLinkPKDelegate
   @abstract 京东云直播SDK-连麦代理方法类
   @discussion 本类包含了所有连麦相关的代理回调接口方法
 */
@protocol JDLinkPKDelegate <NSObject>
/*!
   @abstract   连麦PK回调代理
   @param      livePK  连麦对象
   @param      eventId  事件ID
   @param      param  事件参数字典
   @discussion see JDLinkPK
   eventId = 2000   连麦PK成功
 */
-(void)onJDLinkPK:(JDLinkPK*)livePK withEventId:(int)eventId withParam:(NSDictionary*)param;

/*!
   @abstract   连麦回调代理
   @param      livePK  连麦对象
   @param      errorCode  错误码
   @param      message  错误描述
   @discussion see JDLinkPK
   errorCode = -1000   本地推流失败
   errorCode = -1001   对端拉流失败
   errorCode = -1002   对端拉流中断
   errorCode = -1100   连麦PK超时 60秒超时
 */
-(void)onJDLinkPK:(JDLinkPK*)livePK withErrorCode:(int)errorCode withMessage:(NSString*)message;

@end

/*!
   @class JDLinkPKConfig
   @abstract 京东云直播SDK-连麦PK配置类
   @discussion 本类包含了所有推流连麦PK相关的接口方法，依赖之前JDLive
 */
@interface JDLinkPKConfig : NSObject

/*!
@abstract SDK服务环境
 @discussion see JDLinkPK
 0  开发环境
 1  灰度环境
 2  正式环境
*/
@property(nonatomic,assign)NSInteger envType;
/*!
@abstract  房间号，由业务方分配
*/
@property(nonatomic,strong)NSNumber *roomId;

/*!
@abstract 当前用户标识，由业务方分配
*/
@property(nonatomic,strong)NSString *userId;

/*!
@abstract  appId，用户在JRTC控制台申请JRTC SDK 接入时，appId由JRTC控制台生成
*/
@property(nonatomic,strong)NSString *appId;

/*!
@abstract 用户在JRTC控制台申请JRTC SDK 接入时，token由JRTC控制台生成
*/
@property(nonatomic,strong)NSString *token;

/*!
@abstract 令牌随机码，用户在JRTC控制台申请JRTC SDK 接入时，nonce由JRTC控制台生成
*/
@property(nonatomic,strong)NSString *nonce;

/*!
@abstract 令牌过期时间戳(毫秒)，由业务方生成
*/
@property(nonatomic,strong)NSString *timestamp;

@end


/*!
   @class JDLinkPK
   @abstract 京东云直播SDK-连麦PK接口类
   @discussion 本类包含了所有推流连麦PK相关的接口方法，依赖之前JDLive
 */
@interface JDLinkPK : NSObject

/*!
   @abstract  连麦PK回调代理
   @discussion see JDLinkPKDelegate
 */
@property(nonatomic, weak) id <JDLinkPKDelegate> delegate;

/*!
   @abstract   初始化连麦PK对象
   @param      live  推流对象
   @param      linkPKConfig 连麦PK配置参数 see JDLinkPKConfig
   @result     JDLinkPK实例
 */
-(id)initWithLive:(JDLive*)live JDLinkPKConfig:(JDLinkPKConfig*)linkPKConfig;

/*!
   @abstract   开始连麦PK
   @param      remoteUserId  远端用户Id
 */
- (void)startPK:(NSString*)remoteUserId;

/*!
   @abstract   结束连麦PK
 */
- (void)stopPK;

@end

