//
//  JDLiveDelegate.h
//  JDLive
//

#ifndef JDLiveDelegate_h
#define JDLiveDelegate_h

#import "JDLConstants.h"

/*!
   @protocol JDLiveDelegate
   @abstract 京东云直播SDK-推流代理方法类
   @discussion 本类包含了所有推流相关的代理回调接口方法，包括推流状态变化，错误回调，网络变化，录制状态变化，推流模式变化等功能
 */
@protocol JDLiveDelegate <NSObject>

@optional

/*!
   @abstract  推流状态发生改变
   @param pushStreamState 推流状态
   @discussion see JDLPushStreamState
 */
- (void)onPushStreamStateChange:(JDLPushStreamState)pushStreamState;

/*!
   @abstract  推流错误回调
   @param errorCode 错误码
   @discussion see JDLPushStreamErrorCode
 */
- (void)onPushStreamError:(JDLStreamErrorCode)errorCode;

/*!
   @abstract   网络健康状态发生改变
   @param networkStateCode 网络状态码
   @discussion see JDLNetworkStateCode
 */
- (void)onNetworkStatusChange:(JDLNetworkStateCode)networkStateCode;

/*!
   @abstract  视频录制状态改变
   @param mediaRecordState 录制状态码
   @discussion see JDLMediaRecordState
 */
- (void)onMediaRecordStateChange:(JDLMediaRecordState)mediaRecordState;

/*!
   @abstract  推流模式改变
   @param pushStreamMode 推流模式
   @discussion see JDLPushStreamMode
 */
- (void)onPushStreamModeChange:(JDLPushStreamMode)pushStreamMode;

@end


#endif /* JDLiveDelegate_h */
