//
//  JDMediaEditDelegate.h
//  GPUImage
//
//  Created by jdmt on 2019/4/8.
//  Copyright © 2019 com.jd. All rights reserved.
//

#import <AVFoundation/AVFoundation.h>
#import "JDMEDefines.h"
#import "JDSMovieTimeLineItem.h"
#import "JDSThumbnail.h"

NS_ASSUME_NONNULL_BEGIN

/*!
 @protocol JDMediaEditDelegate
 @abstract 京东云编辑SDK-媒体编辑代理方法类
 @discussion 本类包含了所有视频编辑相关的代理回调接口方法，包括错误回调，预览状态变化，预览进度变化，合成状态变化，合成进度变化等功能
 */
@protocol JDMediaEditDelegate <NSObject>

/*!
 @abstract  媒体编辑错误发生回调
 @param error 错误码
 @discussion see JDMEErrorCode
 */
- (void)onMediaEditError:(JDMEErrorCode)error;

/*!
 @abstract  媒体编辑预览状态变化回调
 @param state 预览状态
 @discussion see JDMEPreviewState
 */
- (void)onPreviewStateChange:(JDMEPreviewState)state;

/*!
 @abstract  媒体编辑预览进度变化回调
 @param time 当前预览时间点
 @discussion time CMTime
 */
- (void)onPreviewCurrentTimeChanged:(CMTime)time;

/*!
 @abstract  媒体编辑合成进度变化回调
 @param progress 当前进度
 */
- (void)onExportProgressChanged:(float)progress;

/*!
 @abstract  点击预览层回调
 @param recognizer 点击事件
 */
- (void)onTapPreviewPlayer:(UITapGestureRecognizer*)recognizer;

/*!
 @abstract  媒体编辑合成完成回调
 */
- (void)onExportFinish;

/*!
 @abstract  媒体编辑合成取消
 */
- (void)onExportCancel;


@optional

/*!
 @abstract  时间线变化回调
 */
- (void)onTimeLineDidChange;
/*!
 @abstract  背景音时间线变化回调
 */
- (void)onBgmTimeLineDidChanged;
/*!
 @abstract  水印字幕时间线变化回调
 */
- (void)onFilterLayerTimeLineDidChanged;

/*!
 @abstract  操作开始
 */
- (void)onMediaEditOperationBegin;
/*!
 @abstract  操作结束
 */
- (void)onMediaEditOperationEnd;

@end

NS_ASSUME_NONNULL_END
