//
//  JDSMovieTimeLineItem.h
//  jdcloud_shortvideo_ios
//
//  Created by 杜治全 on 2019/5/13.
//  Copyright © 2019 com.jdcloud. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>
#import "JDSTimeLineItem.h"
#import "JDSThumbnail.h"
#import <Photos/Photos.h>

NS_ASSUME_NONNULL_BEGIN

@class JDSVideoFilterTimeLineItem;
@class JDSAudioFilterTimeLineItem;
@class JDSMovieTimeLineItemInternal;
@class JDSImageTimeLineItemInternal;

/*!
 @class JDSMovieTimeLineItem
 @abstract 表示时间线上的一个Movie片段
 @discussion 时间线上一个Movie片段的各种属性设置
 */
@interface JDSMovieTimeLineItem : JDSTimeLineItem
{
    @package JDSMovieTimeLineItemInternal *_movieItemInternal;
    @package JDSImageTimeLineItemInternal *_imageItemInternal;
}

/*!
 @abstract Movie的类型
 */
- (JDSTimeLineItemType)type;

/*!
 @abstract 是否是图片
 */
- (BOOL)isImage;

/*!
 @abstract Movie的倍速
 */
@property (nonatomic, assign, readonly) double rate;

/*!
 @abstract Movie的音量
 */
@property (nonatomic, assign, readonly) double volume;

/*!
 @abstract Movie是否静音
 */
@property (nonatomic, assign, readonly) BOOL mute;

/*!
 @abstract Movie拍摄时的旋转模式，只跟原始视频有关GPUImageRotationMode
 */
@property (nonatomic, assign, readonly) int originRotationMode;

/*!
 @abstract Movie的旋转模式GPUImageRotationMode
 */
@property (nonatomic, assign, readonly) int rotationMode;

/*!
 @abstract Asset的时长(变速后)
 */
@property (nonatomic, assign, readonly) CMTime assetDuration;

/*!
 @abstract Movie在原始数据中的时间范围(变速后)
 */
@property (nonatomic, assign, readonly) CMTimeRange timeRangeInAsset;

/*!
 @abstract Movie在时间线上的时间范围
 */
@property (nonatomic, assign, readonly) CMTimeRange timeRangeInWhole;

///*!
// @abstract Movie的trackId
// */
//@property (nonatomic, assign, readonly) int trackId;

/*!
 @abstract Movie的滤镜名
 */
@property (nonatomic, copy, readonly) NSString *filternName;

/*!
 @abstract Movie的音频Filter
 */
@property (nonatomic, strong, readonly) NSArray<JDSAudioFilterTimeLineItem*> *audioFilters;

/*!
 @abstract 初始化
 @param   asset   AVURLAsset资源
 @result     JDSMovieTimeLineItem对象
 */
- (instancetype)initWithAsset:(AVURLAsset*)asset;

/*!
 @abstract 初始化
 @param   phAsset   PHAsset资源
 @result     JDSMovieTimeLineItem对象
 */
- (instancetype)initWithPHAsset:(PHAsset*)phAsset;

/*!
 @abstract 初始化
 @param   image   UIImage对象
 @result     JDSMovieTimeLineItem对象
 */
- (instancetype)initWithImage:(UIImage*)image;

/*!
 @abstract 深拷贝
 */
- (instancetype)clone;

/*!
 @abstract 获取缩略图结构
 @param   time   截图间隔
 @result     返回JDSThumbnail对象数组
 */
- (NSArray<JDSThumbnail*> *)getThumbnailArrayWithTimeSpan:(CMTime)time;

/*!
 @abstract 获取相应的缩略图
 @param   time   时间点
 @param   completion   完成回调
 */
- (void)getThumbnailImageAtTime:(CMTime)time completion:(void(^)(UIImage*))completion;

@end

NS_ASSUME_NONNULL_END
